@@ -2,6 +2,13 @@
 #define ICQEVENT_H
 
 #include "pthread_rdwr.h"
+#include "licq_constants.h"
+#include "licq_message.h"
+
+#include <stdlib.h>
+#include <vector>
+
+using std::vector;
 
 class CPacket;
 class CICQDaemon;
@@ -42,8 +49,11 @@ friend class CICQDaemon;
 
 //-----CSearchAck------------------------------------------------------------
 // Values for the Status() field of the SearchAck
+//! User is offline.
 const unsigned short  SA_OFFLINE = 0;
+//! User is online.
 const unsigned short  SA_ONLINE = 1;
+//! User has disabled online awareness.
 const unsigned short  SA_DISABLED = 2;
 
 /*! \brief The response to a search request.
@@ -67,12 +77,10 @@ public:
   const char *LastName()   { return m_szLastName; }
   //! Returns the e-mail address of the search result.
   const char *Email()      { return m_szEmail; }
-#ifdef PROTOCOL_PLUGIN
   //! Retunrs the Id string
   const char *Id()         { return m_szId; }
   //! Returns the protocol plugin id
   unsigned long PPID()     { return m_nPPID; }
-#endif
   //! If non-zero, the number of search results that were found that could not
   //! be displayed.  The server has a 40 user limit on search results.  This
   //! is valid when Result() is EVENT_SUCCESS.
@@ -90,12 +98,10 @@ public:
 
 protected:
   CSearchAck(unsigned long _nUin);
-#ifdef PROTOCOL_PLUGIN
   CSearchAck(const char *_szId, unsigned long _nPPID);
 
   unsigned long m_nPPID;
   char *m_szId;
-#endif
   unsigned long m_nUin;
   char *m_szAlias;
   char *m_szFirstName;
@@ -123,11 +129,20 @@ enum ConnectType
 
 enum EventResult
 {
+  //! The event has been acked by the reciepient. Most events will
+  //! return this.
   EVENT_ACKED,
+  //! The event was succcessfully sent.  This is used by extended
+  //! events.  Such as searching for users or updating user info.
   EVENT_SUCCESS,
+  //! The event failed for some reason.  This should rarely happen.
   EVENT_FAILED,
+  //! The event timed out while communicating with the remote socket.
   EVENT_TIMEDOUT,
+  //! The event had some kind of error at some point.
   EVENT_ERROR,
+  //! The event was concelled by the user and was not sent.
+  //! A call to icqCancelEvent was made.
   EVENT_CANCELLED
 };
 
@@ -143,14 +158,7 @@ class ICQEvent
 public:
   // Accessors
 
-  //!This is the result of the event, can be EVENT_ACKED (this is what most
-  //!events will return on success), EVENT_SUCCESS (this is returned when
-  //!the command is extended such as searches or info updates and has been
-  //!completed successfully), EVENT_FAILED (same, but returned when the
-  //!event failed for some reason, should rarely happen), EVENT_TIMEDOUT
-  //!(returned if the event timedout trying to talk to the server),
-  //!EVENT_ERROR (if an error occured at some point), or EVENT_CANCELLED
-  //!(the event was cancelled by a call to icqCancelEvent).
+  //!This is the result of the event.
   EventResult Result()         { return m_eResult; }
 
   //!This will be either ICQ_TCPxACK_ACCEPT if the event was accepted by
@@ -183,7 +191,7 @@ public:
 
   //!This is used to identify events internally, but is necessary for
   //!accepting/rejecting chat or file requests.
-  unsigned long Sequence()     { return m_nSequence; }
+  unsigned short Sequence()     { return m_nSequence; }
 
   //!This is used to identify events internally, is are necessary for
   //!accepting/rejecting chat or file requests.
@@ -193,10 +201,14 @@ public:
   //!this was a message/url...
   unsigned long Uin()          { return m_nDestinationUin; }
 
-#ifdef PROTOCOL_PLUGIN
+  //!The user id that the event was destined for.  Only relevant if
+  //!this was a message/url...
   char *Id()                   { return m_szId; }
+  
+  //!The protocol id of the user that the event was destined for.
+  //!Only relevant if this was a message/url...
   unsigned long PPID()         { return m_nPPID; }
-#endif
+
   //!Special structure containing information relevant if this is a
   //!search event.
   CSearchAck *SearchAck()      { return m_pSearchAck; }
@@ -230,10 +242,8 @@ public:
 protected:
   ICQEvent(CICQDaemon *_xDaemon, int _nSocketDesc, CPacket *p, ConnectType _eConnect,
            unsigned long _nUin, CUserEvent *e);
-#ifdef PROTOCOL_PLUGIN
   ICQEvent(CICQDaemon *_xDaemon, int _nSocketDesc, CPacket *p, ConnectType _eConnect,
            const char *_szId, unsigned long _nPPID, CUserEvent *e);
-#endif
   ICQEvent(ICQEvent *);
 
   // Daemon only
@@ -241,8 +251,8 @@ protected:
   unsigned short ExtraInfo()   { return m_nExtraInfo; }
 
   // Compare this event to another one
-  bool CompareEvent(int, unsigned long) const;
-  bool CompareEvent(unsigned long) const;
+  bool CompareEvent(int, unsigned short) const;
+  bool CompareEvent(unsigned short) const;
   bool CompareSubSequence(unsigned long) const;
   unsigned long EventId() const;
 
@@ -257,15 +267,13 @@ protected:
   unsigned short m_nCommand;
   unsigned short m_nSubCommand;
   unsigned long  m_nDestinationUin;
-  unsigned long  m_nSequence;
+  unsigned short m_nSequence;
   unsigned short m_nSubSequence;
   unsigned short m_nSubType;
   unsigned short m_nExtraInfo;
   int            m_nSocketDesc;
-#ifdef PROTOCOL_PLUGIN
   char           *m_szId;
   unsigned long  m_nPPID;
-#endif
   CPacket        *m_pPacket;
   pthread_t      thread_send;
   bool           thread_running;
@@ -282,21 +290,15 @@ protected:
   static unsigned long s_nNextEventId;
 
 friend class CICQDaemon;
+friend class CMSN;
 friend void *ProcessRunningEvent_Client_tep(void *p);
 friend void *ProcessRunningEvent_Server_tep(void *p);
 friend void *MonitorSockets_tep(void *p);
 };
 
 //=====CICQSignal============================================================
-/*---------------------------------------------------------------------------
- * CICQSignal
- *
- * This class controls all asynchronous plugin notifications.  When a plugin
- * registers with the Licq daemon it informs the daemon of what signals it
- * is interested in.  From then on, at any time it may receive a signal
- * from the following list.  Each signal contains the signal type, an
- * optional sub-type, uin and signal specific argument.
- *
+
+/*
  * SIGNAL_UPDATExLIST -
  *   Indicates that the user list has changed in some way.  The sub-type
  *   will be one of the following.  In all cases the argument is 0.
@@ -317,51 +319,71 @@ friend void *MonitorSockets_tep(void *p);
  *    USER_EVENTS - Indicates that the user has received or deleted an
  *    event (message, url...).  The argument will be the event id of the
  *    added event, or the negative of the event id if an event was deleted.
+ *    The parameter will be the socket that the message was sent over.
  *    If the user checked our auto-response then the argument will be 0.
  *    USER_SECURITY - The users security status changed.  If the arg is
  *    1 the user is now secure, if it's 0, the user is no longer secure.
- *
- *  SIGNAL_LOGON - Indicates that we have successfully logged on to the
- *  icq network.  UIN and argument are both 0, as is the sub-type.
+ *    USER_TYPING - Indicates the typing status of the user has changed.
  *
  *  SIGNAL_LOGOFF - Indicates that we logged off.  All parameters are 0.
- *	LOGOFF_REQUESTED - The user requeted to logoff and the action succeed.
- *	LOGOFF_RATE  - loging off: login failed: rate limit exceeded
- *	LOGOFF_PASSWORD - loging off: login failed: Invalid UIN and password
- *      combination.
+
  *
  *  SIGNAL_ONEVENT - FIXME: MISSING DESCRIPTION
- *
- *  SIGNAL_UI_VIEWEVENT - The UIN is that of the user we want to view the
- *  oldest event of or 0 if we just want to view the oldest pending event.
- *  All other parameters are 0.
- *
- *  SIGNAL_UI_MESSAGE - The UIN is that of the user we want to send a
- *  message. All other parameters are 0.
- *
- *  SIGNAL_ADDxSERVERxLIST - The UIN has been successfully added to the server
- *  list.  The server side alias is just the UIN as a string.  The plugin
- *  may call icqRenameUser after receiving this.  Sub-type is 0.
- *
- *  SIGNAL_NEWxPROTO_PLUGIN - A new protocol plugin has been succesfully loaded.
- *  The sub-type is the id of the plugin.  This is used as a parameter for all
- *  functions dealing with this new protocol.  The UIN and all other parameters
- *  are 0.
  *-------------------------------------------------------------------------*/
 const unsigned long SIGNAL_UPDATExLIST           = 0x00000001;
 const unsigned long SIGNAL_UPDATExUSER           = 0x00000002;
+//! Indicates that we have successfully logged on to the specified
+//! network.  Argument is the Protocol ID.  All other arguments are
+//! 0.
 const unsigned long SIGNAL_LOGON                 = 0x00000004;
+//! Indicates that we have logged off.  All parameters are 0.
 const unsigned long SIGNAL_LOGOFF                = 0x00000008;
 const unsigned long SIGNAL_ONEVENT               = 0x00000010;
+//! The UIN is that of the user we want to view the oldest event of
+//! or 0 if we just want to view the oldest pending event. All other
+//! parameters are 0.
 const unsigned long SIGNAL_UI_VIEWEVENT          = 0x00000020;
+//! The UIN is that of the user we want to send a message.  All other
+//! parameters are 0.
 const unsigned long SIGNAL_UI_MESSAGE            = 0x00000040;
+//! The UIN has been successfully added to the server list.  The
+//! server side alias is set to the UIN.  The plugin may call 
+//! icqRenameUser after receiving this.  Sub-type is 0.
 const unsigned long SIGNAL_ADDxSERVERxLIST       = 0x00000080;
+//! Indicates new protocol plugin has been successfully loaded.
+//! The sub-type is the id of the plugin.  This is used as a parameter
+//! for all functions dealing with this new protocol.  The UIN and all
+//! other parameters are 0.
 const unsigned long SIGNAL_NEWxPROTO_PLUGIN      = 0x00000100;
+//! Returns the event tag of an event from a protocol plugin.
+//! The UI plugin will use this event tag to keep track of 
+//! all sent events.
+const unsigned long SIGNAL_EVENTxID              = 0x00000200;
+//! A user has joined a conversation. m_nArgument is the socket
+//! of the conversation that the user has joined.
+const unsigned long SIGNAL_CONVOxJOIN            = 0x00000400;
+//! A user has left a conversation. m_nArgument is the socket
+//! of the conversation that the user has left.
+const unsigned long SIGNAL_CONVOxLEAVE           = 0x00000800;
+//! A socket has been set for this user now.
+const unsigned long SIGNAL_SOCKET                = 0x00010000;
+//! A verification image is available for the gui to use
+//! that is located at BASE_DIR + "verify.jpg"
+const unsigned long SIGNAL_VERIFY_IMAGE          = 0x00020000;
+//! A new user has been successfully registered.
+const unsigned long SIGNAL_NEW_OWNER             = 0x00040000;
+//! Used by a UI plugin to tell the daemon that all signals should be
+//! sent to the UI plugin.
 const unsigned long SIGNAL_ALL                   = 0xFFFFFFFF;
 
 // logoff constants
+//! The user requested to logoff and the action succeeded.
 const unsigned long LOGOFF_REQUESTED             = 0x00000000;
+//! The user requested to logon but failed.  The rate limit of the
+//! server has been exceeded.
 const unsigned long LOGOFF_RATE                  = 0x00000001;
+//! The user requested to logon but failed.  The username and password
+//! combination is invalid.
 const unsigned long LOGOFF_PASSWORD              = 0x00000002;
 
 // User information update constants
@@ -374,6 +396,11 @@ const unsigned long USER_MORE                   = 6;
 const unsigned long USER_WORK                   = 7;
 const unsigned long USER_ABOUT                  = 8;
 const unsigned long USER_SECURITY               = 9;
+const unsigned long USER_MORE2                  = 10;
+const unsigned long USER_HP                     = 11;
+const unsigned long USER_PHONExBOOK             = 12;
+const unsigned long USER_PICTURE                = 13;
+const unsigned long USER_TYPING                 = 14;
 
 const unsigned long LIST_ADD                     = 1;
 const unsigned long LIST_REMOVE                  = 2;
@@ -390,11 +417,10 @@ const unsigned long LIST_ALL                     = 3;
 class CICQSignal
 {
 public:
-  CICQSignal(unsigned long _nSignal, unsigned long _nSubSignal, unsigned long _nUin, int nArgument = 0, char *nParameters = 0);
-#ifdef PROTOCOL_PLUGIN
+  CICQSignal(unsigned long _nSignal, unsigned long _nSubSignal, unsigned long _nUin,
+             int nArg = 0, unsigned long nCID = 0);
   CICQSignal(unsigned long _nSignal, unsigned long _nSubSignal, const char *_szId,
-             unsigned long _nPPID, int nArgument = 0, char *nParameters = 0);
-#endif
+             unsigned long _nPPID, int nArg = 0, unsigned long nCID = 0);
   CICQSignal(CICQSignal *s);
   ~CICQSignal();
 
@@ -404,47 +430,107 @@ public:
   unsigned long SubSignal() { return m_nSubSignal; }
   //!UIN that the signal is related.  See signals to understand how this
   //!value is set.
-  unsigned long Uin() { return m_nUin; }
-#ifdef PROTOCOL_PLUGIN
+  unsigned long Uin()   { return m_nUin; }
   char *Id()            { return m_szId; }
   unsigned long PPID()  { return m_nPPID; }
-#endif
-  int Argument() { return m_nArgument; }
-  char *Parameters() { return m_szParameters; }
+  int Argument()        { return m_nArgument; }
+  unsigned long CID()   { return m_nCID; }
 protected:
   unsigned long m_nSignal;
   unsigned long m_nSubSignal;
   unsigned long m_nUin;
-#ifdef PROTOCOL_PLUGIN
   char *m_szId;
   unsigned long m_nPPID;
-#endif
   int m_nArgument;
-  char * m_szParameters;
+  unsigned long m_nCID;
 };
 
-
-#ifdef PROTOCOL_PLUGIN
+//! Signals that can be sent to protocol plugins.
 enum SIGNAL_TYPE
 {
+  //! The user requested this protocol to logon.
   PROTOxLOGON = 1,
+  //! The user requested this protocol to log off.
   PROTOxLOGOFF,
+  //! The user requested this protocol to change status.
   PROTOxCHANGE_STATUS,
+  //! The user requested this protocol to add a new user.
   PROTOxADD_USER,
-  PROTOxREM_USER
+  //! The user requested this protocol to remove a user.
+  PROTOxREM_USER,
+  //! The user requested this protocol to rename the user on the server's list.
+  PROTOxRENAME_USER,
+  //! The user requested this protocol to send a message.
+  PROTOxSENDxMSG,
+  //! The user has started typing and wants to let the remote user know about
+  //! about it.
+  PROTOxSENDxTYPING_NOTIFICATION,
+  //! The user granted authorization for a user to add them to his
+  //! contact list.
+  PROTOxSENDxGRANTxAUTH,
+  //! The user refused authorization for a user to add them to his
+  //! contact list.
+  PROTOxSENDxREFUSExAUTH,
+  //! The user has requested the user's profile/information.
+  PROTOxREQUESTxINFO,
+  //! The user has requested to update the owner's profile/information.
+  PROTOxUPDATExINFO,
+  //! The user has requested this user be added to the Invisible/Block list.
+  PROTOxBLOCKxUSER,
+  //! The user has requested this user be removed from the Invisible/Block
+  //! list,
+  PROTOxUNBLOCKxUSER,
+  //! The user has requested this user be added to the Visible/Accept list.
+  PROTOxACCEPTxUSER,
+  //! The user has requested this user be removed fromt he Visible/Accept
+  //! list.
+  PROTOxUNACCEPTxUSER,
+  //! The user has requsted this user be put on the Ignore list.
+  PROTOxIGNORExUSER,
+  //! The user has requested this user be removed fromt he Ignore list.
+  PROTOxUNIGNORExUSER,
+  //! The user has requested to send a file to this user.
+  PROTOxSENDxFILE,
+  //! The user has requested to send a chat invitation to this user.
+  PROTOxSENDxCHAT,
+  //! The user is cancelling an event (chat, secure, or file for ICQ)
+  PROTOxCANCELxEVENT,
+  //! The user has requested to send an accept/refuse reply to a file/chat
+  //! request
+  PROTOxSENDxEVENTxREPLY,
+  //! The user has opened a chat window with this user
+  PROTOxOPENEDxWINDOW,
+  //! The user has closed a chat window with this user
+  PROTOxCLOSEDxWINDOW,
+  //! The user has requested an SSL secure channel with this user
+  PROTOxOPENxSECURE,
+  //! The user has requested to close the SSL secure channel with this user
+  PROTOxCLOSExSECURE
 };
 
+//! The class that gets passed to protocol plugins when a signal
+//! is sent.
 class CSignal
 {
 public:
-  CSignal(SIGNAL_TYPE, const char *);
+  CSignal(SIGNAL_TYPE, const char *, unsigned long nCID = 0);
+  CSignal(CSignal *);
   virtual ~CSignal();
+
+  //! The signal is being sent to the plugin.
   SIGNAL_TYPE Type()  { return m_eType; }
+  //! The user id that this signal is being used for.
   char *Id()          { return m_szId; }
+  //! The conversation id to use (gets the socket).
+  unsigned long CID() { return m_nCID; }
+  //! The calling thread.
+  pthread_t Thread() { return thread_plugin; }
 
 private:
   char  *m_szId;
   SIGNAL_TYPE m_eType;
+  unsigned long m_nCID;
+  pthread_t thread_plugin;
 };
 
 class CLogonSignal : public CSignal
@@ -452,6 +538,7 @@ class CLogonSignal : public CSignal
 public:
   CLogonSignal(unsigned long);
   virtual ~CLogonSignal() { }
+  //! The requested initial status.
   unsigned long LogonStatus() { return m_nLogonStatus; }
 
 private:
@@ -468,6 +555,7 @@ class CChangeStatusSignal : public CSignal
 {
 public:
   CChangeStatusSignal(unsigned long);
+  //! The requested status.
   unsigned long Status()  { return m_nStatus; }
 
 private:
@@ -479,6 +567,7 @@ class CAddUserSignal : public CSignal
 public:
   CAddUserSignal(const char *, bool);
   virtual ~CAddUserSignal() { }
+  //! True if authorization is required to add this user.
   bool AuthRequired() { return m_bAuthRequired; }
 
 private:
@@ -491,6 +580,230 @@ public:
   CRemoveUserSignal(const char *);
 };
 
-#endif
+class CRenameUserSignal : public CSignal
+{
+public:
+  CRenameUserSignal(const char *);
+};
+
+/*! \brief Signal to a protocol plugin to send a messasge
+
+    This is sent when ProtoSendMessage has been called. It
+    contains the information necessary for a message to be
+    sent by a protocol plugin
+*/
+class CSendMessageSignal : public CSignal
+{
+public:
+  CSendMessageSignal(const char *szId, const char *szMsg, unsigned long nCID = 0);
+  virtual ~CSendMessageSignal() { if (m_szMsg) free(m_szMsg); }
+  //! The message to be sent
+  char *Message() { return m_szMsg; }
+  
+private:
+  char *m_szMsg;
+};
+
+class CTypingNotificationSignal : public CSignal
+{
+public:
+  CTypingNotificationSignal(const char *szId, bool bActive, unsigned long nCID = 0);
+  bool Active() { return m_bActive; }
+private:
+  bool m_bActive;
+};
+
+class CGrantAuthSignal : public CSignal
+{
+public:
+  CGrantAuthSignal(const char *, const char *);
+  virtual ~CGrantAuthSignal() { if (m_szMsg) free(m_szMsg); }
+  char *Message() { return m_szMsg; }
+
+private:
+  char *m_szMsg;
+};
+
+class CRefuseAuthSignal : public CSignal
+{
+public:
+  CRefuseAuthSignal(const char *, const char *);
+  virtual ~CRefuseAuthSignal() { if (m_szMsg) free(m_szMsg); }
+
+private:
+  char *m_szMsg;
+};
+
+class CRequestInfo : public CSignal
+{
+public:
+  CRequestInfo(const char *);
+};
+
+class CUpdateInfoSignal : public CSignal
+{
+public:
+  CUpdateInfoSignal(const char *szAlias, const char *szFirstName,
+    const char *szLastName, const char *szEmail,
+    const char *szCity,
+    const char *szState, const char *szPhoneNumber,
+    const char *szFaxNumber, const char *szAddress,
+    const char *szCellularNumber, const char *szZipCode);
+  virtual ~CUpdateInfoSignal();
+
+    char *Alias()        { return m_szAlias; }
+    char *FirstName()    { return m_szFirstName; }
+    char *LastName()     { return m_szLastName; }
+    char *Email()        { return m_szEmail; }
+    char *City()         { return m_szCity; }
+    char *State()        { return m_szState; }
+    char *PhoneNumber()  { return m_szPhoneNumber; }
+    char *FaxNumber()    { return m_szFaxNumber; }
+    char *Address()      { return m_szAddress; }
+    char *CellNumber()   { return m_szCellNumber; }
+    char *ZipCode()      { return m_szZipCode; }
+    
+private:
+  char *m_szAlias,
+       *m_szFirstName,
+       *m_szLastName,
+       *m_szEmail,
+       *m_szCity,
+       *m_szState,
+       *m_szPhoneNumber,
+       *m_szFaxNumber,
+       *m_szAddress,
+       *m_szCellNumber,
+       *m_szZipCode;
+};
+
+class CBlockUserSignal : public CSignal
+{
+public:
+  CBlockUserSignal(const char *);
+};
+
+class CUnblockUserSignal : public CSignal
+{
+public:
+  CUnblockUserSignal(const char *);
+};
+
+class CAcceptUserSignal : public CSignal
+{
+public:
+  CAcceptUserSignal(const char *);
+};
+
+class CUnacceptUserSignal : public CSignal
+{
+public:
+  CUnacceptUserSignal(const char *);
+};
+
+class CIgnoreUserSignal : public CSignal
+{
+public:
+  CIgnoreUserSignal(const char *);
+};
+
+class CUnignoreUserSignal : public CSignal
+{
+public:
+  CUnignoreUserSignal(const char *);
+};
+
+class CSendFileSignal : public CSignal
+{
+public:
+  CSendFileSignal(const char *_szUser, const char *_szFile,
+                  const char *_szMessage, ConstFileList &_lFileList);
+  virtual ~CSendFileSignal();
+
+  char *GetFileName() { return m_szFile; }
+  char *GetMessage()  { return m_szMessage; }
+  ConstFileList GetFileList() { return m_lFileList; }
+  
+private:
+  char *m_szFile,
+       *m_szMessage;
+  ConstFileList m_lFileList;
+};
+
+class CSendChatSignal : public CSignal
+{
+public:
+  CSendChatSignal(const char *_szUser, const char *_szMessage);
+  virtual ~CSendChatSignal() { if (m_szMessage) free(m_szMessage); }
+
+  char *GetMessage() { return m_szMessage; }
+
+private:
+  char *m_szMessage;
+};
+
+class CCancelEventSignal : public CSignal
+{
+public:
+  CCancelEventSignal(const char *_szUser, unsigned long _nFlag);
+
+  unsigned long GetFlag() { return m_nFlag; }
+    
+private:
+  unsigned long m_nFlag;
+};
+
+class CSendEventReplySignal : public CSignal
+{
+public:
+  CSendEventReplySignal(const char *_szUser, const char *_szMessage,
+                        bool _bAccepted, unsigned short nPort,
+                        unsigned long nSequence = 0,
+                        unsigned long nFlag1 = 0, unsigned long nFlag2 = 0,
+                        bool bDirect = false);
+  virtual ~CSendEventReplySignal() { if (m_szMessage) free(m_szMessage); }
+
+  char *GetMessage() { return m_szMessage; }
+  bool GetAccept()   { return m_bAccept; }
+  bool GetDirect()   { return m_bDirect; }
+  unsigned short GetPort()    { return m_nPort; }
+  unsigned long GetSequence() { return m_nSequence; }
+  unsigned long GetFlag()     { return m_nFlag; }
+  unsigned long GetFlag2()    { return m_nFlag2; } 
+  
+private:
+  char *m_szMessage;
+  bool m_bAccept,
+       m_bDirect;
+  unsigned short m_nPort;
+  unsigned long m_nSequence,
+                m_nFlag,
+                m_nFlag2;
+};
+
+class COpenedWindowSignal : public CSignal
+{
+public:
+  COpenedWindowSignal(const char *);
+};
+
+class CClosedWindowSignal : public CSignal
+{
+public:
+  CClosedWindowSignal(const char *);
+};
+
+class COpenSecureSignal : public CSignal
+{
+public:
+  COpenSecureSignal(const char *);
+};
+
+class CCloseSecureSignal : public CSignal
+{
+public:
+  CCloseSecureSignal(const char *);
+};
 
 #endif
+
