@@ -20,13 +20,15 @@ public:
 protected:
   CIniFile mKeysIni;
 #ifdef HAVE_LIBGPGME
-  GpgmeCtx mCtx;
+  gpgme_ctx_t mCtx;
 #endif
   char *mGPGPassphrase;
 
   void gpgmeLock();
   void gpgmeUnlock();
-  static const char *PassphraseCallback(void *, const char *, void **);
+#ifdef HAVE_LIBGPGME
+  static gpgme_error_t PassphraseCallback(void *, const char *, const char*, int, int);
+#endif
 };
 
 class CGPGMEMutex
