@@ -61,6 +61,7 @@ const unsigned short ICQ_SNACxSRV_SETxSTATUS       = 0x001e; // client
 const unsigned short ICQ_SNACxLOC_REQUESTxRIGHTS   = 0x0002; // client
 const unsigned short ICQ_SNAXxLOC_RIGHTSxGRANTED   = 0x0003; // server
 const unsigned short ICQ_SNACxLOC_SETxUSERxINFO    = 0x0004;
+const unsigned short ICQ_SNACxLOC_INFOxREQ         = 0x0005; // client
 const unsigned short ICQ_SNACxREPLYxUSERxINFO      = 0x0006; // client
 const unsigned short ICQ_SNACxREQUESTxUSERxINFO    = 0x0015; // client
 
@@ -100,6 +101,8 @@ const unsigned short ICQ_SNACxMETA                 = 0x0002; // client
 const unsigned short ICQ_SNACxNEW_UIN_ERROR        = 0x0001; // server
 const unsigned short ICQ_SNACxREGISTER_USER        = 0x0004; // client
 const unsigned short ICQ_SNACxNEW_UIN              = 0x0005; // server
+const unsigned short ICQ_SNACxREQUEST_IMAGE        = 0x000C; // client
+const unsigned short ICQ_SNACxSEND_IMAGE           = 0x000D; // server
 
 // Subtypes for list family
 const unsigned short ICQ_SNACxLIST_REQUESTxRIGHTS  = 0x0002; // client
@@ -128,6 +131,7 @@ const unsigned short ICQ_ROSTxNORMAL               = 0x0000;
 const unsigned short ICQ_ROSTxGROUP                = 0x0001;
 const unsigned short ICQ_ROSTxVISIBLE              = 0x0002;
 const unsigned short ICQ_ROSTxINVISIBLE            = 0x0003;
+const unsigned short ICQ_ROSTxPDINFO               = 0x0004;
 const unsigned short ICQ_ROSTxICQTIC               = 0x0009;
 const unsigned short ICQ_ROSTxIGNORE               = 0x000E;
 const unsigned short ICQ_ROSTxIMPORT               = 0x0013;
@@ -328,6 +332,8 @@ const unsigned short ICQ_CMDxSUB_VOICExMESSAGE     = 0x00DC;
 const unsigned short ICQ_CMDxSUB_IRCQxNET          = 0x00DD;
 const unsigned short ICQ_CMDxSUB_HOMEPAGExINVITE   = 0x00DE;
 const unsigned short ICQ_CMDxSUB_HOMEPAGExNEWS     = 0x00DF;
+// Created for protocol plugins
+const unsigned short ICQ_CMDxSUB_EMAILxALERT       = 0x00EC;
 // Encryption Licq extension command
 const unsigned short ICQ_CMDxSUB_SECURExOLD        = 0x00ED;  // Licq extension
 const unsigned short ICQ_CMDxSUB_SECURExCLOSE      = 0x00EE;  // Licq extension
@@ -409,6 +415,13 @@ const unsigned short ICQ_TYPING_INACTIVEx1        = 1; //icqlite sends this
 const unsigned short ICQ_TYPING_INACTIVE          = 1; //for plugins
 const unsigned short ICQ_TYPING_ACTIVE            = 2;
 
+// Privacy settings
+const unsigned char ICQ_PRIVACY_ALLOW_ALL         = 1;
+const unsigned char ICQ_PRIVACY_BLOCK_ALL         = 2;
+const unsigned char ICQ_PRIVACY_ALLOW_FOLLOWING   = 3; //used for invisible
+const unsigned char ICQ_PRIVACY_BLOCK_FOLLOWING   = 4;
+const unsigned char ICQ_PRIVACY_ALLOW_ONLY_LIST   = 5;
+
 const unsigned char ICQ_PLUGIN_REQUEST            = 0;
 const unsigned char ICQ_PLUGIN_SUCCESS            = 1;
 const unsigned char ICQ_PLUGIN_STATUSxREPLY       = 2;
@@ -428,6 +441,11 @@ const unsigned long ICQ_PLUGIN_RESP_PHONExBOOK  = 0x00000003;
 const unsigned long ICQ_PLUGIN_RESP_PICTURE     = 0x00000001;
 const unsigned long ICQ_PLUGIN_RESP_STATUSxLIST = 0x00010000;
 
+//charsets
+const unsigned short CHARSET_ASCII   = 0x0000;
+const unsigned short CHARSET_UNICODE = 0x0002;
+const unsigned short CHARSET_CUSTOM  = 0x0003;
+
 //actual plugins
 const char PLUGIN_PHONExBOOK[]        = { 0x90, 0x7C, 0x21, 0x2C, 0x91, 0x4D,
                                           0xD3, 0x11, 0xAD, 0xEB, 0x00, 0x04,
@@ -500,4 +518,15 @@ const char ICQ_CAPABILITY_LICQxVER[]  = { 'L',  'i',  'c',  'q',  ' ',  'c',
                                           'l',  'i',  'e',  'n',  't',  ' ',
                                           0x00, 0x00, 0x00, 0x00 };
  
+// For protocol plugins
+const unsigned long PP_SEND_MSG      = 0x00000001;
+const unsigned long PP_SEND_URL      = 0x00000002;
+const unsigned long PP_SEND_FILE     = 0x00000004;
+const unsigned long PP_SEND_CHAT     = 0x00000008;
+const unsigned long PP_SEND_CONTACT  = 0x00000010;
+const unsigned long PP_SEND_AUTH     = 0x00000020;
+const unsigned long PP_SEND_AUTHxREQ = 0x00000040;
+const unsigned long PP_SEND_SMS      = 0x00000080;
+const unsigned long PP_SEND_SECURE   = 0x00000100;
+
 #endif
