@@ -9,6 +9,8 @@ header file containing all the main procedures to interface with the ICQ server
 #include <vector>
 #include <list>
 #include <deque>
+#include <string>
+#include <algorithm>
 #include <stdarg.h>
 #include <stdio.h>
 
@@ -20,6 +22,8 @@ header file containing all the main procedures to interface with the ICQ server
 #include "licq_color.h"
 #include "licq_protoplugind.h"
 
+using std::string;
+
 class CProtoPlugin;
 class CPlugin;
 class CPacket;
@@ -84,6 +88,39 @@ typedef std::vector<CDaemonStats> DaemonStatsList;
 #define SAVE_STATS
 
 
+//====Conversations=============================================================
+class CConversation
+{
+public:
+  bool HasUser(const char *sz);
+  bool IsEmpty()        { return m_vUsers.size() == 0; }
+  int NumUsers()        { return m_vUsers.size(); }
+    
+  int Socket()          { return m_nSocket; }
+  unsigned long CID()   { return m_nCID; }
+  
+  string GetUser(int n) { return m_vUsers[n]; }
+
+private:  
+  CConversation(int nSocket, unsigned long nPPID);
+  ~CConversation() { }
+    
+  bool AddUser(const char *sz);
+  bool RemoveUser(const char *sz);
+  
+  int m_nSocket;
+  unsigned long m_nPPID;
+  unsigned long m_nCID;
+  vector<string> m_vUsers;
+  
+  static unsigned long s_nCID;
+  static pthread_mutex_t s_xMutex;
+  
+  friend class CICQDaemon;
+};
+
+typedef std::list<CConversation *> ConversationList;
+
 //=====CICQDaemon===============================================================
 enum EDaemonStatus {STATUS_ONLINE, STATUS_OFFLINE_MANUAL, STATUS_OFFLINE_FORCED };
 
@@ -149,7 +186,8 @@ public:
         \param nPPID The protocol ID.
         \param bActive The state of the typing notification. TRUE if active.
   */
-  void ProtoTypingNotification(const char *szId, unsigned long nPPID, bool Active);
+  void ProtoTypingNotification(const char *szId, unsigned long nPPID,
+                               bool Active, int nSocket = -1);
 
   //! Send a message to a user on this protocol.
   /*!
@@ -161,10 +199,12 @@ public:
         \param bMultipleRecipients True if sending the same message to
           more than one user.
         \param pColor The color of the text and background.
+        \param nCID The conversation ID for group messages (MSN & IRC)
   */
   unsigned long ProtoSendMessage(const char *szId, unsigned long nPPID,
      const char *szMessage, bool bOnline, unsigned short nLevel,
-     bool bMultipleRecipients = false, CICQColor *pColor = NULL);
+     bool bMultipleRecipients = false, CICQColor *pColor = NULL,
+     unsigned long nCID = 0);
 
   //! Send a URL to a user on this protocol
   /*!
@@ -195,13 +235,26 @@ public:
      const char *szReason, const char *szChatUsers, unsigned short nPort,
      unsigned short nLevel, bool bServer);
   void ProtoChatRequestRefuse(const char *szId, unsigned long nPPID,
-     const char *szReason, unsigned long nSequence, unsigned long nMsgID[],
+     const char *szReason, unsigned short nSequence, unsigned long nMsgID[],
      bool bDirect);
   void ProtoChatRequestAccept(const char *szId, unsigned long nPPID,
      unsigned short nPort, const char *szClients, unsigned long nSequeunce,
      unsigned long nMsgID[], bool bDirect);
   void ProtoChatRequestCancel(const char *szId, unsigned long nPPID,
+     unsigned short nSequence);
+  
+  unsigned long ProtoFileTransfer(const char *szId, unsigned long nPPID,
+     const char *szFilename, const char *szDescription, ConstFileList &lFileList,
+     unsigned short nLevel, bool bServer);
+  void ProtoFileTransferRefuse(const char *szId, unsigned long nPPID,
+     const char *szReason, unsigned long nSequence, unsigned long nFlag1,
+     unsigned long nFlag2, bool bDirect = false);
+  void ProtoFileTransferCancel(const char *szId, unsigned long nPPID,
      unsigned long nSequence);
+  void ProtoFileTransferAccept(const char *szId, unsigned long nPPID,
+     unsigned short nPort, unsigned long nSequence = 0, unsigned long nFlag1 = 0,
+     unsigned long nFlag2 = 0, const char *szDesc = 0, const char *szFile = 0,
+     unsigned long nFileSize = 0,bool bDirect = false);
   
   unsigned long ProtoAuthorizeGrant(const char *szId, unsigned long nPPID,
      const char *szMessage);
@@ -209,6 +262,19 @@ public:
   unsigned long ProtoAuthorizeRefuse(const char *szId, unsigned long nPPID,
      const char *szMessage);
 
+  unsigned long ProtoRequestInfo(const char *szId, unsigned long nPPID);
+
+  unsigned long ProtoSetGeneralInfo(unsigned long nPPID, const char *szAlias,
+    const char *szFirstName, const char *szLastName, const char *szEmailPrimary,
+    const char *szCity, const char *szState, const char *szPhoneNumber,
+    const char *szFaxNumber, const char *szAddress, const char *szCellularNumber,
+    const char *szZipCode, unsigned short nCountryCode, bool bHideEmail);
+    
+  unsigned long ProtoOpenSecureChannel(const char *szId, unsigned long nPPID);
+  unsigned long ProtoCloseSecureChannel(const char *szId, unsigned long nPPID);
+  void ProtoOpenSecureChannelCancel(const char *szId, unsigned long nPPID,
+    unsigned long nSequence);
+    
   // TCP (user) functions
   // Message
   unsigned long icqSendMessage(const char *szId, const char *szMessage,
@@ -240,22 +306,35 @@ public:
      const char *szReason, const char *szChatUsers, unsigned short nPort,
      unsigned short nLevel, bool bServer);
   void icqChatRequestRefuse(unsigned long nUin, const char *szReason,
-     unsigned long nSequence, unsigned long nMsgID[], bool bDirect);
+     unsigned short nSequence, unsigned long nMsgID[], bool bDirect);
   void icqChatRequestAccept(unsigned long nUin, unsigned short nPort,
-     const char *szClients, unsigned long nSequence, unsigned long nMsgID[], bool bDirect);
-  void icqChatRequestCancel(unsigned long nUin, unsigned long nSequence);
+     const char *szClients, unsigned short nSequence, unsigned long nMsgID[], bool bDirect);
+  void icqChatRequestCancel(unsigned long nUin, unsigned short nSequence);
   // File Transfer
+  unsigned long icqFileTransfer(const char *szId, const char *szFilename,
+     const char *szDescription, ConstFileList &lFileList,
+     unsigned short nLevel, bool bServer);
   unsigned long icqFileTransfer(unsigned long nUin, const char *szFilename,
      const char *szDescription, ConstFileList &lFileList,
      unsigned short nLevel, bool bServer);
+  void icqFileTransferRefuse(const char *szId, const char *szReason,
+     unsigned short nSequence, unsigned long nMsgID[], bool bDirect);  
   void icqFileTransferRefuse(unsigned long nUin, const char *szReason,
-     unsigned long nSequence, unsigned long nMsgID[], bool bDirect);
-  void icqFileTransferCancel(unsigned long nUin, unsigned long nSequence);
+     unsigned short nSequence, unsigned long nMsgID[], bool bDirect);
+  void icqFileTransferCancel(const char *szId, unsigned short nSequence);
+  void icqFileTransferCancel(unsigned long nUin, unsigned short nSequence);
+  void icqFileTransferAccept(const char *szId, unsigned short nPort,
+     unsigned short nSequence, unsigned long nMsgID[], bool bDirect,
+     const char *szDesc, const char *szFile, unsigned long nFileSize);  
   void icqFileTransferAccept(unsigned long nUin, unsigned short nPort,
-     unsigned long nSequence, unsigned long nMsgID[], bool bDirect);
+     unsigned short nSequence, unsigned long nMsgID[], bool bDirect,
+     const char *szDesc, const char *szFile, unsigned long nFileSize);
+  unsigned long icqOpenSecureChannel(const char *szId);
   unsigned long icqOpenSecureChannel(unsigned long nUin);
+  unsigned long icqCloseSecureChannel(const char *szId);
   unsigned long icqCloseSecureChannel(unsigned long nUin);
-  void icqOpenSecureChannelCancel(unsigned long nUin, unsigned long nSequence);
+  void icqOpenSecureChannelCancel(const char *szId, unsigned short nSequence);
+  void icqOpenSecureChannelCancel(unsigned long nUin, unsigned short nSequence);
 
   // Plugins
   unsigned long icqRequestInfoPluginList(const char *szId,
@@ -271,6 +350,9 @@ public:
 
   // Server functions
   void icqRegister(const char *_szPasswd);
+  void icqVerifyRegistration();
+  void icqVerify(const char *);
+  unsigned long icqFetchAutoResponseServer(const char *);
   unsigned long icqFetchAutoResponseServer(unsigned long);
   unsigned long icqLogon(unsigned short logonStatus);
   unsigned long icqUserBasicInfo(const char *);
@@ -355,19 +437,22 @@ public:
   void icqRenameUser(const char *_szId);
   void icqExportUsers(UserStringList &, unsigned short);
   void icqExportGroups(GroupList &);
+  void icqUpdateServerGroups();
   void icqUpdatePhoneBookTimestamp();
   void icqUpdatePictureTimestamp();
   void icqSetPhoneFollowMeStatus(unsigned long nNewStatus);
   void icqUpdateContactList();
   void icqTypingNotification(const char *_szId, bool _bActive);
-
+  void icqCheckInvisible(const char *_szId);
+  
   // Visible/Invisible/Ignore list functions
+  void ProtoToggleInvisibleList(const char *_szId, unsigned long _nPPID);
+  void ProtoToggleVisibleList(const char *_szId, unsigned long _nPPID);
+  
   void icqAddToVisibleList(const char *_szId, unsigned long _nPPID);
   void icqRemoveFromVisibleList(const char *_szId, unsigned long _nPPID);
-  void icqToggleVisibleList(const char *_szId, unsigned long _nPPID);
   void icqAddToInvisibleList(const char *_szId, unsigned long _nPPID);
   void icqRemoveFromInvisibleList(const char *_szId, unsigned long _nPPID);
-  void icqToggleInvisibleList(const char *_szId, unsigned long _nPPID);
   void icqAddToIgnoreList(const char *_szId, unsigned long _nPPID);
   void icqRemoveFromIgnoreList(const char *_szId, unsigned long _nPPID);
   void icqToggleIgnoreList(const char *_szId, unsigned long _nPPID);
@@ -381,7 +466,8 @@ public:
   void icqAddToIgnoreList(unsigned long nUin) __attribute__ ((deprecated)); // deprecated!
   void icqRemoveFromIgnoreList(unsigned long nUin) __attribute__ ((deprecated)); // deprecated!
   void icqToggleIgnoreList(unsigned long nUin) __attribute__ ((deprecated)); // deprecated!
-
+  
+  void icqClearServerList();
   void CheckExport();
 
   void PluginList(PluginsList &l);
@@ -399,10 +485,10 @@ public:
   char *ProtoPluginName(unsigned long);
 
   void PluginUIViewEvent(const char *szId, unsigned long nPPID ) {
-    PushPluginSignal(new CICQSignal(SIGNAL_UI_VIEWEVENT, 0, szId, nPPID, 0, 0));
+    PushPluginSignal(new CICQSignal(SIGNAL_UI_VIEWEVENT, 0, szId, nPPID, 0));
   }
   void PluginUIMessage(const char *szId, unsigned long nPPID) {
-    PushPluginSignal(new CICQSignal(SIGNAL_UI_MESSAGE, 0, szId, nPPID, 0,0));
+    PushPluginSignal(new CICQSignal(SIGNAL_UI_MESSAGE, 0, szId, nPPID, 0));
   }
 
   void UpdateAllUsers();
@@ -419,7 +505,8 @@ public:
   unsigned short BirthdayRange() { return m_nBirthdayRange; }
   void BirthdayRange(unsigned short r) { m_nBirthdayRange = r; }
 
-  bool AddUserToList(const char *szId, unsigned long PPID, bool bNotify = true);
+  bool AddUserToList(const char *szId, unsigned long PPID, bool bNotify = true,
+                     bool bTempUser = false);
   bool AddUserToList(unsigned long _nUin, bool bNotify = true);
   void AddUserToList(ICQUser *);
   void RemoveUserFromList(unsigned long _nUin);
@@ -509,15 +596,25 @@ public:
   time_t Uptime() { return time(NULL) - m_nStartTime; }
   void ResetStats();
 
+  // Conversation functions
+  CConversation *AddConversation(int nSocket, unsigned long nPPID);
+  bool AddUserConversation(unsigned long nCID, const char *szId);
+  bool AddUserConversation(int nSocket, const char *szId);
+  bool RemoveUserConversation(unsigned long nCID, const char *szId);
+  bool RemoveUserConversation(int nSocket, const char *szId);
+  CConversation *FindConversation(int nSocket);
+  CConversation *FindConversation(unsigned long nCID);
+  bool RemoveConversation(unsigned long nCID);
+  
   // Common message handler
   void ProcessMessage(ICQUser *user, CBuffer &packet, char *message,
      unsigned short nMsgType, unsigned long nMask,
-     unsigned long nMsgID[], unsigned long nSequence,
+     unsigned long nMsgID[], unsigned short nSequence,
      bool bIsAck, bool &bNewUser);
 
   bool ProcessPluginMessage(CBuffer &packet, ICQUser *u, unsigned char nChannel,
      bool bIsAck, unsigned long nMsgID1,
-     unsigned long nMsgID2, unsigned long nSequence,
+     unsigned long nMsgID2, unsigned short nSequence,
      TCPSocket *pSock);
   bool WaitForReverseConnection(unsigned short id, unsigned long uin);
   
@@ -550,7 +647,8 @@ protected:
        m_bOnlineNotifies,
        m_bAlwaysOnlineNotify,
        m_bTCPEnabled,
-       m_bFirewall;
+       m_bFirewall,
+       m_bVerify;
   time_t m_tLogonTime;
   char *m_szRegisterPasswd;
   pthread_t m_nRegisterThreadId;
@@ -582,6 +680,9 @@ protected:
   static pthread_mutex_t mutex_reverseconnect;
   static pthread_cond_t  cond_reverseconnect_done;
 
+  ConversationList m_lConversations;
+  pthread_mutex_t mutex_conversations;
+  
   std::list <ICQEvent *> m_lxRunningEvents;
   pthread_mutex_t mutex_runningevents;
   std::list <ICQEvent *> m_lxExtendedEvents;
@@ -603,23 +704,24 @@ protected:
   void ChangeUserStatus(ICQUser *u, unsigned long s);
   bool AddUserEvent(ICQUser *, CUserEvent *);
   void RejectEvent(unsigned long, CUserEvent *);
-  ICQUser *FindUserForInfoUpdate(unsigned long nUin, ICQEvent *e, const char *);
+  ICQUser *FindUserForInfoUpdate(const char *szId, ICQEvent *e, const char *);
   unsigned long FindUinByCellular(const char *_szCellular);
 
   void icqRegisterFinish();
   void icqPing();
   void icqSendVisibleList();
   void icqSendInvisibleList();
+  void icqCreatePDINFO();
   void icqRequestSystemMsg();
   ICQEvent *icqSendThroughServer(const char *szId, unsigned char format, char *_sMessage,
-    CUserEvent *, unsigned short = 0);
+    CUserEvent *, unsigned short = 0, size_t = 0);
   ICQEvent* icqSendThroughServer(unsigned long nUin, unsigned char format, char *_sMessage, CUserEvent *, unsigned short = 0);
   void SaveUserList();
 
   void FailEvents(int sd, int err);
   ICQEvent *DoneServerEvent(unsigned long, EventResult);
   ICQEvent *DoneEvent(ICQEvent *e, EventResult _eResult);
-  ICQEvent *DoneEvent(int _nSD, unsigned long _nSequence, EventResult _eResult);
+  ICQEvent *DoneEvent(int _nSD, unsigned short _nSequence, EventResult _eResult);
   ICQEvent *DoneEvent(unsigned long tag, EventResult _eResult);
   ICQEvent *DoneExtendedServerEvent(const unsigned short, EventResult);
   ICQEvent *DoneExtendedEvent(ICQEvent *, EventResult);
@@ -648,7 +750,7 @@ protected:
 
   //--- Families ---------
   void ProcessServiceFam(CBuffer&, unsigned short);
-  void ProcessLocationFam(const CBuffer&, unsigned short);
+  void ProcessLocationFam(CBuffer&, unsigned short);
   void ProcessBuddyFam(CBuffer&, unsigned short);
   void ProcessMessageFam(CBuffer&, unsigned short);
   void ProcessVariousFam(CBuffer&, unsigned short);
