@@ -10,6 +10,7 @@
 
 typedef std::list<const char *> ConstFileList;
 
+#define LICQ_PPID 0x4C696371  // "Licq"
 #define EVENT_HEADER_SIZE  80
 
 // Define some event flags, leave the 2 LSB's for the licq version
@@ -29,8 +30,8 @@ class CUserEvent
 {
 public:
    CUserEvent(unsigned short _nSubCommand, unsigned short _nCommand,
-              unsigned long _nSequence, time_t _tTime,
-              unsigned long _nFlags);
+              unsigned short _nSequence, time_t _tTime,
+              unsigned long _nFlags, unsigned long _nConvoId = 0);
    CUserEvent(const CUserEvent *);
    virtual ~CUserEvent();
 
@@ -40,7 +41,7 @@ public:
    time_t Time()  {  return m_tTime; }
    const char *LicqVersionStr();
    static const char *LicqVersionToString(unsigned long);
-   unsigned long Sequence()  { return m_nSequence; }
+   unsigned short Sequence()  { return m_nSequence; }
    unsigned short Command()  { return m_nCommand; }
    unsigned short SubCommand()  { return m_nSubCommand; }
    int Id()  { return m_nId; }
@@ -53,14 +54,15 @@ public:
    unsigned short LicqVersion()  { return m_nFlags & E_LICQxVER; }
    direction Direction()  {  return m_eDir; }
    CICQColor *Color() { return &m_sColor; }
-
+   int ConvoId() { return m_nConvoId; }
+   
    bool Pending() { return m_bPending; }
    void SetPending(bool b)  { m_bPending = b; }
 
 protected:
    virtual void AddToHistory(ICQUser *, direction) = 0;
    int AddToHistory_Header(direction, char *);
-   void AddToHistory_Flush(ICQUser *, char *);
+   void AddToHistory_Flush(ICQUser *, char *, unsigned long = LICQ_PPID);
 
    void SetDirection(direction d)  { m_eDir = d; }
    void Cancel() { m_nFlags |= E_CANCELLED; }
@@ -75,7 +77,7 @@ protected:
    char          *m_szText;
    unsigned short m_nCommand;
    unsigned short m_nSubCommand;
-   unsigned long  m_nSequence;
+   unsigned short m_nSequence;
    int            m_nId;
    time_t         m_tTime;
    unsigned long  m_nFlags;
@@ -83,8 +85,10 @@ protected:
    direction      m_eDir;
    bool           m_bPending;
    CICQColor      m_sColor;
-
+   unsigned long  m_nConvoId;
+   
 friend class CICQDaemon;
+friend class CMSN;
 friend class CUserHistory;
 };
 
@@ -95,7 +99,7 @@ class CEventMsg : public CUserEvent
 {
 public:
    CEventMsg(const char *_szMessage, unsigned short _nCommand,
-             time_t _tTime, unsigned long _nFlags);
+             time_t _tTime, unsigned long _nFlags, unsigned long _nConvoId = 0);
    virtual ~CEventMsg();
    virtual CEventMsg *Copy()
       {
@@ -106,7 +110,8 @@ public:
    const char *Message()  { return m_szMessage; }
    virtual void AddToHistory(ICQUser *, direction);
 
-   static CEventMsg *Parse(char *sz, unsigned short nCmd, time_t nTime, unsigned long nFlags);
+   static CEventMsg *Parse(char *sz, unsigned short nCmd, time_t nTime,
+     unsigned long nFlags, unsigned long nConvoId = 0);
 protected:
    void CreateDescription();
    char *m_szMessage;
@@ -119,15 +124,15 @@ class CEventFile : public CUserEvent
 public:
    CEventFile(const char *_szFilename, const char *_szFileDescription,
               unsigned long _nFileSize, ConstFileList &lFileList,
-              unsigned long _nSequence, time_t _tTime,
-              unsigned long _nFlags, unsigned long _nMsgID1 = 0,
-              unsigned long _nMsgID2 = 0);
+              unsigned short _nSequence, time_t _tTime,
+              unsigned long _nFlags, unsigned long _nConovId = 0,
+              unsigned long _nMsgID1 = 0, unsigned long _nMsgID2 = 0);
    virtual ~CEventFile();
    virtual void AddToHistory(ICQUser *, direction);
    virtual CEventFile *Copy()
       {
         CEventFile *e = new CEventFile(m_szFilename, m_szFileDescription,
-         m_nFileSize, m_lFileList, m_nSequence, m_tTime, m_nFlags, m_nMsgID[0],
+         m_nFileSize, m_lFileList, m_nSequence, m_tTime, m_nFlags, m_nConvoId, m_nMsgID[0],
          m_nMsgID[1]);
         e->CopyBase(this);
         return e;
@@ -154,7 +159,7 @@ class CEventUrl : public CUserEvent
 public:
    CEventUrl(const char *_szUrl, const char *_szUrlDescription,
              unsigned short _nCommand, time_t _tTime,
-             unsigned long _nFlags);
+             unsigned long _nFlags, unsigned long _nConvoId = 0);
    virtual ~CEventUrl();
    virtual void AddToHistory(ICQUser *, direction);
    virtual CEventUrl *Copy()
@@ -167,7 +172,8 @@ public:
    const char *Url()  { return m_szUrl; }
    const char *Description()  { return m_szUrlDescription; }
 
-   static CEventUrl *Parse(char *sz, unsigned short nCmd, time_t nTime, unsigned long nFlags);
+   static CEventUrl *Parse(char *sz, unsigned short nCmd, time_t nTime,
+     unsigned long nFlags, unsigned long nConvoId = 0);
 protected:
    void CreateDescription();
    char *m_szUrl;
@@ -179,17 +185,18 @@ protected:
 class CEventChat : public CUserEvent
 {
 public:
-   CEventChat(const char *szReason, unsigned long nSequence, time_t tTime,
-      unsigned long nFlags, unsigned long nMsgID1 = 0, unsigned long nMsgID2=0);
+   CEventChat(const char *szReason, unsigned short nSequence, time_t tTime,
+      unsigned long nFlags, unsigned long nConvoId = 0, unsigned long nMsgID1 = 0,
+      unsigned long nMsgID2 = 0);
    CEventChat(const char *szReason, const char *szClients, unsigned short nPort,
-      unsigned long nSequence, time_t tTime, unsigned long nFlags,
-      unsigned long nMsgID1 = 0, unsigned long nMsgID2 = 0);
+      unsigned short nSequence, time_t tTime, unsigned long nFlags,
+      unsigned long _nConvoId = 0, unsigned long nMsgID1 = 0, unsigned long nMsgID2 = 0);
   virtual ~CEventChat();
   virtual void AddToHistory(ICQUser *, direction);
   virtual CEventChat *Copy()
      {
        CEventChat *e = new CEventChat(m_szText, m_szClients, m_nPort,
-        m_nSequence, m_tTime, m_nFlags, m_nMsgID[0], m_nMsgID[1]);
+        m_nSequence, m_tTime, m_nFlags, m_nConvoId, m_nMsgID[0], m_nMsgID[1]);
        e->CopyBase(this);
        return e;
      }
@@ -210,11 +217,9 @@ protected:
 class CEventAdded : public CUserEvent
 {
 public:
-#ifdef PROTOCOL_PLUGIN
    CEventAdded(const char *_szId, unsigned long _nPPID, const char *_szAlias,
                const char *_szFirstName, const char *_szLastName, const char *_szEmail,
                unsigned short _nCommand, time_t _tTime, unsigned long _nFlags);
-#endif
    CEventAdded(unsigned long _nUin, const char *_szAlias, const char *_szFirstName,
                const char *_szLastName, const char *_szEmail,
                unsigned short _nCommand, time_t _tTime, unsigned long _nFlags);
@@ -222,28 +227,19 @@ public:
    virtual void AddToHistory(ICQUser *, direction);
    virtual CEventAdded *Copy()
       {
-#ifdef PROTOCOL_PLUGIN
         CEventAdded *e = new CEventAdded(m_szId, m_nPPID, m_szAlias, m_szFirstName,
          m_szLastName, m_szEmail, m_nCommand, m_tTime, m_nFlags);
-#else
-        CEventAdded *e = new CEventAdded(m_nUin, m_szAlias, m_szFirstName,
-         m_szLastName, m_szEmail, m_nCommand, m_tTime, m_nFlags);
-#endif
         e->CopyBase(this);
         return e;
       }
    unsigned long Uin()  { return m_nUin; };
-#ifdef PROTOCOL_PLUGIN
    char *IdString()     { return m_szId; }
    unsigned long PPID() { return m_nPPID; }
-#endif
 protected:
    void CreateDescription();
    unsigned long m_nUin;
-#ifdef PROTOCOL_PLUGIN
    char *m_szId;
    unsigned long m_nPPID;
-#endif
    char *m_szAlias;
    char *m_szFirstName;
    char *m_szLastName;
@@ -256,12 +252,10 @@ protected:
 class CEventAuthRequest : public CUserEvent
 {
 public:
-#ifdef PROTOCOL_PLUGIN
    CEventAuthRequest(const char *_szId, unsigned long _nPPID, const char *_szAlias,
                      const char *_szFirstName, const char *_szLastName, const char *_szEmail,
                      const char *_szReason, unsigned short _nCommand, time_t _tTime,
                      unsigned long _nFlags);
-#endif
    CEventAuthRequest(unsigned long _nUin, const char *_szAlias, const char *_szFirstName,
                  const char *_szLastName, const char *_szEmail, const char *_szReason,
                  unsigned short _nCommand, time_t _tTime, unsigned long _nFlags);
@@ -269,30 +263,20 @@ public:
    virtual void AddToHistory(ICQUser *, direction);
    virtual CEventAuthRequest *Copy()
       {
-#ifdef PROTOCOL_PLUGIN
         CEventAuthRequest *e = new CEventAuthRequest(m_szId, m_nPPID,
          m_szAlias, m_szFirstName, m_szLastName, m_szEmail, m_szReason,
          m_nCommand, m_tTime, m_nFlags);
-#else
-        CEventAuthRequest *e = new CEventAuthRequest(m_nUin, m_szAlias,
-         m_szFirstName, m_szLastName, m_szEmail, m_szReason, m_nCommand,
-         m_tTime, m_nFlags);
-#endif
         e->CopyBase(this);
         return e;
       }
    unsigned long Uin()  { return m_nUin; };
-#ifdef PROTOCOL_PLUGIN
    char *IdString()     { return m_szId; }
    unsigned long PPID() { return m_nPPID; }
-#endif
 protected:
    void CreateDescription();
    unsigned long m_nUin;
-#ifdef PROTOCOL_PLUGIN
    char *m_szId;
    unsigned long m_nPPID;
-#endif
    char *m_szAlias;
    char *m_szFirstName;
    char *m_szLastName;
@@ -305,38 +289,27 @@ protected:
 class CEventAuthGranted : public CUserEvent
 {
 public:
-#ifdef PROTOCOL_PLUGIN
    CEventAuthGranted(const char *_szId, unsigned long _nPPID, const char *_szMsg,
                      unsigned short _nCommand, time_t _tTime, unsigned long _nFlags);
-#endif
    CEventAuthGranted(unsigned long _nUin, const char *_szMessage,
               unsigned short _nCommand, time_t _tTime, unsigned long _nFlags);
    virtual ~CEventAuthGranted();
    virtual void AddToHistory(ICQUser *, direction);
    virtual CEventAuthGranted *Copy()
       {
-#ifdef PROTOCOL_PLUGIN
         CEventAuthGranted *e = new CEventAuthGranted(m_szId, m_nPPID, m_szMessage,
          m_nCommand, m_tTime, m_nFlags);
-#else
-        CEventAuthGranted *e = new CEventAuthGranted(m_nUin, m_szMessage,
-         m_nCommand, m_tTime, m_nFlags);
-#endif
         e->CopyBase(this);
         return e;
       }
    unsigned long Uin()  { return m_nUin; };
-#ifdef PROTOCOL_PLUGIN
    char *IdString()     { return m_szId; }
    unsigned long PPID() { return m_nPPID; }
-#endif
 protected:
    void CreateDescription();
    unsigned long m_nUin;
-#ifdef PROTOCOL_PLUGIN
    char *m_szId;
    unsigned long m_nPPID;
-#endif
    char *m_szMessage;
 };
 
@@ -345,38 +318,27 @@ protected:
 class CEventAuthRefused : public CUserEvent
 {
 public:
-#ifdef PROTOCOL_PLUGIN
    CEventAuthRefused(const char *_szId, unsigned long _nPPID, const char *_szMsg,
                      unsigned short _nCommand, time_t _tTime, unsigned long _nFlags);
-#endif
    CEventAuthRefused(unsigned long _nUin, const char *_szMessage,
               unsigned short _nCommand, time_t _tTime, unsigned long _nFlags);
    virtual ~CEventAuthRefused();
    virtual void AddToHistory(ICQUser *, direction);
    virtual CEventAuthRefused *Copy()
       {
-#ifdef PROTOCOL_PLUGIN
         CEventAuthRefused *e = new CEventAuthRefused(m_szId, m_nPPID,
          m_szMessage, m_nCommand, m_tTime, m_nFlags);
-#else
-        CEventAuthRefused *e = new CEventAuthRefused(m_nUin, m_szMessage,
-         m_nCommand, m_tTime, m_nFlags);
-#endif
         e->CopyBase(this);
         return e;
       }
    unsigned long Uin()  { return m_nUin; };
-#ifdef PROTOCOL_PLUGIN
    char *IdString()     { return m_szId; }
    unsigned long PPID() { return m_nPPID; }
-#endif
 protected:
    void CreateDescription();
    unsigned long m_nUin;
-#ifdef PROTOCOL_PLUGIN
    char *m_szId;
    unsigned long m_nPPID;
-#endif
    char *m_szMessage;
 };
 
@@ -431,30 +393,21 @@ protected:
 class CContact
 {
 public:
-#ifdef PROTOCOL_PLUGIN
   CContact(const char *s, unsigned long n, const char *a)
-    {  m_szId = strdup(s); m_nPPID = n; m_szAlias = strdup(a); }
-#endif
-  CContact(unsigned long n, const char *a) { m_nUin = n; m_szAlias = strdup(a); }
-#ifdef PROTCOL_PLUGIN
+    {  m_szId = strdup(s); m_nPPID = n; m_szAlias = strdup(a); m_nUin = 0; }
+  CContact(unsigned long n, const char *a)
+    { m_szId = 0; m_nPPID = 0; m_nUin = n; m_szAlias = strdup(a); }
   ~CContact() { free(m_szAlias); free(m_szId); }
-#else
-  ~CContact() { free(m_szAlias); }
-#endif
 
   unsigned long Uin() { return m_nUin; }
   const char *Alias() { return m_szAlias; }
-#ifdef PROTOCOL_PLUGIN
   char *IdString()    { return m_szId; }
   unsigned long PPID(){ return m_nPPID; }
-#endif
 protected:
   unsigned long m_nUin;
   char *m_szAlias;
-#ifdef PROTOCOL_PLUGIN
   char *m_szId;
   unsigned long m_nPPID;
-#endif
 };
 typedef std::list<CContact *> ContactList;
 
@@ -535,6 +488,62 @@ protected:
        *m_szMessage;
 };
 
+//-----CEventEmailAlert-----------------------------------------------------
+class CEventEmailAlert : public CUserEvent
+{
+public:
+  CEventEmailAlert(const char *_szName, const char *_szEmail,
+                   const char *_szTo, const char *_szSubject, time_t _tTime,
+                   const char *_szMSPAuth = 0, const char *_szSID = 0,
+                   const char *_szKV = 0, const char *_szId = 0,
+                   const char *_szPostURL = 0, const char *_szMsgURL = 0,
+                   const char *_szCreds = 0, unsigned long _nSessionLength = 0);
+  virtual ~CEventEmailAlert();
+
+  virtual void AddToHistory(ICQUser *, direction);
+
+  virtual CEventEmailAlert *Copy()
+  {
+    CEventEmailAlert *e = new CEventEmailAlert(m_szName, m_szTo, m_szEmail,
+      m_szSubject, m_tTime, m_szMSPAuth, m_szSID, m_szKV, m_szId,
+      m_szPostURL, m_szMsgURL, m_szCreds, m_nSessionLength);
+    e->CopyBase(this);
+    return e;
+  }
+
+  char *From()    { return m_szName; }
+  char *To()      { return m_szTo; }
+  char *Email()   { return m_szEmail; }
+  char *Subject() { return m_szSubject; }
+  
+  char *MSPAuth() { return m_szMSPAuth; }
+  char *SID()     { return m_szSID; }
+  char *KV()      { return m_szKV; }
+  char *Id()      { return m_szId; }
+  char *PostURL() { return m_szPostURL; }
+  char *MsgURL()  { return m_szMsgURL; }
+  char *Creds()   { return m_szCreds; }
+  unsigned long SessionLength() { return m_nSessionLength; }
+  
+protected:
+  void CreateDescription();
+
+  // Info
+  char *m_szName,
+       *m_szTo,
+       *m_szEmail,
+       *m_szSubject;
+       
+  // For Licq to view an MSN email
+  char *m_szMSPAuth,
+       *m_szSID,
+       *m_szKV,
+       *m_szId,
+       *m_szPostURL,
+       *m_szMsgURL,
+       *m_szCreds;
+  unsigned long m_nSessionLength;
+};
 //-----CEventPlugin------------------------------------------------------------
 class CEventPlugin : public CUserEvent
 {
