@@ -15,9 +15,7 @@
 
 class TCPSocket;
 
-#ifdef PROTOCOL_PLUGIN
 #define LICQ_PPID 0x4C696371  // "Licq"
-#endif
 
 /*---------------------------------------------------------------------------
  * FOR_EACH_USER
@@ -36,7 +34,6 @@ class TCPSocket;
       pUser->Lock(x);                                    \
       {
 
-#ifdef PROTOCOL_PLUGIN
 #define FOR_EACH_PROTO_USER_START(x, y)                  \
   {                                                      \
     ICQUser *pUser;                                      \
@@ -74,8 +71,6 @@ class TCPSocket;
           break;                                          \
         }
 
-#endif
-
 #define FOR_EACH_USER_END                \
       }                                  \
       pUser->Unlock();                   \
@@ -83,7 +78,6 @@ class TCPSocket;
     gUserManager.UnlockUserList();       \
   }
 
-#ifdef PROTOCOL_PLUGIN
 #define FOR_EACH_PROTO_USER_END        \
         }                                \
         pUser->Unlock();                 \
@@ -91,7 +85,6 @@ class TCPSocket;
     }                                    \
     gUserManager.UnlockUserList();       \
   }
-#endif
 
 #define FOR_EACH_USER_BREAK              \
         {                                \
@@ -99,7 +92,6 @@ class TCPSocket;
           break;                         \
         }
 
-#ifdef PROTOCOL_PLUGIN
 #define FOR_EACH_PROTO_USER_BREAK        \
         {                                \
           gUserManager.DropUser(pUser);  \
@@ -111,7 +103,6 @@ class TCPSocket;
           gUserManager.DropUser(pUser);  \
           continue;                      \
         }
-#endif
 
 #define FOR_EACH_USER_CONTINUE           \
         {                                \
@@ -138,7 +129,6 @@ class TCPSocket;
       nUin = (*_i_)->Uin();                              \
       {
 
-#ifdef PROTOCOL_PLUGIN
 #define FOR_EACH_PROTO_ID_START(x)                       \
   {                                                      \
     char *szId;                                          \
@@ -150,7 +140,6 @@ class TCPSocket;
       {                                                  \
         szId = (*_i_)->IdString();                       \
         {
-#endif
 
 #define FOR_EACH_UIN_END                 \
       }                                  \
@@ -158,46 +147,39 @@ class TCPSocket;
     gUserManager.UnlockUserList();       \
   }
 
-#ifdef PROTOCOL_PLUGIN
 #define FOR_EACH_PROTO_ID_END            \
         }                                \
       }                                  \
     }                                    \
     gUserManager.UnlockUserList();       \
   }
-#endif
 
 #define FOR_EACH_UIN_BREAK               \
         {                                \
           break;                         \
         }
 
-#ifdef PROTOCOL_PLUGIN
 #define FOR_EACH_PROTO_ID_BREAK          \
         {                                \
           break;                         \
         }
-#endif
 
 #define FOR_EACH_UIN_CONTINUE            \
         {                                \
           continue;                      \
         }
 
-#ifdef PROTOCOL_PLUGIN
 #define FOR_EACH_PROTO_ID_CONTINUE       \
         {                                \
           continue;                      \
         }
-#endif
 
 typedef std::list<ICQUser *> UserList;
-#ifdef PROTOCOL_PLUGIN
 typedef std::list<class ICQOwner *> OwnerList;
-#endif
 typedef std::vector<char *> GroupList;
 typedef std::vector<unsigned short> GroupIDList;
 typedef std::list<unsigned long> UinList;
+typedef std::list<char *> UserStringList;
 typedef std::vector <class CUserEvent *> UserEventList;
 
 // Cheap hack as I'm too lazy to move the relevant functions to user.cpp
@@ -255,9 +237,117 @@ const unsigned short LAST_RECV_EVENT    = 1;
 const unsigned short LAST_SENT_EVENT    = 2;
 const unsigned short LAST_CHECKED_AR    = 3;
 
+const unsigned short MAX_CATEGORY_SIZE  = 60;
+
+typedef enum
+{
+  CAT_INTERESTS,
+  CAT_ORGANIZATION,
+  CAT_BACKGROUND,
+  CAT_MAX
+} UserCat;
 
 //+++++OBJECTS++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 
+//====ICQUserCategory===========================================================
+class ICQUserCategory
+{
+public:
+  ICQUserCategory(UserCat uc);
+  ~ICQUserCategory();
+  bool AddCategory (unsigned short cat, const char *descr);
+  void Clean();
+
+  bool Get(unsigned d,unsigned short *id, char const ** descr) const;
+  UserCat GetCategory() { return m_uc; }
+
+  static const unsigned MAX_CATEGORIES = 4;
+
+private:
+  bool SaveToDisk(CIniFile &m_fConf,const char *const szN,
+                  const char *const szCat,const char *const szDescr);
+  bool LoadFromDisk(CIniFile &m_fConf, const char *const szN,
+                    const char *const szCat,const char *const szDescr);
+
+  unsigned short used;
+  
+  struct cat
+  {       unsigned short id;
+          char *descr;
+  };
+  struct cat data[MAX_CATEGORIES];
+  UserCat m_uc;
+  friend class ICQUser;
+};
+
+struct PhoneBookEntry
+{
+  char *szDescription;
+  char *szAreaCode;
+  char *szPhoneNumber;
+  char *szExtension;
+  char *szCountry;
+  unsigned long nActive;
+  unsigned long nType;
+  char *szGateway;
+  unsigned long nGatewayType;
+  unsigned long nSmsAvailable;
+  unsigned long nRemoveLeading0s;
+  unsigned long nPublish;
+};
+
+const unsigned short MAX_DESCRIPTION_SIZE  = 16;
+const unsigned short MAX_AREAxCODE_SIZE    =  5;
+const unsigned short MAX_PHONExNUMBER_SIZE = 16;
+const unsigned short MAX_EXTENSION_SIZE    = 20;
+const unsigned short MAX_GATEWAY_SIZE      = 64;
+const unsigned short MAX_PICTURE_SIZE      = 8081;
+
+enum EPhoneType
+{
+  TYPE_PHONE,
+  TYPE_CELLULAR,
+  TYPE_CELLULARxSMS,
+  TYPE_FAX,
+  TYPE_PAGER,
+  TYPE_MAX
+};
+
+enum EGatewayType
+{
+  GATEWAY_BUILTIN = 1,
+  GATEWAY_CUSTOM
+};
+
+enum EPublish
+{
+  PUBLISH_ENABLE = 1,
+  PUBLISH_DISABLE
+};
+
+//====ICQUserPhoneBook=========================================================
+class ICQUserPhoneBook
+{
+public:
+  ICQUserPhoneBook();
+  ~ICQUserPhoneBook();
+  void AddEntry(const struct PhoneBookEntry *entry);
+  void SetEntry(const struct PhoneBookEntry *entry, unsigned long nEntry);
+  void ClearEntry(unsigned long nEntry);
+  void Clean();
+  void SetActive(long nEntry);
+
+  bool Get(unsigned long nEntry, const struct PhoneBookEntry  **entry);
+
+private:
+  bool SaveToDisk  (CIniFile &m_fConf);
+  bool LoadFromDisk(CIniFile &m_fConf);
+
+  std::vector<struct PhoneBookEntry> PhoneBookVector;
+
+  friend class ICQUser;
+};
+
 //=====ICQUser==================================================================
 /*! \brief Details about an ICQ user and operations to perform on them.
 
@@ -270,18 +360,22 @@ class ICQUser
 public:
   ICQUser(unsigned long id, char *filename);
   ICQUser(unsigned long id);
-#ifdef PROTOCOL_PLUGIN
   ICQUser(const char *id, unsigned long ppid, char *filename);
-  ICQUser(const char *id, unsigned long ppid);
-#endif
+  ICQUser(const char *id, unsigned long ppid, bool bTempUser = false);
   virtual ~ICQUser();
   void RemoveFiles();
 
   virtual void SaveLicqInfo();
   void SaveGeneralInfo();
   void SaveMoreInfo();
+  void SaveHomepageInfo();
   void SaveWorkInfo();
   void SaveAboutInfo();
+  void SaveInterestsInfo();
+  void SaveBackgroundsInfo();
+  void SaveOrganizationsInfo();
+  void SavePhoneBookInfo();
+  void SavePictureInfo();
   void SaveExtInfo();
   void SaveNewMessagesInfo();
 
@@ -319,6 +413,8 @@ public:
   //!Returns true if the user requires you to be authorized to add
   //!them to anyone's ICQ list.
   bool GetAuthorization()               {  return m_bAuthorization;  }
+  //!Retrieves the users's web status
+  unsigned char GetWebAwareStatus()     {  return m_nWebAwareStatus; }
   //!Returns true if the user has attempted to hide the e-mail addresses
   //!provided in their information.
   bool GetHideEmail()                   {  return m_bHideEmail;  }
@@ -346,6 +442,16 @@ public:
   //!Useful when retrieving their languages in a loop.
   char GetLanguage(unsigned char l)        {  return m_nLanguage[l];  }
 
+  // Homepage Info
+  //!Returns true if the user has entered a homepage category
+  bool GetHomepageCatPresent()             {  return m_bHomepageCatPresent; }
+  //!Retrieves the user's homepage category code
+  unsigned short GetHomepageCatCode()      {  return m_nHomepageCatCode; }
+  //!Retrivies the users homepage description
+  char *GetHomepageDesc()               {  return m_szHomepageDesc; }
+  //!Returns true if the user has an ICQ homepage (http://<uin>.home.icq.com/)
+  bool GetICQHomepagePresent()          {  return m_bICQHomepagePresent; }
+
   // Work Info
   //!Retrieves the city of the company the user is employed by.
   char *GetCompanyCity()                {  return m_szCompanyCity;  }
@@ -368,6 +474,8 @@ public:
   char *GetCompanyDepartment()          {  return m_szCompanyDepartment;  }
   //!Retrieves the user's job title.
   char *GetCompanyPosition()            {  return m_szCompanyPosition;  }
+  //!Retrieves the users's occupation code
+  unsigned short GetCompanyOccupation() {  return m_nCompanyOccupation; }
   //!Retrieves the URL of the company the user is employed by.
   char *GetCompanyHomepage()            {  return m_szCompanyHomepage;  }
 
@@ -375,26 +483,51 @@ public:
   //!Retrieves the self description of the user.
   char *GetAbout()                      { return m_szAbout; }
 
+  // More2 Info
+  //!Retrieves the user's interests
+  ICQUserCategory *GetInterests()       { return m_Interests; }
+  //!Retrieves the user's backgrounds
+  ICQUserCategory *GetBackgrounds()     { return m_Backgrounds; }
+  //!Retrieves the user's organizations
+  ICQUserCategory *GetOrganizations()   { return m_Organizations; }
+
+  // Phone Book Info
+  //!Retrives the user's phone book
+  ICQUserPhoneBook *GetPhoneBook()      { return m_PhoneBook; }
+
+  // Picture Info
+  bool GetPicturePresent()              { return m_bPicturePresent; }
+
   // Licq Info
   bool GetAwaitingAuth()                { return m_bAwaitingAuth; }
   unsigned short GetSID()               { return m_nSID[NORMAL_SID]; }
   unsigned short GetInvisibleSID()      { return m_nSID[INV_SID]; }
   unsigned short GetVisibleSID()        { return m_nSID[VIS_SID]; }
   unsigned short GetGSID()              { return m_nGSID; }
+  unsigned short GetTyping()            { return m_nTyping; }
   //!Retrieves the user's auto response message that was last seen.
   char *AutoResponse()                  { return m_szAutoResponse; }
   //!Retrieves the encoding Licq uses for this user
   char *UserEncoding()                  { return m_szEncoding; }
+  //!True if they have sent the UTF8 Cap
+  bool SupportsUTF8()                   { return m_bSupportsUTF8; }
   bool SendServer()                     { return m_bSendServer; }
   unsigned short SendLevel()            { return m_nSendLevel; }
   bool EnableSave()                     { return m_bEnableSave; }
   bool ShowAwayMsg()                    { return m_bShowAwayMsg; }
   unsigned long Uin()                   { return m_nUin; }
   char *UinString()                     { return m_szUinString; }
-  unsigned long Sequence(bool = false);
+  unsigned short Sequence(bool = false);
   char Mode()                           { return m_nMode; }
   unsigned long Version()               { return m_nVersion; }
+  char *ClientInfo()                    { return m_szClientInfo; }
   unsigned long ClientTimestamp()       { return m_nClientTimestamp; }
+  unsigned long OurClientTimestamp()    { return m_nOurClientTimestamp; }
+  unsigned long ClientInfoTimestamp()   { return m_nClientInfoTimestamp; }
+  unsigned long OurClientInfoTimestamp() { return m_nOurClientInfoTimestamp; }
+  unsigned long ClientStatusTimestamp() { return m_nClientStatusTimestamp; }
+  unsigned long OurClientStatusTimestamp() { return m_nOurClientStatusTimestamp; }
+  bool UserUpdated()                    { return m_bUserUpdated; }
   SecureChannelSupport_et SecureChannelSupport();
   unsigned short LicqVersion();
   unsigned short ConnectionVersion();
@@ -404,6 +537,8 @@ public:
   time_t LastCheckedAutoResponse()      { return m_nLastCounters[LAST_CHECKED_AR]; }
   time_t OnlineSince()      		{ return m_nOnlineSince; }
   time_t IdleSince()                    { return m_nIdleSince; }
+  bool UseGPG()				{ return m_bUseGPG; }
+  char* GPGKey()			{ return m_szGPGKey; }
   bool AutoChatAccept()                 { return m_nAutoAccept & AUTO_ACCEPT_CHAT; }
   bool AutoFileAccept()                 { return m_nAutoAccept & AUTO_ACCEPT_FILE; }
   bool AutoSecure()                     { return m_nAutoAccept & AUTO_SECURE; }
@@ -414,10 +549,9 @@ public:
   unsigned short StatusToUser()         { return m_nStatusToUser; }
   bool KeepAliasOnUpdate()              { return m_bKeepAliasOnUpdate; }
   char *CustomAutoResponse()            { return m_szCustomAutoResponse; }
-#ifdef PROTOCOL_PLUGIN
   unsigned long PPID()                  { return m_nPPID; }
   char *IdString()                      { return m_szId; }
-#endif
+  bool NotInList()                      { return m_bNotInList; }
 
   char *usprintf(const char *szFormat, unsigned long nFlags = 0);
 
@@ -438,6 +572,7 @@ public:
   void SetCountryCode (unsigned short n)     {  m_nCountryCode = n;  SaveGeneralInfo();  }
   void SetTimezone (const char n)            {  m_nTimezone = n;  SaveGeneralInfo();  }
   void SetAuthorization (bool n)             {  m_bAuthorization = n;  SaveGeneralInfo();  }
+  virtual void SetWebAwareStatus (char n)    {  m_nWebAwareStatus = n;  }
   void SetHideEmail (bool n)                 {  m_bHideEmail = n;  SaveGeneralInfo();  }
 
   // More Info
@@ -452,6 +587,12 @@ public:
   void SetLanguage3 (const char n)           {  m_nLanguage[2] = n;  SaveMoreInfo();  }
   void SetLanguage (unsigned char l, char n) {  m_nLanguage[l] = n;  SaveMoreInfo();  }
 
+  // Homepage Info
+  void SetHomepageCatPresent(bool n)         {  m_bHomepageCatPresent = n; SaveHomepageInfo(); }
+  void SetHomepageCatCode(unsigned short n)  {  m_nHomepageCatCode = n; SaveHomepageInfo(); }
+  void SetHomepageDesc(const char *n)        {  SetString(&m_szHomepageDesc, n); SaveHomepageInfo(); }
+  void SetICQHomepagePresent(bool n)         {  m_bICQHomepagePresent = n; SaveHomepageInfo(); }
+
   // Work Info
   void SetCompanyCity (const char *n)        {  SetString(&m_szCompanyCity, n);  SaveWorkInfo();  }
   void SetCompanyState (const char *n)       {  SetString(&m_szCompanyState, n);  SaveWorkInfo();  }
@@ -463,11 +604,15 @@ public:
   void SetCompanyName (const char *n)        {  SetString(&m_szCompanyName, n);  SaveWorkInfo();  }
   void SetCompanyDepartment (const char *n)  {  SetString(&m_szCompanyDepartment, n);  SaveWorkInfo();  }
   void SetCompanyPosition (const char *n)    {  SetString(&m_szCompanyPosition, n);  SaveWorkInfo();  }
+  void SetCompanyOccupation (unsigned short n) {  m_nCompanyOccupation = n;  SaveWorkInfo();  }
   void SetCompanyHomepage (const char *n)    {  SetString(&m_szCompanyHomepage, n);  SaveWorkInfo();  }
 
   // About Info
   void SetAbout(const char *n)        {  SetString(&m_szAbout, n);  SaveAboutInfo();  }
 
+  // Picture info
+  void SetPicturePresent(bool b)      { m_bPicturePresent = b; SavePictureInfo(); }
+
   // Licq Info
   void SetAwaitingAuth(bool b)        { m_bAwaitingAuth = b; }
   void SetSID(unsigned short s)       { m_nSID[NORMAL_SID] = s; }
@@ -477,13 +622,21 @@ public:
   void SetEnableSave(bool s)          { if (m_bOnContactList) m_bEnableSave = s; }
   void SetSendServer(bool s)          { m_bSendServer = s; }
   void SetSendLevel(unsigned short s) { m_nSendLevel = s; }
-  void SetSequence(unsigned long s)   { m_nSequence = s; }
+  void SetSequence(unsigned short s)  { m_nSequence = s; }
   void SetAutoResponse(const char *s) { SetString(&m_szAutoResponse, s); }
   void SetUserEncoding(const char* s) { SetString(&m_szEncoding, s); }
+  void SetSupportsUTF8(bool b)        { m_bSupportsUTF8 = b; }
   void SetShowAwayMsg(bool s)         { m_bShowAwayMsg = s; }
   void SetMode(char s)                { m_nMode = s; }
   void SetVersion(unsigned long s)    { m_nVersion = s; }
+  void SetClientInfo(char *s)         { SetString(&m_szClientInfo, s); }
   void SetClientTimestamp(unsigned long s) { m_nClientTimestamp = s; }
+  void SetOurClientTimestamp(unsigned long s) { m_nOurClientTimestamp = s; }
+  void SetClientInfoTimestamp(unsigned long s) { m_nClientInfoTimestamp = s; }
+  void SetOurClientInfoTimestamp(unsigned long s) { m_nOurClientInfoTimestamp = s; }
+  void SetClientStatusTimestamp(unsigned long s) { m_nClientStatusTimestamp = s; }
+  void SetOurClientStatusTimestamp(unsigned long s) { m_nOurClientStatusTimestamp = s; }
+  void SetUserUpdated(bool s)         { m_bUserUpdated = s; }
   void SetConnectionVersion(unsigned short s)    { m_nConnectionVersion = s; }
   void SetAutoChatAccept(bool s)      { s ? m_nAutoAccept |= AUTO_ACCEPT_CHAT : m_nAutoAccept &= ~AUTO_ACCEPT_CHAT; SaveLicqInfo(); }
   void SetAutoFileAccept(bool s)      { s ? m_nAutoAccept |= AUTO_ACCEPT_FILE : m_nAutoAccept &= ~AUTO_ACCEPT_FILE; SaveLicqInfo(); }
@@ -492,14 +645,19 @@ public:
   void SetAcceptInNA(bool s)          { s ? m_nAutoAccept |= ACCEPT_IN_NA : m_nAutoAccept &= ~ACCEPT_IN_NA; SaveLicqInfo(); }
   void SetAcceptInOccupied(bool s)    { s ? m_nAutoAccept |= ACCEPT_IN_OCCUPIED : m_nAutoAccept &= ~ACCEPT_IN_OCCUPIED; SaveLicqInfo(); }
   void SetAcceptInDND(bool s)         { s ? m_nAutoAccept |= ACCEPT_IN_DND : m_nAutoAccept &= ~ACCEPT_IN_DND; SaveLicqInfo(); }
+  void SetUseGPG(bool b)		{ m_bUseGPG = b; SaveLicqInfo(); }
+  void SetGPGKey(const char *c)		{ SetString(&m_szGPGKey, c); SaveLicqInfo(); }
   void SetStatusToUser(unsigned short s)    { m_nStatusToUser = s; SaveLicqInfo(); }
   void SetKeepAliasOnUpdate(bool b)   { m_bKeepAliasOnUpdate = b; }
   void SetCustomAutoResponse(const char *s) { SetString(&m_szCustomAutoResponse, s); SaveLicqInfo(); }
   void ClearCustomAutoResponse()            { SetCustomAutoResponse(""); }
-#ifdef PROTOCOL_PLUGIN
+  void SetTyping(unsigned short nTyping)    { m_nTyping = nTyping; }
   void SetPPID(unsigned long n)       { m_nPPID = n; }
   void SetId(const char *s)            { SetString(&m_szId, s); SaveLicqInfo(); }
-#endif
+  void SetClientInfo(const char *s)
+  { if(m_szClientInfo) free(m_szClientInfo); 
+    if(s) m_szClientInfo = strdup(s); else m_szClientInfo = NULL; 
+  }
 
   // Status
   unsigned short Status();
@@ -509,13 +667,20 @@ public:
   bool StatusHideIp()          {  return m_nStatus & ICQ_STATUS_FxHIDExIP; }
   bool StatusBirthday()        {  return m_nStatus & ICQ_STATUS_FxBIRTHDAY;  }
   bool StatusOffline()         {  return (unsigned short)m_nStatus == ICQ_STATUS_OFFLINE;  }
+  unsigned long PhoneFollowMeStatus() { return m_nPhoneFollowMeStatus; }
+  unsigned long ICQphoneStatus()      { return m_nICQphoneStatus; }
+  unsigned long SharedFilesStatus()   { return m_nSharedFilesStatus; }
   void SetStatus(unsigned long n)  {  m_nStatus = n;  }
+  void SetPhoneFollowMeStatus(unsigned long n)  { m_nPhoneFollowMeStatus = n; SaveLicqInfo(); }
+  void SetICQphoneStatus(unsigned long n)       { m_nICQphoneStatus = n; }
+  void SetSharedFilesStatus(unsigned long n)    { m_nSharedFilesStatus = n; }  
   virtual void SetStatusOffline();
   const char *StatusStr();
   const char *StatusStrShort();
   bool Away();
   static const char *StatusToStatusStr(unsigned short, bool);
   static const char *StatusToStatusStrShort(unsigned short, bool);
+  static char *MakeRealId(const char *, unsigned long, char *&);
   int Birthday(unsigned short nDayRange = 0);
 
   // Message/History functions
@@ -588,8 +753,9 @@ public:
   char *PortStr(char *rbuf);
   
   // Don't call these:
-  int SocketDesc()          { return m_nSocketDesc; }
+  int SocketDesc(unsigned char);
   void ClearSocketDesc();
+  void ClearSocketDesc(unsigned char);
   void SetSocketDesc(TCPSocket *);
 
   // Events functions
@@ -603,17 +769,23 @@ public:
 
   // Crypto
   bool Secure() { return m_bSecure; }
-
+    
   virtual bool User()  { return true; }
   void Lock(unsigned short);
   void Unlock();
 
 protected:
-  ICQUser() { /* ICQOwner inherited constructor - does nothing */ };
+  ICQUser() { /* ICQOwner inherited constructor - does nothing */ }
   void LoadGeneralInfo();
   void LoadMoreInfo();
+  void LoadHomepageInfo();
   void LoadWorkInfo();
   void LoadAboutInfo();
+  void LoadInterestsInfo();
+  void LoadBackgroundsInfo();
+  void LoadOrganizationsInfo();
+  void LoadPhoneBookInfo();
+  void LoadPictureInfo();
   void LoadLicqInfo();
   void Init(unsigned long nUin);
   void Init(const char *, unsigned long);
@@ -621,11 +793,11 @@ protected:
   void SetDefaults();
   void AddToContactList();
 
+  void SetSecure(bool s) { m_bSecure = s; }
   void SetOfflineOnDisconnect(bool b) { m_bOfflineOnDisconnect = b; }
   bool OfflineOnDisconnect() { return m_bOfflineOnDisconnect; }
   bool ConnectionInProgress() { return m_bConnectionInProgress; }
   void SetConnectionInProgress(bool c)  { m_bConnectionInProgress = c; }
-  void SetSecure(bool s) { m_bSecure = s; }
 
   void SetLastSentEvent()           { m_nLastCounters[LAST_SENT_EVENT] = time(NULL); }
   void SetLastReceivedEvent()       { m_nLastCounters[LAST_RECV_EVENT] = time(NULL); }
@@ -636,21 +808,29 @@ protected:
 
   CIniFile m_fConf;
   CUserHistory m_fHistory;
-  int m_nSocketDesc;
+  int m_nNormalSocketDesc, m_nInfoSocketDesc, m_nStatusSocketDesc;
   time_t m_nTouched;
   time_t m_nLastCounters[4];
   time_t m_nOnlineSince;
   time_t m_nIdleSince;
   bool m_bOnContactList;
   unsigned long m_nIp, m_nIntIp, m_nVersion, m_nClientTimestamp, m_nCookie;
+  unsigned long m_nClientInfoTimestamp, m_nClientStatusTimestamp;
+  unsigned long m_nOurClientTimestamp, m_nOurClientInfoTimestamp;
+  unsigned long m_nOurClientStatusTimestamp;
+  bool m_bUserUpdated;
   unsigned short m_nPort, m_nLocalPort, m_nConnectionVersion;
+  unsigned short m_nTyping;
   unsigned long m_nUin,
                 m_nStatus,
-                m_nSequence,
                 m_nGroups[2];
+  unsigned short m_nSequence;
+  unsigned long m_nPhoneFollowMeStatus, m_nICQphoneStatus, m_nSharedFilesStatus;
   char m_nMode;
+  char *m_szClientInfo;
   char *m_szAutoResponse;
   char *m_szEncoding;
+  bool m_bSupportsUTF8;
   char *m_szCustomAutoResponse;
   char *m_szId;
   char m_szUinString[13];
@@ -661,11 +841,16 @@ protected:
        m_bShowAwayMsg,
        m_bOfflineOnDisconnect,
        m_bConnectionInProgress,
-       m_bSecure;
+       m_bSecure,
+       m_bNotInList;
   unsigned short m_nStatusToUser, m_nSendLevel;
   bool m_bKeepAliasOnUpdate;
   unsigned short m_nAutoAccept;
 
+  // GPG data
+  bool m_bUseGPG;
+  char *m_szGPGKey;
+
   // General Info
   char *m_szAlias;
   char *m_szFirstName;
@@ -684,11 +869,16 @@ protected:
   char m_nTimezone;
   bool m_bAuthorization;
   bool m_bHideEmail;
-
+  unsigned char m_nWebAwareStatus;
+  
   // More Info
   unsigned short m_nAge;
   char m_nGender;
   char *m_szHomepage;
+  bool m_bHomepageCatPresent;
+  unsigned short m_nHomepageCatCode;
+  char *m_szHomepageDesc;
+  bool m_bICQHomepagePresent;
   unsigned short m_nBirthYear;
   char m_nBirthMonth;
   char m_nBirthDay;
@@ -705,15 +895,25 @@ protected:
   char *m_szCompanyName;
   char *m_szCompanyDepartment;
   char *m_szCompanyPosition;
+  unsigned short m_nCompanyOccupation;
   char *m_szCompanyHomepage;
 
   // About Info
   char *m_szAbout;
+  
+  // More2 Info
+  ICQUserCategory *m_Interests;
+  ICQUserCategory *m_Backgrounds;
+  ICQUserCategory *m_Organizations;
+
+  // Phone Book Info
+  ICQUserPhoneBook *m_PhoneBook;
+
+  // Picture Info
+  bool m_bPicturePresent;
 
-#ifdef PROTOCOL_PLUGIN
   // Protocol ID
   unsigned long m_nPPID;
-#endif
 
   // Server Side ID, Group SID
   bool m_bAwaitingAuth;
@@ -731,7 +931,9 @@ protected:
   friend class CUserGroup;
   friend class CUserManager;
   friend class CICQDaemon;
+  friend class CMSN;
   friend class CSocketManager;
+  friend void *MonitorSockets_tep(void *);
 };
 
 
@@ -740,9 +942,7 @@ class ICQOwner : public ICQUser
 {
 public:
   ICQOwner();
-#ifdef PROTOCOL_PLUGIN
   ICQOwner(const char *, unsigned long);
-#endif
   virtual ~ICQOwner();
   bool Exception()  { return m_bException; }
 
@@ -750,9 +950,11 @@ public:
   char *Password() { return(m_szPassword); }
   void SetPassword(const char *s) { SetString(&m_szPassword, s); SaveLicqInfo(); }
   void SetWebAware(bool b)     {  m_bWebAware = b; SaveLicqInfo(); }
+  virtual void SetWebAwareStatus(unsigned char c) { SetWebAware(c); }
   void SetHideIp(bool b)       {  m_bHideIp = b; SaveLicqInfo(); }
   void SetSavePassword(bool b) {  m_bSavePassword = b; SaveLicqInfo(); }
-  void SetUin(unsigned long n)    { m_nUin = n; SaveLicqInfo(); }
+  void SetUin(unsigned long n) { m_nUin = n; SaveLicqInfo(); }
+  void SetId(const char *s)    { SetString(&m_szId, s); SaveLicqInfo(); }
   void SetRandomChatGroup(unsigned long n)  { m_nRandomChatGroup = n; SaveLicqInfo(); }
   bool WebAware()             { return m_bWebAware; }
   bool HideIp()               { return m_bHideIp; }
@@ -765,7 +967,11 @@ public:
   void SetSSTime(time_t t)            { m_nSSTime = t; }
   unsigned short GetSSCount()         { return m_nSSCount; }
   void SetSSCount(unsigned short n)   { m_nSSCount = n; }
+  unsigned short GetPDINFO()          { return m_nPDINFO; }
+  void SetPDINFO(unsigned short n)    { m_nPDINFO = n; SaveLicqInfo(); }
 
+  void SetPicture(const char *f);
+  
   // Virtual overloaded functions
   virtual void SaveLicqInfo();
   virtual void SetStatusOffline();
@@ -779,7 +985,8 @@ protected:
        m_bSavePassword;
   unsigned long  m_nRandomChatGroup;
   unsigned short m_nSSCount;
-  time_t         m_nSSTime;
+  time_t unsigned long m_nSSTime;
+  unsigned short m_nPDINFO;
 };
 
 
@@ -789,23 +996,19 @@ class CUserHashTable
 {
 public:
   CUserHashTable(unsigned short _nSize);
-
-#ifdef PROTOCOL_PLUGIN
+  
   // For protocol plugins
   ICQUser *Retrieve(const char *, unsigned long);
 	void Store(ICQUser *, const char *, unsigned long);
   void Remove(const char *, unsigned long);
-#endif
 
   // For ICQ (Licq way)
   ICQUser *Retrieve(unsigned long _nUin);
   void Store(ICQUser *u, unsigned long _nUin);
   void Remove(unsigned long _nUin);
 protected:
-#ifdef PROTOCOL_PLUGIN
 	// For protocol plugin
   unsigned short HashValue(const char *);
-#endif
 
   // For ICQ (Licq way)
   unsigned short HashValue(unsigned long _nUin);
@@ -825,19 +1028,17 @@ public:
   ~CUserManager();
   bool Load();
   void SetOwnerUin(unsigned long _nUin);
-
-#ifdef PROTOCOL_PLUGIN
+  
   // For protocol plugins
   void AddOwner(const char *, unsigned long);
   void AddUser(ICQUser *, const char *, unsigned long);
   void RemoveUser(const char *, unsigned long);
+  void RemoveOwner(unsigned long);
   ICQUser *FetchUser(const char *, unsigned long, unsigned short);
-  //void DropUser(ICQUser *);
   ICQOwner *FetchOwner(unsigned long, unsigned short);
   void DropOwner(unsigned long);
   bool IsOnList(const char *, unsigned long);
   ICQOwner *FindOwner(const char *, unsigned long);
-#endif
 
   // ICQ Protocol only (from original Licq)
   unsigned long AddUser(ICQUser *);
@@ -855,10 +1056,8 @@ public:
   void UnlockGroupList();
   GroupIDList *LockGroupIDList(unsigned short);
   void UnlockGroupIDList();
-#ifdef PROTOCOL_PLUGIN
   OwnerList *LockOwnerList(unsigned short);
   void UnlockOwnerList();
-#endif
 
   bool AddGroup(char *, unsigned short = 0);
   void RemoveGroup(unsigned short);
@@ -878,39 +1077,32 @@ public:
 
   void AddUserToGroup(unsigned long _nUin, unsigned short _nGroup);
   void RemoveUserFromGroup(unsigned long _nUin, unsigned short _nGroup);
+  void AddUserToGroup(const char *, unsigned long, unsigned short);
+  void RemoveUserFromGroup(const char *, unsigned long, unsigned short);
   void SaveAllUsers();
 
+  bool UpdateUsersInGroups();
+  
   unsigned short NumUsers();
+  unsigned short NumOwners();
   unsigned short DefaultGroup()  { return m_nDefaultGroup; }
   void SetDefaultGroup(unsigned short n)  { m_nDefaultGroup = n; SaveGroups(); }
   unsigned short NewUserGroup()  { return m_nNewUserGroup; }
   void SetNewUserGroup(unsigned short n)  { m_nNewUserGroup = n; SaveGroups(); }
 
 protected:
-  pthread_rdwr_t mutex_grouplist, mutex_userlist, mutex_groupidlist
-#ifdef PROTOCOL_PLUGIN
-                 , mutex_ownerlist;
-#else
-  ;
-#endif
+  pthread_rdwr_t mutex_grouplist, mutex_userlist, mutex_groupidlist, mutex_ownerlist;
 
   GroupList m_vszGroups;
   UserList m_vpcUsers;
-#ifdef PROTOCOL_PLUGIN
   OwnerList m_vpcOwners;
-#endif
   GroupIDList m_vnGroupsID;
   CUserHashTable m_hUsers;
   ICQOwner *m_xOwner;
   unsigned long m_nOwnerUin;
   unsigned short m_nDefaultGroup, m_nNewUserGroup,
                  m_nUserListLockType, m_nGroupListLockType,
-                 m_nGroupIDListLockType
-#ifdef PROTOCOL_PLUGIN
-  , m_nOwnerListLockType;
-#else
-  ;
-#endif
+                 m_nGroupIDListLockType, m_nOwnerListLockType;
   bool m_bAllowSave;
 
   friend class CICQDaemon;
