@@ -26,7 +26,8 @@ protected:
   bool m_bExit, m_bEnabled, m_bDelete;
   char *m_szStatus;
   char m_szProgram[512], m_szArguments[512];
-  bool m_bPassMessage, m_bFailOnExitCode, m_bAbortDeleteOnExitCode;
+  bool m_bPassMessage, m_bFailOnExitCode, m_bAbortDeleteOnExitCode,
+       m_bSendThroughServer;
 
   CICQDaemon *licqDaemon;
   TCPSocket *tcp;
@@ -36,8 +37,8 @@ public:
   void ProcessSignal(CICQSignal *);
   void ProcessEvent(ICQEvent *);
 
-  void ProcessUserEvent(unsigned long, unsigned long);
-  bool AutoReplyEvent(unsigned long, CUserEvent *);
+  void ProcessUserEvent(const char *, unsigned long, unsigned long);
+  bool AutoReplyEvent(const char *, unsigned long, CUserEvent *);
 
   bool POpen(const char *cmd);
   int PClose();
