@@ -125,7 +125,7 @@ void CLicqConsole::PrintPrompt()
 void CLicqConsole::PrintStatus()
 {
   static char szMsgStr[16];
-  static char szLastUser[32];
+  static char *szLastUser;
 
   werase(winStatus->Win());
 
@@ -150,15 +150,15 @@ void CLicqConsole::PrintStatus()
     ICQUser *u = gUserManager.FetchUser(winMain->sLastContact.szId,
       winMain->sLastContact.nPPID, LOCK_R);
     if (u == NULL)
-      strcpy(szLastUser, "<Removed>");
+      szLastUser = strdup("<Removed>");
     else
     {
-      strcpy(szLastUser, u->GetAlias());
+      szLastUser = strdup(u->GetAlias());
       gUserManager.DropUser(u);
     }
   }
   else
-    strcpy(szLastUser, "<None>");
+    szLastUser = strdup("<None>");
 
   o = gUserManager.FetchOwner(LOCK_R);
   wbkgdset(winStatus->Win(), COLOR_PAIR(8));
@@ -180,6 +180,7 @@ void CLicqConsole::PrintStatus()
   
   wclrtoeol(winStatus->Win());
   winStatus->RefreshWin();
+  free(szLastUser);
 }
 
 
@@ -334,16 +335,16 @@ void CLicqConsole::CreateUserList()
       s->color = m_cColorNew;
 
     // Create the line to printout now
-	if (pUser->NewMessages() > 0)
+    if (pUser->NewMessages() > 0)
     {
       s->szLine = new char[strlen(szTmp) + 19];
       snprintf(s->szLine, strlen(szTmp) + 19, "</%d></K>%s<!K><!%d>", s->color->nColor - 6, szTmp ? szTmp : "", s->color->nColor - 6);
       s->szLine[strlen(szTmp) + 18] = '\0';
-	} else {
+    } else {
       s->szLine = new char[strlen(szTmp) + 11];
-	  snprintf(s->szLine, strlen(szTmp) + 11, "</%d>%s<!%d>", s->color->nColor, szTmp ? szTmp : "", s->color->nColor);
+      snprintf(s->szLine, strlen(szTmp) + 11, "</%d>%s<!%d>", s->color->nColor, szTmp ? szTmp : "", s->color->nColor);
       s->szLine[strlen(szTmp) + 10] = '\0';
-	}
+    }
     free(szTmp);
     
     // Insert into the list
@@ -372,7 +373,7 @@ void CLicqConsole::CreateUserList()
 void CLicqConsole::PrintUsers()
 {
   char *title = "<C></B/40>Contacts";
-  char *ulist[10000];
+  char *ulist[1024];
   int i = 0;
   struct SScrollUser *s = NULL;
   
@@ -383,11 +384,11 @@ void CLicqConsole::PrintUsers()
        it++)
   {
     s = new SScrollUser;
-	s->pos = i;
-	s->nPPID = (*it)->nPPID;
-    s->color = (*it)->color;
-	sprintf(s->szId, "%s", (*it)->szId);
-	
+    s->pos = i;
+    s->nPPID = (*it)->nPPID;
+        s->color = (*it)->color;
+    sprintf(s->szId, "%s", (*it)->szId);
+
     m_lScrollUsers.push_back(s);
     ulist[i++] = copyChar((*it)->szLine);
   }
@@ -415,7 +416,8 @@ void CLicqConsole::PrintUsers()
 /*---------------------------------------------------------------------------
  * CLicqConsole::UserListHighlight
  *-------------------------------------------------------------------------*/
-void CLicqConsole::UserListHighlight(chtype type, chtype input) {
+void CLicqConsole::UserListHighlight(chtype type, chtype input)
+{
   // There has got to be a better way to do this...
   list <SScrollUser *>::iterator it;
   int down;
@@ -427,10 +429,10 @@ void CLicqConsole::UserListHighlight(chtype type, chtype input) {
     down = 0;
   for (it = m_lScrollUsers.begin(); it != m_lScrollUsers.end(); it++)
   {
-    if ((*it)->pos == cdkUserList->currentItem + down)
+    if ((*it)->pos == (cdkUserList->currentItem + down))
     {
       ICQUser *u = gUserManager.FetchUser((*it)->szId, (*it)->nPPID, LOCK_R);
-      if (u->NewMessages())
+      if (u && u->NewMessages())
         setCDKScrollHighlight(cdkUserList, COLOR_PAIR((*it)->color->nColor - 6) | type);
       else
         setCDKScrollHighlight(cdkUserList, COLOR_PAIR((*it)->color->nColor) | type);
