@@ -2,7 +2,9 @@
 #define WINDOW_H
 
 #include <iostream>
-#include <curses.h>
+extern "C" {
+#include <cdk.h>
+}
 
 #define COLOR_YELLOW_BLUE COLOR_YELLOW + 8
 #define COLOR_WHITE_BLUE  COLOR_WHITE + 8
@@ -15,10 +17,17 @@ enum InputState { STATE_COMMAND, STATE_PENDING, STATE_MLE, STATE_LE, STATE_QUERY
 class CLicqConsole;
 class CData;
 
+struct SContact
+{
+  char *szId;
+  unsigned long nPPID;
+};
+
+
 class CWindow
 {
 public:
-  CWindow(int _rows, int _cols, int _y, int _x, int _scrollback);
+  CWindow(int _rows, int _cols, int _y, int _x, int _scrollback, int _useCDK = false);
   ~CWindow();
   void RefreshWin();
   void ScrollUp();
@@ -32,6 +41,7 @@ public:
   CWindow& operator<<(unsigned short d);
   void wprintf(char *formatIn, ...);
   WINDOW *Win()  { return win; }
+  CDKSCREEN *CDKScreen() { return cdkscreen; } 
   static void StartScreen();
   static void EndScreen();
   int Rows() { return rows; }
@@ -42,9 +52,11 @@ public:
   InputState state;
   CData *data;
   unsigned long nLastUin;
+  struct SContact sLastContact;
   unsigned short nLastHistory;
 protected:
   WINDOW *win;
+  CDKSCREEN *cdkscreen;
   bool pad, active;
   int rows, cols, x, y, height, cur_y;
 };
