@@ -20,6 +20,14 @@
 
 #include "my_xosd.h"
 #include "licq_osd.conf.h"
+                                 
+//#if CVSLICQ==1
+//#warning compiling for licq>=1.2.8
+//#else
+//#undef CVSLICQ
+//#warning compiling for licq<1.2.8
+//#endif
+
 #define _(String) gettext (String)
 
 // if you don't want to use codepage translation, comment out this
@@ -29,8 +37,8 @@
 #define L_OSD_STR "[OSD] "
 
 // licq does not define an invisible status constant
-// so I chose one which wont be taken most likely
-// cf licq sources: include/licq_icq.h
+// so I chose one which won't be taken most likely
+// by licq sources: include/licq_icq.h
 #define OSD_STATUS_INVISIBLE 0x8000
 // licq defines ICQ_STATUS_ONLINE as 0, that's not andable so we use our own
 #define OSD_STATUS_ONLINE 0x4000
@@ -43,7 +51,8 @@ struct Config {
     unsigned long quiettimeout; //=QUIETTIMEOUT;
     string pluginfont;
     unsigned long ShowInModes;
-	string colour;
+    unsigned long ShowMsgsInModes;
+    string colour;
     string controlcolour;
     bool osd_wait;
     unsigned long timeout;
@@ -57,7 +66,7 @@ struct Config {
     string hpos;
     string shadowcolour;
     string outlinecolour;
-	string localencoding;
+    string localencoding;
     bool marksecuremessages;
 } config;
 
@@ -205,6 +214,7 @@ unsigned long parseShowInModesStr(char *ShowInModesStr)
 bool LP_Init(int argc, char **argv)
 {
     char ShowInModesStr[MAX_LINE_LEN+1];
+    char ShowMsgsInModesStr[MAX_LINE_LEN+1];
     char temp[MAX_LINE_LEN+1];
     string filename;
     try {
@@ -283,15 +293,17 @@ bool LP_Init(int argc, char **argv)
 
 	conf.ReadStr("ShowInModes", ShowInModesStr, SHOWINMODESSTR);
         ShowInModesStr[MAX_LINE_LEN]=0;
+	conf.ReadStr("ShowMsgsInModes", ShowMsgsInModesStr, SHOWMSGSINMODESSTR);
+        ShowMsgsInModesStr[MAX_LINE_LEN]=0;
 
 	conf.CloseFile();
 
 	config.ShowInModes=parseShowInModesStr(ShowInModesStr);
+	config.ShowMsgsInModes=parseShowInModesStr(ShowMsgsInModesStr);
 
 	setlocale(LC_ALL, "");
 
 	config.localencoding=nl_langinfo(CODESET);
-	//    setlocale (LC_ALL, "");
 	bindtextdomain (PACKAGE, LOCALEDIR);
 	bind_textdomain_codeset(PACKAGE, config.localencoding.c_str());
 	textdomain (PACKAGE);
@@ -300,15 +312,10 @@ bool LP_Init(int argc, char **argv)
 		     config.colour, config.osd_wait, config.Showmessages, config.Showlogon, config.shadowoffset, config.outlineoffset, config.shadowcolour,
 		     config.outlinecolour, config.localencoding);
 
-	//    bindtextdomain(PACKAGE, LOCALEDIR);
-	//    textdomain(PACKAGE);
-	gLog.Info("%sOSD plugin has been initialized correctly\n", L_OSD_STR);
 	return 1;
     }
     catch (...)
-	{
-	gLog.Error("%sException while initializing plugin\n", L_OSD_STR);
-
+    {
 	return 0;
     }
 }
@@ -322,7 +329,6 @@ int LP_Main(CICQDaemon *_licqDaemon)
     bool Exit=false; // exit plugin?
     char buf[16];
 
-	gLog.Info("%sOSD plugin entering LP_Main loop\n", L_OSD_STR);
     if (nPipe==-1)
     {
 	gLog.Warn("%sInvalid Pipe received\n", L_ERRORxSTR);
@@ -338,10 +344,8 @@ int LP_Main(CICQDaemon *_licqDaemon)
 	read(nPipe, buf, 1); // Information about a new signal is sent through pipe
 	if (!Configured)
 	{
-	gLog.Info("%sOSD plugin calling my_xosd_init\n", L_OSD_STR);
 	    if (!my_xosd_init(config.pluginfont, config.colour, config.hoffset, config.voffset, config.vpos, config.hpos, config.timeout, config.DelayPerCharacter, config.lines, config.linelen, config.osd_wait, config.shadowoffset, config.outlineoffset, config.shadowcolour, config.outlinecolour))
 		return 0;
-	gLog.Info("%sOSD plugin called my_xosd_init successfully --> Configured is true\n", L_OSD_STR);
 	    Configured=true;
 	}
 
@@ -399,12 +403,10 @@ int LP_Main(CICQDaemon *_licqDaemon)
 
     if (Configured)
     {
-	gLog.Info("%sOSD plugin calling my_xosd_exit\n", L_OSD_STR);
 	my_xosd_exit();
         Configured=false;
     }
     // unregister the plugin
-	gLog.Info("%sOSD plugin leaving LP_Main loop\n", L_OSD_STR);
     _licqDaemon->UnregisterPlugin();
 
     return 0;
@@ -417,7 +419,8 @@ void ProcessSignal(CICQSignal *s)
     bool notify=false;
     bool ignore=false;
     bool invisible=false;
-	bool want_osd=true; // if we are in DND,... we maybe dont want OSD
+    bool want_osd=true; // if we are in DND,... we maybe dont want OSD
+    bool want_osd_msgs_only=false; // though we don't want OSD we want msgs
     bool secure=false;
     unsigned long status=0;
     char *userencoding=0;
@@ -469,15 +472,40 @@ void ProcessSignal(CICQSignal *s)
 		    else if ((o->StatusInvisible()) && (!(config.ShowInModes&OSD_STATUS_INVISIBLE))) // is reached when a user gets invisible
 			want_osd=false;
 
+		    if (!want_osd) {
+			if ((status&ICQ_STATUS_DND) && (config.ShowMsgsInModes&ICQ_STATUS_DND))
+			    want_osd_msgs_only=true;
+			else if ((status&ICQ_STATUS_OCCUPIED) && (config.ShowMsgsInModes&ICQ_STATUS_OCCUPIED))
+			    want_osd_msgs_only=true;
+			else if ((status&ICQ_STATUS_NA) && (config.ShowMsgsInModes&ICQ_STATUS_NA))
+			    want_osd_msgs_only=true;
+			else if ((status&ICQ_STATUS_AWAY) && (config.ShowMsgsInModes&ICQ_STATUS_AWAY))
+			    want_osd_msgs_only=true;
+			else if ((status==0) && (config.ShowMsgsInModes&OSD_STATUS_ONLINE))
+			    want_osd_msgs_only=true;
+			else if ((status&ICQ_STATUS_FREEFORCHAT) && (config.ShowMsgsInModes&ICQ_STATUS_FREEFORCHAT))
+			    want_osd_msgs_only=true;
+			else if ((o->StatusInvisible()) && (config.ShowMsgsInModes&OSD_STATUS_INVISIBLE)) // is reached when a user gets invisible
+			    want_osd_msgs_only=true;
+
+			if (want_osd_msgs_only) {
+			    want_osd=true;
+			}
+		    }
+
 		    gUserManager.DropOwner();
 		}
 	    }
 
 	    if (want_osd)
 	    {
+//#ifdef CVSLICQ
 		char *szId = s->Id();
 		unsigned long nPPID = s->PPID();
 		u = gUserManager.FetchUser(szId, nPPID, LOCK_R);
+//#else
+//		u = gUserManager.FetchUser(s->Uin(), LOCK_R);
+//#endif
 		if (u)
 		{
 		    // user alias as displayed in licq -
@@ -531,7 +559,7 @@ void ProcessSignal(CICQSignal *s)
 	    // this is some evil functionality - most users dont know
 	    // that you realize when they check your auto-response
 	    // i implemented this just for fun :)
-	    if ((want_osd) && // not ignored or disabled
+	    if ((want_osd) && (!want_osd_msgs_only) && // not ignored or disabled
 		(s->SubSignal() == USER_EVENTS) &&
 		(s->Argument() == 0) // auto response
 	       )
@@ -542,9 +570,8 @@ void ProcessSignal(CICQSignal *s)
 		   )
 		{
 		    string msg(username);
-			msg+=_(" checked your auto-response");
-            my_xosd_setcolour(config.controlcolour);
-		    my_xosd_display("autoresponse", msg);
+		    msg+=_(" checked your auto-response");
+		    my_xosd_display("autoresponse", msg, config.controlcolour);
 		}
 	    }
 
@@ -560,14 +587,11 @@ void ProcessSignal(CICQSignal *s)
 		   )
 		{
 		    char *translated=my_translate(s->Uin(), e->Text(), userencoding);
-			my_xosd_setcolour(config.colour);
-			string msg="";
-			if (secure && config.marksecuremessages)
-				msg="(S) ";
-			msg+=translated;
-            gLog.Info("%sOSD plugin calling my_xosd_display for a message\n", L_OSD_STR);
-
-		    my_xosd_display(username.c_str(), msg.c_str());
+		    string msg="";
+		    if (secure && config.marksecuremessages)
+			msg="(S) ";
+		    msg+=translated;
+		    my_xosd_display(username.c_str(), msg.c_str(), config.colour);
 		    delete[] translated;
 		}
 		if (
@@ -577,12 +601,10 @@ void ProcessSignal(CICQSignal *s)
 		{
 		    string msg=_("Message from ");
 		    msg += username;
-            my_xosd_setcolour(config.colour);
-            gLog.Info("%sOSD plugin calling my_xosd_display for a message just telling about the message\n", L_OSD_STR);
- 		    my_xosd_display("", msg);
+		    my_xosd_display("", msg, config.colour);
 		}
 	    }
-	    if (want_osd && (s->SubSignal() == USER_STATUS)) // logon/logoff or status change
+	    if (want_osd && (!want_osd_msgs_only) && (s->SubSignal() == USER_STATUS)) // logon/logoff or status change
 	    {
 		string msg = username;
 		if (s->Argument()!=0) // logon/logoff
@@ -597,9 +619,7 @@ void ProcessSignal(CICQSignal *s)
 			else
 			    msg+=_(" logged off");
 //			my_xosd_settimeout(config.timeout);
-            my_xosd_setcolour(config.controlcolour);
-            gLog.Info("%sOSD plugin calling my_xosd_display for logon/off\n", L_OSD_STR);
-			my_xosd_display("", msg);
+			my_xosd_display("", msg, config.controlcolour);
 		    }
 		}
 		else // status change
@@ -630,9 +650,7 @@ void ProcessSignal(CICQSignal *s)
 			else // shouldnt be reached
 			    msg+=_("unknown");
 //			my_xosd_settimeout(config.timeout);
-            my_xosd_setcolour(config.controlcolour);
-            gLog.Info("%sOSD plugin calling my_xosd_display for status change\n", L_OSD_STR);
-			my_xosd_display("", msg);
+			my_xosd_display("", msg, config.controlcolour);
 		    }
 		}
 
