@@ -9,6 +9,9 @@
 #include <qbitmap.h>
 #include <qimage.h>
 #include <qtextcodec.h>
+#include <qtabwidget.h>
+#include <qtabbar.h>
+#include <qcursor.h>
 #ifdef USE_KDE
 #include <kapp.h>
 #include <kmessagebox.h>
@@ -19,6 +22,8 @@
 #include <qapplication.h>
 #include <qstyle.h>
 #include <qdatetime.h>
+#include <ctype.h>
+
 #include "licq_history.h"
 #include "licq_events.h"
 #include "mainwin.h"
@@ -29,17 +34,48 @@
 
 using namespace std;
 
-bool QueryUser(QWidget *q, QString szQuery, QString szBtn1, QString szBtn2)
+/*! \brief Dialog with configurable yes/no buttons
+ *
+ * In it's easiest form, this dialog displays szQuery and the buttons szBtn1 and szBtn2. 
+ * szBtn1 means true/yes, 
+ * szBtn2 means false/no. 
+ * 
+ * When bConfirmYes is true, then a second dialog asks the user to confirm his positive 
+ * decision, the displayed confirmation message is passed using the QString szConfirmYes.
+ *
+ * When bConfirmNo is true, then a second dialog asks the user to confirm his negative 
+ * decision, the displayed confirmation message is passed using the QString szConfirmNo.
+ */
+bool QueryUser(QWidget *q, QString szQuery, QString szBtn1, QString szBtn2, bool bConfirmYes, QString szConfirmYes, bool bConfirmNo, QString szConfirmNo)
 {
+  bool result;
+
 #ifdef USE_KDE
 #if KDE_VERSION >= 290
-  return ( KMessageBox::questionYesNo(q, szQuery, QMessageBox::tr("Licq Question"), szBtn1, szBtn2, QString::null, false) == KMessageBox::Yes);
+  result = ( KMessageBox::questionYesNo(q, szQuery, QMessageBox::tr("Licq Question"), szBtn1, szBtn2, QString::null, false) == KMessageBox::Yes);
+  // The user must confirm his decision!
+  if(result == true && bConfirmYes && szConfirmYes)
+    result = ( KMessageBox::questionYesNo(q, szConfirmYes, QMessageBox::tr("Licq Question"), QMessageBox::tr("Yes"), QMessageBox::tr("No"), QString::null, false) == KMessageBox::Yes);
+  else if(result == false && bConfirmNo && szConfirmNo)
+    result = ( KMessageBox::questionYesNo(q, szConfirmNo, QMessageBox::tr("Licq Question"), QMessageBox::tr("Yes"), QMessageBox::tr("No"), QString::null, false) == KMessageBox::Yes);
 #else
-  return ( KMessageBox::questionYesNo(q, szQuery, QMessageBox::tr("Licq Question"), szBtn1, szBtn2, false) == KMessageBox::Yes);
+  result = ( KMessageBox::questionYesNo(q, szQuery, QMessageBox::tr("Licq Question"), szBtn1, szBtn2, false) == KMessageBox::Yes);
+  // The user must confirm his decision!
+  if(result == true && bConfirmYes && szConfirmYes)
+    result = ( KMessageBox::questionYesNo(q, szConfirmYes, QMessageBox::tr("Licq Question"), QMessageBox::tr("Yes"), QMessageBox::tr("No"), false) == KMessageBox::Yes);
+  else if(result == false && bConfirmNo && szConfirmNo)
+    result = ( KMessageBox::questionYesNo(q, szConfirmNo, QMessageBox::tr("Licq Question"), QMessageBox::tr("Yes"), QMessageBox::tr("No"), false) == KMessageBox::Yes);
 #endif
 #else
-  return ( QMessageBox::information(q, QMessageBox::tr("Licq Question"), szQuery, szBtn1, szBtn2) == 0);
+  result = ( QMessageBox::information(q, QMessageBox::tr("Licq Question"), szQuery, szBtn1, szBtn2) == 0);
+  // The user must confirm his decision!
+  if(result == true && bConfirmYes && szConfirmYes)
+    result = ( QMessageBox::information(q, QMessageBox::tr("Licq Question"), szConfirmYes, QMessageBox::tr("Yes"), QMessageBox::tr("No")) == 0);
+  else if(result == false && bConfirmNo && szConfirmNo)
+    result = ( QMessageBox::information(q, QMessageBox::tr("Licq Question"), szConfirmNo, QMessageBox::tr("Yes"), QMessageBox::tr("No")) == 0);
 #endif
+  
+  return result;
 }
 
 
@@ -125,6 +161,22 @@ void CELabel::clearPrependPixmap()
   update();
 }
 
+void CELabel::addPixmap(const QPixmap &p)
+{
+  m_lPixmaps.push_back(p);
+  if (m_lPixmaps.size() == 1)
+    startingIndent = indent();
+  update();
+}
+
+void CELabel::clearPixmaps()
+{
+  if (m_lPixmaps.size() == 0) return;
+  m_lPixmaps.clear();
+  setIndent(startingIndent);
+  update();
+}
+
 void CELabel::setBold(bool isBold)
 {
    QFont newFont(font());
@@ -194,6 +246,17 @@ void CELabel::drawContents(QPainter* p)
   if(!addPix.isNull())
     p->drawPixmap(addIndent, height() / 2 - addPix.height() / 2, addPix);
 
+  if (m_lPixmaps.size())
+  {
+    list<QPixmap>::iterator it;
+    int i = 2;
+    for (it = m_lPixmaps.begin(); it != m_lPixmaps.end(); it++)
+    {
+      p->drawPixmap(i, height() / 2 - it->height() / 2, *it);
+      i += it->width() + 2;
+    }
+  }
+
   QLabel::drawContents(p);
 }
 
@@ -410,6 +473,102 @@ void CEComboBox::setNamedBgColor(char *theColor)
    setPalette(QPalette(newNormal, pal.disabled(), newNormal));
 }
 
+//CETabBar
+void CETabBar::setTabColor(int id, const QColor &color)
+{
+  QTab *t = tab(id);
+  if (t)
+  {
+    mTabColors.insert(id, color);
+    repaint(t->rect(), false);
+  }
+}
+
+const QColor &CETabBar::tabColor(int id) const
+{
+  if (mTabColors.contains(id))
+    return mTabColors[id];
+    
+  return colorGroup().foreground();
+}
+void CETabBar::removeTab(QTab *t)
+{
+  mTabColors.remove(t->identifier());
+  QTabBar::removeTab(t);
+}
+
+void CETabBar::paintLabel(QPainter* p, const QRect &br,
+                          QTab* t, bool has_focus) const
+{
+  QRect r = br;
+  bool selected = currentTab() == t->identifier();
+  if (t->iconSet())
+  {
+    // the tab has an iconset, draw it in the right mode
+    QIconSet::Mode mode = (t->isEnabled() && isEnabled())
+         ? QIconSet::Normal : QIconSet::Disabled;
+    if (mode == QIconSet::Normal && has_focus)
+      mode = QIconSet::Active;
+    
+    QPixmap pixmap = t->iconSet()->pixmap(QIconSet::Small, mode);
+    int pixw = pixmap.width();
+    int pixh = pixmap.height();
+    r.setLeft(r.left() + pixw + 4);
+    r.setRight(r.right() + 2);
+
+#if QT_VERSION >= 0x030200 
+    int xoff = 0, yoff = 0;
+    if (!selected)
+    {
+      xoff = style().pixelMetric(QStyle::PM_TabBarTabShiftHorizontal, this);
+      yoff = style().pixelMetric(QStyle::PM_TabBarTabShiftVertical, this);
+    }
+    
+    p->drawPixmap(br.left() + 2 + xoff, br.center().y()-pixh/2 + yoff, pixmap);
+#else
+    p->drawPixmap(br.left() + 2 + ((selected == TRUE) ? 0 : 2),
+                  br.center().y()-pixh/2 + ((selected == TRUE) ? 0 : 2),
+                  pixmap);
+#endif
+  }
+    
+  QStyle::SFlags flags = QStyle::Style_Default;
+
+  if (isEnabled() && t->isEnabled())
+    flags |= QStyle::Style_Enabled;
+  if (has_focus)
+    flags |= QStyle::Style_HasFocus;
+  if (selected)
+    flags |= QStyle::Style_Selected;
+  //else if (t == d->pressed)
+  //  flags |= QStyle::Style_Sunken;
+  
+  if (t->rect().contains(mapFromGlobal(QCursor::pos())))
+    flags |= QStyle::Style_MouseOver;
+      
+  QColorGroup cg(colorGroup());
+  if (mTabColors.contains(t->identifier()))
+     cg.setColor(QColorGroup::Foreground, mTabColors[t->identifier()]);
+  
+  style().drawControl(QStyle::CE_TabBarLabel, p, this, r,
+                      t->isEnabled() ? cg: palette().disabled(),
+                      flags, QStyleOption(t));
+}
+         
+
+//CETabWidget
+CETabWidget::CETabWidget(QWidget *parent, const char *name, WFlags f)
+  : QTabWidget(parent, name, f)
+{
+  setTabBar(new CETabBar(this, "tabbar"));
+}
+
+void CETabWidget::setTabColor(QWidget *w, const QColor &color)
+{
+  QTab *t = tabBar()->tabAt(indexOf(w));
+  if (t)
+    static_cast<CETabBar *>(tabBar())->setTabColor(t->identifier(), color);
+}
 
 //-----CInfoField::constructor--------------------------------------------------
 CInfoField::CInfoField(QWidget *parent, bool readonly)
@@ -457,7 +616,6 @@ void CInfoField::setData(const char *data)
   setText(QString::fromLocal8Bit(data));
 }
 
-
 void CInfoField::setData(QString data)
 {
   setText(data);
@@ -516,16 +674,22 @@ void CHistoryWidget::paintCell(QPainter* p, int row, int col)
 #endif
 
 //- Message View Widget ---------------------------------------------------------
-
-#ifdef QT_PROTOCOL_PLUGIN
 CMessageViewWidget::CMessageViewWidget(const char *szId, unsigned long nPPID,
   CMainWindow *m, QWidget* parent, const char * name)
   : CHistoryWidget(parent, name)
 {
   m_szId = szId ? strdup(szId) : 0;
   m_nPPID = nPPID;
+  m_nMsgStyle = m->m_nMsgStyle;
+  m_bAppendLineBreak = m->m_bAppendLineBreak;
+  m_colorRcv = m->m_colorRcv;
+  m_colorSnt = m->m_colorSnt;
+  m_colorRcvHistory = m->m_colorRcvHistory;
+  m_colorSntHistory = m->m_colorSntHistory;
+  setPaletteBackgroundColor(m->m_colorChatBkg);
   mainwin = m;
 
+/*
   // add all unread messages.
   vector<CUserEvent*> newEventList;
   ICQUser *u = gUserManager.FetchUser(m_szId, m_nPPID, LOCK_W);
@@ -541,13 +705,21 @@ CMessageViewWidget::CMessageViewWidget(const char *szId, unsigned long nPPID,
   gUserManager.DropUser(u);
   for (unsigned short i = 0; i < newEventList.size(); i++)
     addMsg(newEventList[i]);
+*/
 }
-#endif
 
 CMessageViewWidget::CMessageViewWidget(unsigned long _nUin, CMainWindow *m, QWidget* parent, const char * name)
 :CHistoryWidget(parent,name)
 {
   m_nUin= _nUin;
+  m_szId = NULL; // avoid desalocation error at destructor
+  m_nMsgStyle = m->m_nMsgStyle;
+  m_bAppendLineBreak = m->m_bAppendLineBreak;
+  m_colorRcv = m->m_colorRcv;
+  m_colorSnt = m->m_colorSnt;
+  m_colorRcvHistory = m->m_colorRcvHistory;
+  m_colorSntHistory = m->m_colorSntHistory;
+  setPaletteBackgroundColor(m->m_colorChatBkg);
   mainwin = m;
 /*
   // add all unread messages.
@@ -569,60 +741,186 @@ CMessageViewWidget::CMessageViewWidget(unsigned long _nUin, CMainWindow *m, QWid
 
 CMessageViewWidget::~CMessageViewWidget()
 {
-#ifdef QT_PROTOCOL_PLUGIN
   if (m_szId)
     free(m_szId);
-#endif
+}
+
+void CMessageViewWidget::setOwner(const char *_szId)
+{
+  if (!_szId) return;
+
+  if (m_szId)
+    free(m_szId);
+  m_szId = strdup(_szId);
 }
 
 void CMessageViewWidget::addMsg(ICQEvent * _e)
 {
-#ifdef QT_PROTOCOL_PLUGIN
-  if (strcmp(_e->Id(), m_szId) == 0 && _e->PPID() == m_nPPID)
-#else
-  if ( _e->Uin() == m_nUin && _e->UserEvent() )
-#endif
+  if (strcmp(_e->Id(), m_szId) == 0 && _e->PPID() == m_nPPID &&
+    _e->UserEvent())
     addMsg( _e->UserEvent() );
 }
 
-void CMessageViewWidget::addMsg(CUserEvent* e )
+void CMessageViewWidget::addMsg(direction dir, bool fromHistory, QString eventDescription, QDateTime date, 
+  bool isDirect, bool isMultiRec, bool isUrgent, bool isEncrypted, 
+  QString contactName, QString messageText) 
+{
+  QString s;
+  QString color;
+  bool bAIM = (m_nPPID == LICQ_PPID) && !isdigit(m_szId[0]);
+
+  if (fromHistory) 
+    if (dir == D_RECEIVER) 
+      color = m_colorRcvHistory.name();
+    else 
+      color = m_colorSntHistory.name();  
+  else
+    if (dir == D_RECEIVER) 
+      color = m_colorRcv.name();
+    else 
+      color = m_colorSnt.name();  
+  
+  /* Remove trailing line breaks. */
+  for (unsigned int i = messageText.length() - 1; i>=0; i--) {
+    if (messageText.at(i) == '\n' || messageText.at(i) == '\r') {
+      messageText.truncate(i);
+    } else {
+      break;
+    }
+  }
+     
+  
+  switch (m_nMsgStyle) {
+    case 0:
+      s = QString("<html><body><font color=\"%1\"><b>%2%3 [%4%5%6%7] %8:</b></font><br>")
+                  .arg(color)
+                  .arg(eventDescription)
+                  .arg(date.time().toString())
+                  .arg(isDirect ? 'D' : '-')
+                  .arg(isMultiRec ? 'M' : '-')
+                  .arg(isUrgent ? 'U' : '-')
+                  .arg(isEncrypted ? 'E' : '-')
+                  .arg(contactName);
+      s.append(QString("<font color=\"%1\">%2</font></body></html>")
+                      .arg(color)
+                      .arg(messageText));
+      break;
+    case 1:
+      s = QString("<font color=\"%1\"><b>(%2%3) [%4%5%6%7] %8: </b></font>")
+                  .arg(color)
+                  .arg(eventDescription)
+                  .arg(date.time().toString())
+                  .arg(isDirect ? 'D' : '-')
+                  .arg(isMultiRec ? 'M' : '-')
+                  .arg(isUrgent ? 'U' : '-')
+                  .arg(isEncrypted ? 'E' : '-')
+                  .arg(contactName);
+      s.append(QString("<font color=\"%1\">%2</font>")
+                      .arg(color)
+                      .arg(messageText));
+      if (bAIM)
+      {
+        s.prepend("<html><body>");
+        s.append("</body></html>");
+      }
+
+      break;
+    case 2:
+      s = QString("<font color=\"%1\"><b>%2%3 - %4: </b></font>")
+                  .arg(color)
+                  .arg(eventDescription)
+                  .arg(date.time().toString())
+                  .arg(contactName);
+      s.append(QString("<font color=\"%1\">%2</font>")
+                      .arg(color)
+                      .arg(messageText));
+
+      if (bAIM)
+      {
+        s.prepend("<html><body>");
+        s.append("</body></html>");
+      }
+
+      break;  
+    case 3:
+      s = QString("<table border=\"1\"><tr><td><b><font color=\"%1\">%2%3</font><b><td><b><font color=\"%4\">%5</font></b></font></td>")
+                  .arg(color)
+                  .arg(eventDescription)
+                  .arg(date.time().toString())
+                  .arg(color)
+                  .arg(contactName);
+      s.append(QString("<td><font color=\"%1\">%2</font></td></tr></table>")
+                      .arg(color)
+                      .arg(messageText));
+      break; 
+    case 4:
+      s = QString("<font color=\"%1\"><b>%2<br>%3 [%4%5%6%7]</b></font><br><br>")
+                  .arg(color)
+                  .arg((dir == D_RECEIVER ? tr("%1 from %2") : tr("%1 to %2")))
+                  .arg(eventDescription)
+                  .arg(contactName)
+                  .arg(date.toString())
+                  .arg(isDirect ? 'D' : '-')
+                  .arg(isMultiRec ? 'M' : '-')
+                  .arg(isUrgent ? 'U' : '-')
+                  .arg(isEncrypted ? 'E' : '-');
+
+      // We break the paragraph here, since the history text
+      // could be in a different BiDi directionality than the
+      // header and timestamp text.
+      s.append(QString("<font color=\"%1\">%2</font><br><br>")
+                  .arg(color)
+                  .arg(messageText));
+    break;     
+  }
+  
+  append(s);
+  if (m_bAppendLineBreak) {
+    append("<hr>");
+  }
+}
+
+void CMessageViewWidget::addMsg(CUserEvent* e, const char *_szId, unsigned long _nPPID)
 {
   QDateTime date;
   date.setTime_t(e->Time());
   QString sd = date.time().toString();
+  bool bUseHTML = false;
 
   QString contactName;
   QTextCodec *codec = QTextCodec::codecForLocale();
 
   {
-#ifdef QT_PROTOCOL_PLUGIN
-    ICQUser *u = gUserManager.FetchUser(m_szId, m_nPPID, LOCK_R);
-#else
-    ICQUser *u = gUserManager.FetchUser(m_nUin, LOCK_R);
-#endif
-    if (u != NULL)
+
+    ICQUser *u = _szId ? gUserManager.FetchUser(_szId, _nPPID, LOCK_R) :
+                         gUserManager.FetchUser(m_szId, m_nPPID, LOCK_R);
+    if (u)
     {
       codec = UserCodec::codecForICQUser(u);
       if (e->Direction() == D_RECEIVER)
-        contactName = codec->toUnicode(u->GetAlias());
-      gUserManager.DropUser(u);
+        contactName = QString::fromUtf8(u->GetAlias());
+      for (unsigned int x = 0; x < strlen(m_szId); x++)
+      {
+        if (!isdigit(m_szId[x]) && m_nPPID == LICQ_PPID)
+        {
+          bUseHTML = true;
+          break;
+        }
+      }
     }
+    gUserManager.DropUser(u);
   }
 
   if (e->Direction() != D_RECEIVER)
   {
-#ifdef QT_PROTOCOL_PLUING
     ICQOwner *o = gUserManager.FetchOwner(m_nPPID, LOCK_R);
-#else
-    ICQOwner *o = gUserManager.FetchOwner(LOCK_R);
-#endif
     if (o != NULL)
     {
       // Don't use this codec to decode our conversation with the contact
       // since we're using the contact's encoding, not ours.
       QTextCodec *ownerCodec = UserCodec::codecForICQUser(o);
       contactName = ownerCodec->toUnicode(o->GetAlias());
-      gUserManager.DropOwner();
+      gUserManager.DropOwner(m_nPPID);
     }
   }
 
@@ -635,31 +933,20 @@ void CMessageViewWidget::addMsg(CUserEvent* e )
   else
      messageText = codec->toUnicode(e->Text());
 
-  const char *color = (e->Direction() == D_RECEIVER) ? "red" : "blue";
-
-  // QTextEdit::append adds a paragraph break so we don't have to.
-  s.sprintf("<font color=\"%s\"><b>%s%s [%c%c%c%c] %s:</b></font>",
-            color,
-            e->SubCommand() == ICQ_CMDxSUB_MSG ? "" :
-              (EventDescription(e) + " ").utf8().data(),
-            sd.utf8().data(),
-            e->IsDirect() ? 'D' : '-',
-            e->IsMultiRec() ? 'M' : '-',
-            e->IsUrgent() ? 'U' : '-',
-            e->IsEncrypted() ? 'E' : '-',
-            contactName.utf8().data()
-           );
-  append(s);
-  s.sprintf("<font color=\"%s\">%s</font>",
-            color,
-            MLView::toRichText(messageText, true).utf8().data()
-           );
-  append(s);
+  addMsg(e->Direction(), false,
+         (e->SubCommand() == ICQ_CMDxSUB_MSG ? QString("") : (EventDescription(e) + " ")),
+         date,
+         e->IsDirect(),
+         e->IsMultiRec(),
+         e->IsUrgent(),
+         e->IsEncrypted(),
+         contactName,
+         MLView::toRichText(messageText, true, bUseHTML));
 #else
   QString messageText = codec->toUnicode(e->Text());
   s.sprintf("%c%s%s [%c%c%c%c] %s:\n%s",
             (e->Direction() == D_RECEIVER) ? '\001' : '\002',
-            e->SubCommand() == ICQ_CMDxSUB_MSG ? "" :
+            e->SubCommand() == ICQ_CMDxSUB_MSG ? QString("") :
               (EventDescription(e) + " ").utf8().data(),
             sd.utf8().data(),
             e->IsDirect() ? 'D' : '-',
@@ -694,4 +981,55 @@ void CMessageViewWidget::addMsg(CUserEvent* e )
   }
 }
 
+void CMessageViewWidget::addNotice(QString dateTime, QString messageText)
+{
+  QString color = "green";
+  QString s = "";
+  
+  /* Remove trailing line breaks. */
+  for (unsigned int i = messageText.length() - 1; i >= 0; i--)
+  {
+    if (messageText.at(i) == '\n' || messageText.at(i) == '\r')
+      messageText.truncate(i);
+    else
+      break;
+  }
+     
+  
+  switch (m_nMsgStyle)
+  {
+    case 1:
+      s = QString("<font color=\"%1\"><b>[%2] %3</b></font>")
+                  .arg(color)
+                  .arg(dateTime)
+                  .arg(messageText);
+      break;
+    case 2:
+      s = QString("<font color=\"%1\"><b>[%2] %3</b></font>")
+                  .arg(color)
+                  .arg(dateTime)
+                  .arg(messageText);
+      break;  
+    case 3:
+      s = QString("<table border=\"1\"><tr><td><b><font color=\"%1\">%2</font><b><td><b><font color=\"%3\">%4</font></b></font></td></tr></table>")
+                  .arg(color)
+                  .arg(dateTime)
+                  .arg(color)
+                  .arg(messageText);
+      break; 
+
+    case 0:
+    default:
+      s = QString("<html><body><font color=\"%1\"><b>[%2] %3</b></font><br></body></html>")
+                  .arg(color)
+                  .arg(dateTime)
+                  .arg(messageText);
+      break;    
+  }
+  
+  append(s);
+  if (m_bAppendLineBreak)
+    append("<hr>");
+}
+
 #include "ewidgets.moc"
