@@ -59,6 +59,10 @@ class UserInfoDlg;
 class UserEventTabDlg;
 #endif
 
+#ifdef USE_KDE
+class LicqKIMIface;
+#endif
+
 #if QT_VERSION < 300
   typedef QList<UserViewEvent> UserViewEventList;
   typedef QList<UserInfoDlg> UserInfoList;
@@ -69,24 +73,6 @@ class UserEventTabDlg;
   typedef QPtrList<UserSendCommon> UserSendEventList;
 #endif
 
-enum { /* ID's for the menu items in mnuUserAdm.
-          Needed to reference individual menu items,
-          i.e. when changing icons or status of an item.
-       */
-       MNU_USER_ADM_ADD_USER = 0,
-       MNU_USER_ADM_SEARCH_USER = 1,
-       MNU_USER_ADM_AUTHORIZE_USER = 2,
-       MNU_USER_ADM_REQUEST_AUTH = 3,
-       MNU_USER_ADM_RANDOM_CHAT = 4,
-       MNU_USER_ADM_POPUP_ALL_MSG = 5,
-       MNU_USER_ADM_EDIT_GROUPS = 6,
-       MNU_USER_ADM_UPDATE_ALL_USERS = 7,
-       MNU_USER_ADM_UPDATE_CURRENT_GROUP = 8,
-       MNU_USER_ADM_REDRAW_USER_WIN = 9,
-       MNU_USER_ADM_SAVE_ALL_USERS = 10,
-       MNU_USER_ADM_REGISTER_USER = 11
-};
-
 using std::vector;
 
 //=====CMainWindow==============================================================
@@ -99,11 +85,12 @@ public:
   CMainWindow(CICQDaemon *theServer, CSignalManager *theSigMan,
               CQtLogWindow *theLogWindow, bool bStartHidden,
               const char *skinName, const char *iconsName,
-              const char *extendedIconsName, QWidget *parent = 0);
+              const char *extendedIconsName, bool bDisableDockIcon,
+              QWidget *parent = 0);
   virtual ~CMainWindow();
   bool RemoveUserFromList(unsigned long, QWidget *);
   bool RemoveUserFromGroup(GroupType gtype, unsigned long group, unsigned long, QWidget *);
-  UserEventCommon *callFunction(int fcn, const char *, unsigned long);
+  UserEventCommon *callFunction(int fcn, const char *, unsigned long, int = -1);
   bool RemoveUserFromList(const char *, unsigned long, QWidget *);
   bool RemoveUserFromGroup(GroupType, unsigned long, const char *,
     unsigned long, QWidget *);
@@ -158,9 +145,14 @@ public:
        m_bPopLastOnline,
        m_bPopOnlineSince,
        m_bPopIdleTime,
+       m_bPopID,
        m_bShowAllEncodings,
        m_bTabbedChatting,
-       m_bSortColumnAscending;
+       m_bShowHistory,
+       m_bDisableDockIcon,
+       m_bSortColumnAscending,
+       m_bAppendLineBreak,
+       m_bFlashTaskbar;
 
   QString m_MsgAutopopupKey;
   QString m_DefaultEncoding;
@@ -172,12 +164,22 @@ public:
 
   unsigned long m_nCurrentGroup, m_nGroupStates;
   unsigned short m_nSortByStatus,
-                 m_nSortColumn;
+                 m_nSortColumn,
+                 m_nMsgStyle;
+                 
+  QColor m_colorRcvHistory,
+         m_colorSntHistory,
+         m_colorRcv,
+         m_colorSnt,
+         m_colorTab,
+         m_colorTabTyping,
+         m_colorChatBkg;
+                   
   GroupType m_nGroupType;
   QString usprintfHelp;
 
 public slots:
-  void callInfoTab(int, const char *, unsigned long, bool toggle=false);
+  void callInfoTab(int, const char *, unsigned long, bool = false, bool =false);
 
 public:
   // Command Tools
@@ -218,7 +220,8 @@ public:
              *mnuUtilities,
              *mnuMiscModes,
              *mnuSend,
-             *mnuProtocolStatus[16];
+             *mnuProtocolStatus[16],
+             *mnuProtocolOwnerAdm[16];
   CELabel *lblStatus, *lblMsg;
   CEButton *btnSystem;
   CEComboBox *cmbUserGroups;
@@ -237,11 +240,11 @@ public:
           pmPrivate, pmFFC, pmMessage, pmUrl, pmChat, pmFile, pmContact, 
           pmSms, pmAuthorize, pmSMS, pmSecureOn, pmSecureOff, pmHistory, 
           pmInfo, pmRemove, pmEncoding, pmBirthday, pmPhone, pmCellular, 
-          pmInvisible, pmCustomAR, pmCollapsed, pmExpanded, pmSearch,
+          pmInvisible, pmTyping, pmCustomAR, pmCollapsed, pmExpanded, pmSearch,
           pmICQphoneActive, pmICQphoneBusy, pmPhoneFollowMeActive,
           pmPhoneFollowMeBusy, pmSharedFiles, pmMSNOnline, pmMSNOffline,
           pmMSNOccupied, pmMSNPrivate, pmMSNAway, pmAIMOnline, pmAIMAway,
-          pmAIMOffline;
+          pmAIMOffline, pmGPGKey, pmGPGKeyEnabled, pmGPGKeyDisabled;
   unsigned long m_nUserMenuUin;
   unsigned int positionChanges;
   int m_nProtoNum;
@@ -262,6 +265,10 @@ public:
   DockMode m_nDockMode;
   int mouseX, mouseY;
 
+#ifdef USE_KDE
+    LicqKIMIface* kdeIMInterface;
+#endif
+  
   // Functions
   void CreateUserView();
   void CreateUserFloaty(unsigned long nUin, unsigned short x = 0,
@@ -284,7 +291,7 @@ public:
   friend class OptionsDlg;
   friend class CUserViewItem;
   friend class UserSendCommon;
-
+  
 public slots:
   void updateUserWin();
   void slot_shutdown();
@@ -303,7 +310,8 @@ protected slots:
   void updateEvents();
   void updateStatus(CICQSignal * = NULL);
   void updateGroups();
-  void changeStatus(int index, unsigned long nPPID = 0xFFFFFFFF); //all
+  void changeStatus(int index, unsigned long nPPID = 0xFFFFFFFF,
+                    bool _bAutoLogon = false); //all
   void changeStatusManualProtocol(int index);
   void changeStatusManual(int index);
   void changePFMStatus(int index);
@@ -316,6 +324,9 @@ protected slots:
   void callFileFunction (const char *);
   void callUrlFunction (const char *);
   void callUserFunction(int);
+  void slot_socket(const char *, unsigned long, unsigned long);
+  void slot_convoJoin(const char *, unsigned long, unsigned long);
+  void slot_convoLeave(const char *, unsigned long, unsigned long);
   //TODO
   //void callUserFunction(const char *, unsigned long);
   void slot_userfinished(const char *, unsigned long);
@@ -337,8 +348,9 @@ protected slots:
   void showEditGrpDlg();
   void showSkinBrowser();
   void showPluginDlg();
+  void showGPGKeyManager();
   void slot_AwayMsgDlg();
-  void showAwayMsgDlg(unsigned short status);
+  void showAwayMsgDlg(unsigned short status, bool autoclose = false);
   void showOptionsDlg(int);
   void popupOptionsDlg() { emit showOptionsDlg(0); };
   void showAuthUserDlg();
@@ -365,11 +377,56 @@ protected slots:
   void slot_doneUserEventTabDlg();
   void slot_pluginUnloaded(unsigned long);
 
+  void sendMsg(const char* szId, unsigned long nPPID, const QString& message);
+  void sendFileTransfer(const char* szId, unsigned long nPPID,
+                        const QString& filename, const QString& description);
+  void sendChatRequest(const char* szId, unsigned long nPPID);
+  void addUser(const char* szId, unsigned long nPPID);
+
 signals:
   void changeDockStatus(unsigned short);
   void signal_sentevent(ICQEvent *e);
   void signal_doneRegisterUser(ICQEvent *e);
 
+private:
+  enum { /* ID's for the menu items in mnuUserAdm.
+            Needed to reference individual menu items,
+            i.e. when changing icons or status of an item.
+        */
+        MNU_USER_ADM_ADD_USER = 0,
+        MNU_USER_ADM_SEARCH_USER = 1,
+        MNU_USER_ADM_AUTHORIZE_USER = 2,
+        MNU_USER_ADM_REQUEST_AUTH = 3,
+        MNU_USER_ADM_RANDOM_CHAT = 4,
+        MNU_USER_ADM_POPUP_ALL_MSG = 5,
+        MNU_USER_ADM_EDIT_GROUPS = 6,
+        MNU_USER_ADM_UPDATE_ALL_USERS = 7,
+        MNU_USER_ADM_UPDATE_CURRENT_GROUP = 8,
+        MNU_USER_ADM_REDRAW_USER_WIN = 9,
+        MNU_USER_ADM_SAVE_ALL_USERS = 10,
+        MNU_USER_ADM_REGISTER_USER = 11
+  };
+  
+  enum { /* ID's for the menu items in mnuSystem.
+            Needed to reference individual menu items,
+            i.e. when changing icons or status of an item.
+        */
+        MNU_SYS_SYSTEM_FUNCTIONS = 0,
+        MNU_SYS_USER_FUNCTIONS = 1,
+        MNU_SYS_STATUS = 2,
+        MNU_SYS_GROUP= 3,
+        MNU_SYS_AUTO_RESPONSE = 4,
+        MNU_SYS_NETWORKWIN = 5,
+        MNU_SYS_MINI_MODE = 6,
+        MNU_SYS_SHOW_OFFLINE = 7,
+        MNU_SYS_THREAD_VIEW = 8,
+        MNU_SYS_OPTIONS = 9,
+        MNU_SYS_SKINBROWSER = 10,
+        MNU_SYS_PLUGINS = 11,
+        MNU_SYS_GPG = 12
+  };
+
+
 };
 
 // -----------------------------------------------------------------------------
