@@ -120,6 +120,21 @@ void CSignalManager::ProcessSignal(CICQSignal *s)
   case SIGNAL_EVENTxID:
     emit signal_eventTag(s->Id(), s->PPID(), s->Argument());
     break;
+  case SIGNAL_SOCKET:
+    emit signal_socket(s->Id(), s->PPID(), s->CID());
+    break;
+  case SIGNAL_CONVOxJOIN:
+    emit signal_convoJoin(s->Id(), s->PPID(), s->CID());
+    break;
+  case SIGNAL_CONVOxLEAVE:
+    emit signal_convoLeave(s->Id(), s->PPID(), s->CID());
+    break;
+  case SIGNAL_VERIFY_IMAGE:
+    emit signal_verifyImage(s->PPID());
+    break;
+  case SIGNAL_NEW_OWNER:
+    emit signal_newOwner(s->Id(), s->PPID());
+    break;
   default:
     gLog.Warn("%sInternal error: CSignalManager::ProcessSignal(): Unknown signal command received from daemon: %ld.\n",
               L_WARNxSTR, s->Signal());
@@ -151,6 +166,8 @@ void CSignalManager::ProcessEvent(ICQEvent *e)
   case MAKESNAC(ICQ_SNACxFAM_MESSAGE, ICQ_SNACxMSG_SERVERxMESSAGE):
   case MAKESNAC(ICQ_SNACxFAM_MESSAGE, ICQ_SNACxMSG_SERVERxREPLYxMSG):
   case MAKESNAC(ICQ_SNACxFAM_MESSAGE, ICQ_SNACxMSG_SENDxSERVER):
+  case MAKESNAC(ICQ_SNACxFAM_LOCATION, ICQ_SNACxREQUESTxUSERxINFO):
+  case MAKESNAC(ICQ_SNACxFAM_LOCATION, ICQ_SNACxLOC_INFOxREQ):
     emit signal_doneUserFcn(e);
     break;
 
