@@ -68,7 +68,16 @@ CUserViewItem::CUserViewItem(ICQUser *_cUser, QListView *parent)
                            .arg(_cUser->GetAlias()).arg(_cUser->IdString()));
 
   m_nUin = _cUser->Uin();
-  m_szId = _cUser->IdString() ? strdup(_cUser->IdString()) : 0;
+  if (_cUser->IdString())
+  {
+    char *szRealId = 0;
+    ICQUser::MakeRealId(_cUser->IdString(), _cUser->PPID(), szRealId);
+    m_szId = strdup(szRealId);
+    delete [] szRealId;
+  }
+  else
+    m_szId = 0;
+  m_szAlias = _cUser->GetAlias();
   m_nPPID = _cUser->PPID();
   m_bUrgent = false;
   m_bSecure = false;
@@ -77,6 +86,9 @@ CUserViewItem::CUserViewItem(ICQUser *_cUser, QListView *parent)
   m_bCellular = false;
   m_nOnlCount = 0;
   m_nEvents = 0;
+  m_nStatus = ICQ_STATUS_OFFLINE;
+  m_bGPGKey = false;
+  m_bNotInList = _cUser->NotInList();
   setGraphics(_cUser);
 }
 
@@ -87,16 +99,29 @@ CUserViewItem::CUserViewItem (ICQUser *_cUser, CUserViewItem* item)
 {
   m_nGroupId = (unsigned short)(-1);
   m_nUin = _cUser->Uin();
-  m_szId = _cUser->IdString() ? strdup(_cUser->IdString()) : 0;
+  if (_cUser->IdString())
+  {
+    char *szRealId = 0;
+    ICQUser::MakeRealId(_cUser->IdString(), _cUser->PPID(), szRealId);
+    m_szId = strdup(szRealId);
+    delete [] szRealId;
+  }
+  else
+    m_szId = 0;
+
+  m_szAlias = _cUser->GetAlias();
   m_nPPID = _cUser->PPID();
   m_bUrgent = false;
   m_bSecure = false;
   m_bBirthday = false;
   m_bPhone = false;
   m_bCellular = false;
+  m_bGPGKey = false;
   m_nOnlCount = 0;
   m_nEvents = 0;
   m_nStatus = ICQ_STATUS_OFFLINE;
+  m_bGPGKey = false;
+  m_bNotInList = _cUser->NotInList();
   setGraphics(_cUser);
 }
 
@@ -107,6 +132,7 @@ CUserViewItem::CUserViewItem(unsigned short Id, const char* name, QListView* lv)
 {
   m_nUin = 0;
   m_szId = 0;
+  m_szAlias = 0;
   m_nPPID =0;
   m_pIcon = NULL;
   m_cBack = s_cBack;
@@ -120,8 +146,17 @@ CUserViewItem::CUserViewItem(unsigned short Id, const char* name, QListView* lv)
   m_bCellular = false;
   m_nOnlCount = 0;
   m_nEvents = 0;
+  m_nStatus = 0;
   // Other users group is sorted at the end
-  m_sSortKey = m_nGroupId ? QString("%1").arg((int)m_nGroupId, 10) : QString("9999999999");
+  if (m_nGroupId)
+  {
+    QString strTemp = QString("%1").arg((int)m_nGroupId);
+    while (strTemp.length() < 10)
+      strTemp = "0" + strTemp;
+    m_sSortKey = strTemp;
+  }
+  else
+    m_sSortKey = QString("9999999999");
   m_sPrefix = "1";
   setPixmap(0, gMainWindow->pmCollapsed);
   setText(1, QString::fromLocal8Bit(name));
@@ -138,6 +173,7 @@ CUserViewItem::CUserViewItem(BarType barType, QListView *parent)
   m_nPPID = 0;
   m_nOnlCount = 0;
   m_nEvents = 0;
+  m_nStatus = 0;
   m_pIcon = NULL;
   m_cBack = s_cBack;
   m_cFore = s_cOnline;
@@ -150,8 +186,10 @@ CUserViewItem::CUserViewItem(BarType barType, QListView *parent)
   m_sSortKey = "";
   if (barType == BAR_ONLINE)
     m_sPrefix = "0";
-  else
+  else if (barType == BAR_OFFLINE)
     m_sPrefix = "2";
+  else
+    m_sPrefix = "4";
 }
 
 void CUserViewItem::SetThreadViewGroupTitle()
@@ -170,22 +208,28 @@ CUserViewItem::~CUserViewItem()
 {
   CUserView *v = (CUserView *)listView();
 
-  if (v == NULL || m_szId == 0) return;
-
   if (m_szId)
-    free (m_szId);
-  else
+    free(m_szId);
+
+  if (v == NULL) return;
+
+  if (this == v->barOffline || this == v->barOnline || this == v->barNotInList)
     return;
 
-  if (m_nStatus == ICQ_STATUS_OFFLINE)
-    v->numOffline--;
+  if (m_bNotInList)
+    v->numNotInList--;
   else
-    v->numOnline--;
+  {
+    if (m_nStatus == ICQ_STATUS_OFFLINE)
+      v->numOffline--;
+    else
+      v->numOnline--;
+  }
 
   if (parent())
   {
     CUserViewItem *i = static_cast<CUserViewItem*>(parent());
-    if (m_nStatus != ICQ_STATUS_OFFLINE) i->m_nOnlCount--;
+    if (m_nPPID && m_nStatus != ICQ_STATUS_OFFLINE) i->m_nOnlCount--;
     i->m_nEvents -= m_nEvents;
     i->SetThreadViewGroupTitle();
   }
@@ -200,6 +244,11 @@ CUserViewItem::~CUserViewItem()
     delete v->barOnline;
     v->barOnline = NULL;
   }
+  if (v->numNotInList == 0 && v->barNotInList != NULL)
+  {
+    delete v->barNotInList;
+    v->barNotInList = NULL;
+  }
 }
 
 //-----CUserViewItem::setGraphics-----------------------------------------------
@@ -225,6 +274,7 @@ void CUserViewItem::setGraphics(ICQUser *u)
    m_nStatus = u->Status();
    m_nStatusFull = u->StatusFull();
    m_bStatusInvisible = u->StatusInvisible();
+   m_bStatusTyping = u->GetTyping() == ICQ_TYPING_ACTIVE;
    m_nPhoneFollowMeStatus = u->PhoneFollowMeStatus();
    m_nICQphoneStatus = u->ICQphoneStatus();
    m_nSharedFilesStatus = u->SharedFilesStatus();
@@ -235,25 +285,38 @@ void CUserViewItem::setGraphics(ICQUser *u)
    m_bBirthday =  (u->Birthday() == 0);
    m_bPhone  = u->GetPhoneNumber()[0] != '\0';
    m_bCellular = u->GetCellularNumber()[0] !='\0';
+   m_bGPGKey = ( u->GPGKey()!=NULL ) && ( strcmp( u->GPGKey(), "" )!=0 );
+   m_bGPGKeyEnabled = ( u->UseGPG() );
 
    // Create any necessary bars
-   if (u->StatusOffline())
+   if (u->NotInList())
    {
-     v->numOffline++;
-     if (v->barOffline == NULL && v->parent()
-         && gMainWindow->m_bShowDividers
+     v->numNotInList++;
+     if (v->barNotInList == NULL && v->parent()
          && !(gMainWindow->m_bThreadView && gMainWindow->m_nGroupType == GROUPS_USER
-              && gMainWindow->m_nCurrentGroup == 0))
-       v->barOffline = new CUserViewItem(BAR_OFFLINE, listView());
+         && gMainWindow->m_nCurrentGroup == 0))
+       v->barNotInList = new CUserViewItem(BAR_NOTINLIST, listView());
    }
    else
    {
-     v->numOnline++;
-     if (v->barOnline == NULL && v->parent()
-         && gMainWindow->m_bShowDividers &&
-         !(gMainWindow->m_bThreadView && gMainWindow->m_nGroupType == GROUPS_USER
+     if (u->StatusOffline())
+     {
+       v->numOffline++;
+       if (v->barOffline == NULL && v->parent()
+           && gMainWindow->m_bShowDividers
+           && !(gMainWindow->m_bThreadView && gMainWindow->m_nGroupType == GROUPS_USER
            && gMainWindow->m_nCurrentGroup == 0))
-       v->barOnline = new CUserViewItem(BAR_ONLINE, listView());
+         v->barOffline = new CUserViewItem(BAR_OFFLINE, listView());
+     }
+     else
+     {
+       v->numOnline++;
+       if (v->barOnline == NULL && v->parent()
+           && gMainWindow->m_bShowDividers &&
+           !(gMainWindow->m_bThreadView && gMainWindow->m_nGroupType == GROUPS_USER
+             && gMainWindow->m_nCurrentGroup == 0))
+         v->barOnline = new CUserViewItem(BAR_ONLINE, listView());
+     }
    }
 
    m_sPrefix = "1";
@@ -277,6 +340,9 @@ void CUserViewItem::setGraphics(ICQUser *u)
        m_cFore = s_cOnline;
    }
 
+   if (m_bNotInList)
+     m_sPrefix = "5";
+
    //if (u->StatusInvisible())
    //   m_cFore = s_cAway;
 
@@ -343,7 +409,10 @@ void CUserViewItem::setGraphics(ICQUser *u)
    {
      QTextCodec * codec = UserCodec::codecForICQUser(u);
      sTemp = u->usprintf(gMainWindow->colInfo[i]->m_szFormat);
-     setText(i + 1, codec->toUnicode(sTemp));
+     if (strcmp(gMainWindow->colInfo[i]->m_szFormat, "%a") == 0)
+       setText(i + 1, QString::fromUtf8(sTemp));
+     else
+       setText(i + 1, codec->toUnicode(sTemp));
      free(sTemp);
    }
 
@@ -411,22 +480,9 @@ void CUserViewItem::paintCell( QPainter *p, const QColorGroup & cgdefault, int c
 
   const QPixmap *pix = NULL;
 
-#if QT_VERSION >= 220
   if (listView()->parent() && gMainWindow->skin->frame.transparent )
-    pix = listView()->QListView::parentWidget()->backgroundPixmap();
-
-  listView()->setStaticBackground(pix && listView()->contentsHeight() >= listView()->viewport()->height());
-#else
-  if ((listView()->contentsHeight() < listView()->viewport()->height() ||
-       listView()->vScrollBarMode() == QListView::AlwaysOff) &&
-      listView()->parent() && gMainWindow->skin->frame.transparent )
-#if QT_VERSION >= 300
     pix = listView()->QListView::parentWidget()->paletteBackgroundPixmap();
-#else
-    pix = listView()->QListView::parentWidget()->backgroundPixmap();
-#endif
-
-#endif
+  listView()->setStaticBackground(pix && listView()->contentsHeight() >= listView()->viewport()->height());
 
   if (pix != NULL)
   {
@@ -457,79 +513,118 @@ void CUserViewItem::paintCell( QPainter *p, const QColorGroup & cgdefault, int c
         f.setPointSize(f.pointSize() - 2);
       p->setFont(f);
     }
-    
+
     QListViewItem::paintCell(p, cg, column, width, align);
 
+    // first determine where to start painting for the current column
+    int w = 0; // width
+    switch (align)
+    {
+      case LEFT:
+                w = p->fontMetrics().width(text(column)) + 6;
+                break;
+      case RIGHT:
+                w = listView()->columnWidth(column) - p->fontMetrics().width(text(column)) - 6;
+                break;
+      case CENTER:
+                w = (listView()->columnWidth(column) / 2) + (p->fontMetrics().width(text(column)) / 2) + 6;
+                break;
+      default:
+                w = 0;
+                break;
+    }
+
     if (isGroupItem())
     {
       if (column == 1)
       {
-        int w = p->fontMetrics().width(text(1)) + 4;
+        w = (align == RIGHT) ? w+2 : w-2;  //  +/- 2
 
         if (m_nEvents > 0 && !isOpen())
         {
+          if (align == RIGHT)
+            w -= gMainWindow->pmMessage.width();
           p->drawPixmap(w, 0, gMainWindow->pmMessage);
-          w += gMainWindow->pmMessage.width() + 4;
+          w = (align == RIGHT) ? w - 2 : w + gMainWindow->pmMessage.width() + 2;
         }
-
-#if QT_VERSION >= 300
         qDrawShadeLine( p, 0, height() >> 1, width - 5, (height() >> 1) + 2, cg, true, 1, 0);
-#else
-        listView()->style().drawSeparator(p,
-                                          w, height() >> 1, width - (listView()->header()->count() == 1 ? 5 : 1),
-                                          height() >> 1, cg);
-#endif
       }
       else if (column == listView()->header()->count() - 1)
       {
-#if QT_VERSION >= 300
         qDrawShadeLine( p, 0, height() >> 1, width - 5, (height() >> 1) + 2, cg, true, 1, 0);
-#else
-        listView()->style().drawSeparator(p, 0, height() >> 1, width - 5,
-           height() >> 1, cg);
-#endif
       }
       else if (column > 1)
       {
-#if QT_VERSION >= 300
         qDrawShadeLine( p, 0, height() >> 1, width - 5, (height() >> 1) + 2, cg, true, 1, 0);
-#else
-        listView()->style().drawSeparator(p, 0, height() >> 1, width - 1,
-           height() >> 1, cg);
-#endif
       }
-    }
-    // If this is the first column then add some extra icons after the text
-    else if (column == 1 && gMainWindow->m_bShowExtendedIcons)
+    } // isGroupItem
+    else if (m_szAlias                                        // this row contains a user 
+             && text(column).ascii()                          // ...+ it is not empty 
+             && strcmp(m_szAlias, text(column).ascii()) == 0  // ...+ this col contains the users alias 
+             && gMainWindow->m_bShowExtendedIcons)            // ...+ extended icons are enabled
     {
-      int w = p->fontMetrics().width(text(1)) + 6;
-
+      // pmPhone
       if (width - w > 8 && (m_bPhone))
       {
+        if (align == RIGHT)
+          w -= gMainWindow->pmPhone.width();
         p->drawPixmap(w, 0, gMainWindow->pmPhone);
-        w += gMainWindow->pmPhone.width() + 2;
+        w = (align == RIGHT) ? w - 2 : w + gMainWindow->pmPhone.width() + 2;
       }
+
+      // pmCellular
       if (width - w > 8 && (m_bCellular))
       {
+        if (align == RIGHT)
+          w -= gMainWindow->pmCellular.width();
         p->drawPixmap(w, 0, gMainWindow->pmCellular);
-        w += gMainWindow->pmCellular.width() + 2;
+        w = (align == RIGHT) ? w - 2 : w + gMainWindow->pmCellular.width() + 2;
       }
+
+      // pmBirthday
       if (width - w > 8 && (m_bBirthday))
       {
+        if (align == RIGHT)
+          w -= gMainWindow->pmBirthday.width();
         p->drawPixmap(w, 0, gMainWindow->pmBirthday);
-        w += gMainWindow->pmBirthday.width() + 2;
+        w = (align == RIGHT) ? w - 2 : w + gMainWindow->pmBirthday.width() + 2;
       }
+
+      // pmPrivate / pmInvisible
       if (width - w > 8 && m_bStatusInvisible)
       {
         if (gMainWindow->pmInvisible.isNull())
         {
+          if (align == RIGHT)
+            w -= gMainWindow->pmPrivate.width();
           p->drawPixmap(w, 0, gMainWindow->pmPrivate);
-          w += gMainWindow->pmPrivate.width() + 2;
+          w = (align == RIGHT) ? w - 2 : w + gMainWindow->pmPrivate.width() + 2;
         }
         else
         {
+          if (align == RIGHT)
+            w -= gMainWindow->pmInvisible.width();
           p->drawPixmap(w, 0, gMainWindow->pmInvisible);
-          w += gMainWindow->pmInvisible.width() + 2;
+          w = (align == RIGHT) ? w - 2 : w + gMainWindow->pmInvisible.width() + 2;
+        }
+      }
+      
+      // pmGPGKey
+      if ( width - w > 8 && m_bGPGKey )
+      {
+        if ( m_bGPGKeyEnabled )
+        {
+          if (align == RIGHT)
+            w -= gMainWindow->pmGPGKeyEnabled.width();
+          p->drawPixmap(w, 0, gMainWindow->pmGPGKeyEnabled);
+          w = (align == RIGHT) ? w - 2 : w + gMainWindow->pmGPGKeyEnabled.width() + 2;
+        }
+        else
+        {
+          if (align == RIGHT)
+            w -= gMainWindow->pmGPGKeyDisabled.width();
+          p->drawPixmap(w, 0, gMainWindow->pmGPGKeyDisabled);
+          w = (align == RIGHT) ? w - 2 : w + gMainWindow->pmGPGKeyDisabled.width() + 2;
         }
       }
       if (m_nStatus != ICQ_STATUS_OFFLINE)
@@ -538,43 +633,64 @@ void CUserViewItem::paintCell( QPainter *p, const QColorGroup & cgdefault, int c
         {
           if (m_nPhoneFollowMeStatus == ICQ_PLUGIN_STATUSxACTIVE)
           {
+            if (align == RIGHT)
+              w -= gMainWindow->pmPhoneFollowMeActive.width();
             p->drawPixmap(w, 0, gMainWindow->pmPhoneFollowMeActive);
-            w += gMainWindow->pmPhoneFollowMeActive.width() + 2;
+            w = (align == RIGHT) ? w - 2 : w + gMainWindow->pmPhoneFollowMeActive.width() + 2;
           }
           else if (m_nPhoneFollowMeStatus == ICQ_PLUGIN_STATUSxBUSY)
           {
+            if (align == RIGHT)
+              w -= gMainWindow->pmPhoneFollowMeBusy.width();
             p->drawPixmap(w, 0, gMainWindow->pmPhoneFollowMeBusy);
-            w += gMainWindow->pmPhoneFollowMeBusy.width() + 2;
+            w = (align == RIGHT) ? w - 2 : w + gMainWindow->pmPhoneFollowMeBusy.width() + 2;
           }
         }
         if (width - w > 8)
         {
           if (m_nICQphoneStatus == ICQ_PLUGIN_STATUSxACTIVE)
           {
+            if (align == RIGHT)
+              w -= gMainWindow->pmICQphoneActive.width();
             p->drawPixmap(w, 0, gMainWindow->pmICQphoneActive);
-            w += gMainWindow->pmICQphoneActive.width() + 2;
+            w = (align == RIGHT) ? w - 2 : w + gMainWindow->pmICQphoneActive.width() + 2;
           }
           else if (m_nICQphoneStatus == ICQ_PLUGIN_STATUSxBUSY)
           {
+            if (align == RIGHT)
+              w -= gMainWindow->pmICQphoneBusy.width();
             p->drawPixmap(w, 0, gMainWindow->pmICQphoneBusy);
-            w += gMainWindow->pmICQphoneBusy.width() + 2;
+            w = (align == RIGHT) ? w - 2 : w + gMainWindow->pmICQphoneBusy.width() + 2;
           }
         }
         if (width - w > 8 && m_nSharedFilesStatus == ICQ_PLUGIN_STATUSxACTIVE)
         {
+          if (align == RIGHT)
+            w -= gMainWindow->pmSharedFiles.width();
           p->drawPixmap(w, 0, gMainWindow->pmSharedFiles);
-          w += gMainWindow->pmSharedFiles.width() + 2;
+          w = (align == RIGHT) ? w - 2 : w + gMainWindow->pmSharedFiles.width() + 2;
+        }
+        if (width - w > 8 && m_bStatusTyping && m_nPPID == LICQ_PPID)
+        {
+          if (align == RIGHT)
+            w -= gMainWindow->pmTyping.width();
+          p->drawPixmap(w, 0, gMainWindow->pmTyping);
+          w = (align == RIGHT) ? w - 2 : w + gMainWindow->pmTyping.width() + 2;
         }
       }
       if (width - w > 8 && m_bSecure)
       {
+        if (align == RIGHT)
+          w -= gMainWindow->pmSecureOn.width();
         p->drawPixmap(w, 0, gMainWindow->pmSecureOn);
-        w += gMainWindow->pmSecureOn.width() + 2;
+        w = (align == RIGHT) ? w - 2 : w + gMainWindow->pmSecureOn.width() + 2;
       }
       if (width - w > 8 && m_bCustomAR)
       {
+        if (align == RIGHT)
+          w -= gMainWindow->pmCustomAR.width();
         p->drawPixmap(w, 0, gMainWindow->pmCustomAR);
-        w += gMainWindow->pmCustomAR.width() + 2;
+        w = (align == RIGHT) ? w - 2 : w + gMainWindow->pmCustomAR.width() + 2;
       }
     }
   }
@@ -599,6 +715,8 @@ void CUserViewItem::paintCell( QPainter *p, const QColorGroup & cgdefault, int c
       QString sz = CUserView::tr("Offline");
       if (m_sPrefix == "0")
         sz = CUserView::tr("Online");
+      else if (m_sPrefix == "4")
+        sz = CUserView::tr("Not In List");
 
       if (pix)
       {
@@ -641,14 +759,8 @@ void CUserViewItem::paintCell( QPainter *p, const QColorGroup & cgdefault, int c
 void CUserView::paintEmptyArea( QPainter *p, const QRect &r )
 {
   const QPixmap *pix = NULL;
-#if QT_VERSION >= 220
   if( parent() && gMainWindow->skin->frame.transparent)
     pix = QListView::parentWidget()->backgroundPixmap();
-#else
-  if ((contentsHeight() < viewport()->height() || vScrollBarMode() == AlwaysOff)
-      && parent() && gMainWindow->skin->frame.transparent)
-    pix = QListView::parentWidget()->backgroundPixmap();
-#endif
 
   if (pix != NULL)
   {
@@ -692,15 +804,22 @@ void CUserView::timerEvent(QTimerEvent* e)
       {
         CUserViewItem* item = static_cast<CUserViewItem*>(it.current());
 
-        if (item->ItemId() && (strcmp(item->ItemId(), carId) == 0) &&
+        char *szRealId = 0;
+        ICQUser::MakeRealId(item->ItemId(), item->ItemPPID(), szRealId);
+        if (item->ItemId() && (strcmp(szRealId, carId) == 0) &&
             item->ItemPPID() == carPPID)
         {
           if(carCounter == 1)
             item->repaint();
           else
             item->drawCAROverlay(&p);
-          if(!doGroupView)  break;
+          if(!doGroupView)
+          {
+            delete [] szRealId;
+            break;
+          }
         }
+        delete [] szRealId;
       }
     }
 
@@ -728,13 +847,23 @@ void CUserView::timerEvent(QTimerEvent* e)
       {
         CUserViewItem* item = static_cast<CUserViewItem*>(it.current());
 
-        if (item->ItemId() && (strcmp(item->ItemId(), onlId) == 0) &&
+        char *szRealId = 0;
+        ICQUser::MakeRealId(item->ItemId(), item->ItemPPID(), szRealId);
+
+        if (szRealId && (strcmp(szRealId, onlId) == 0) &&
             item->ItemPPID() == onlPPID)
         {
           found = true;
           item->repaint();
-          if(!doGroupView)  break;
+          if(!doGroupView)
+          {
+            delete [] szRealId;
+            break;
+          }
         }
+
+        if (szRealId)
+          delete [] szRealId;
       }
     }
 
@@ -805,19 +934,15 @@ UserFloatyList* CUserView::floaties = 0;
 
 //-----UserList::constructor-----------------------------------------------------------------------
 CUserView::CUserView(QPopupMenu *m, QWidget *parent, const char *name)
-#if QT_VERSION >= 220
   : QListView(parent, name, parent == NULL ? WStyle_Customize | WStyle_NoBorder | WResizeNoErase | WRepaintNoErase /*| WStyle_StaysOnTop*/
               : WRepaintNoErase),
-#else
-  : QListView(parent, name),
-#endif
     QToolTip(viewport())
 {
   m_nFlashCounter = carCounter = onlCounter = 0;
   msgTimerId = carTimerId = onlTimerId = 0;
   mnuUser = m;
-  barOnline = barOffline = NULL;
-  numOnline = numOffline = 0;
+  barOnline = barOffline = barNotInList = NULL;
+  numOnline = numOffline = numNotInList = 0;
 
   m_typeAhead = "";
   m_typePos   = 0;
@@ -832,13 +957,8 @@ CUserView::CUserView(QPopupMenu *m, QWidget *parent, const char *name)
 
   setAcceptDrops(true);
   viewport()->setAcceptDrops(true);
-  #if QT_VERSION < 300
-  viewport()->setBackgroundMode(NoBackground);
-  #endif
 
-#if QT_VERSION >= 210
   setShowSortIndicator(true);
-#endif
   setAllColumnsShowFocus(true);
   setTreeStepSize(0);
   setSorting(gMainWindow->m_nSortColumn, gMainWindow->m_bSortColumnAscending);
@@ -879,7 +999,7 @@ CUserView::CUserView(QPopupMenu *m, QWidget *parent, const char *name)
 
 CUserView::~CUserView()
 {
-  barOnline = barOffline = NULL;
+  barOnline = barOffline = barNotInList = NULL;
   if (parent() == NULL)
   {
     unsigned int i = 0;
@@ -918,9 +1038,9 @@ CUserView *CUserView::FindFloaty(const char *szId, unsigned long nPPID)
 
 void CUserView::clear()
 {
-  barOnline = barOffline = NULL;
+  barOnline = barOffline = barNotInList = NULL;
   QListView::clear();
-  numOffline = numOnline = 0;
+  numOffline = numOnline = numNotInList = 0;
 }
 
 
@@ -1009,27 +1129,8 @@ void CUserView::viewportMousePressEvent(QMouseEvent *e)
     m_typeAhead = "";
     m_typePos = 0;
   }
-#if QT_VERSION < 300
-  else if (e->button() == RightButton)
-  {
-    CUserViewItem *clickedItem = (CUserViewItem *)itemAt(e->pos());
-    if (clickedItem != NULL)
-    {
-      setSelected(clickedItem, true);
-      setCurrentItem(clickedItem);
-      if (clickedItem->ItemId())
-      {
-        gMainWindow->SetUserMenuUser(clickedItem->ItemId(), clickedItem->ItemPPID());
-        mnuUser->popup(viewport()->mapToGlobal(e->pos()), 1);
-      }
-    }
-    m_typeAhead = "";
-    m_typePos = 0;
-  }
-#endif
 }
 
-#if QT_VERSION >= 300
 void CUserView::contentsContextMenuEvent ( QContextMenuEvent* e )
 {
   CUserViewItem *clickedItem = (CUserViewItem *)itemAt(contentsToViewport( e->pos()) );
@@ -1046,7 +1147,6 @@ void CUserView::contentsContextMenuEvent ( QContextMenuEvent* e )
     }
   }
 }
-#endif
 
 void CUserView::viewportDragEnterEvent(QDragEnterEvent* e)
 {
@@ -1066,12 +1166,22 @@ void CUserView::viewportDropEvent(QDropEvent* e)
       QStrList lst;
       if(QUriDrag::decode(e, lst))
       {
-        if(!(text = QUriDrag::uriToLocalFile(lst.first())).isEmpty())
+        QStrListIterator strIter(lst);
+        if(!(text = QUriDrag::uriToLocalFile(strIter)).isEmpty())
         {
           UserSendFileEvent *e = static_cast<UserSendFileEvent *>
             (gMainWindow->callFunction(mnuUserSendFile, it->ItemId(),
                 it->ItemPPID()));
           e->setFile(text, QString::null);
+          
+          // Add all the files
+          while (strIter != lst.getLast())
+          {
+            ++strIter;
+            if (!(text = QUriDrag::uriToLocalFile(strIter)).isEmpty())
+              e->addFile(text);
+          }
+          
           e->show();
         }
         else
@@ -1083,6 +1193,7 @@ void CUserView::viewportDropEvent(QDropEvent* e)
           e->show();
         }
       }
+      
       //TODO change this
       else if(QTextDrag::decode(e, text)) {
 //        const char *p = (text.left(4).latin1());
@@ -1435,6 +1546,8 @@ void CUserView::maybeTip(const QPoint& c)
     
     if (item->m_nStatus != ICQ_STATUS_OFFLINE)
     {
+      if (item->m_bStatusTyping)
+        s += tr("<br>Typing&nbsp;a&nbsp;message");
       if (item->m_nPhoneFollowMeStatus == ICQ_PLUGIN_STATUSxACTIVE)
         s += tr("<br>Phone&nbsp;&quot;Follow&nbsp;Me&quot;:&nbsp;Available");
       else if (item->m_nPhoneFollowMeStatus == ICQ_PLUGIN_STATUSxBUSY)
@@ -1465,11 +1578,7 @@ void CUserView::maybeTip(const QPoint& c)
       if (u->AutoResponse() && *u->AutoResponse() &&
           item->m_nStatus != ICQ_STATUS_OFFLINE &&
           item->m_nStatus != ICQ_STATUS_ONLINE)
-#if QT_VERSION >= 300
-        s += tr("<br><u>Auto Response:</u>") + QStyleSheet::convertFromPlainText(codec->toUnicode(u->AutoResponse()), QStyleSheetItem::WhiteSpaceNormal);
-#else
-        s += tr("<br><u>Auto Response:</u>") + QStyleSheet::convertFromPlainText(codec->toUnicode(u->AutoResponse()));
-#endif
+        s += tr("<br><u>Auto Response:</u>") + codec->toUnicode(u->AutoResponse());
       gUserManager.DropUser(u);
     }
 
@@ -1555,6 +1664,16 @@ void CUserView::maybeTip(const QPoint& c)
         tip (r, s);
       }
     }
+   
+    if (gMainWindow->m_bPopID)
+    {
+      char *szTemp;
+      szTemp = u->usprintf("%u");
+      QString temp(szTemp);
+      free(szTemp);
+      s += tr("<br><nobr>ID: ") + temp + tr("</nobr>");
+      tip (r, s);
+    }
 
     tip(r, s);
   }
