@@ -19,11 +19,22 @@
 #ifndef USEREVENTDLG_H
 #define USEREVENTDLG_H
 
+#include <qframe.h>
+#include <qlabel.h>
 #include <qwidget.h>
+#include <functional>
 #include <list>
+#include <string>
+#include <utility>
+
+using std::binary_function;
+using std::list;
+using std::pair;
+using std::string;
 
 #include "licq_color.h"
 #include "licq_filetransfer.h"
+#include "licq_message.h"
 
 class QTabWidget;
 class QBoxLayout;
@@ -55,12 +66,23 @@ class CETabWidget;
 class CMessageViewWidget;
 class MLView;
 
+/* ----------------------------------------------------------------------------- */
+typedef pair<CUserEvent *, char *> MessageIter;
+
+struct OrderMessages : public binary_function<const MessageIter &, const MessageIter &, bool>
+{
+  bool operator()(const MessageIter& m1, const MessageIter& m2)
+  {
+    return (m1.first->Time() < m2.first->Time());
+  }
+};
+
 /* ----------------------------------------------------------------------------- */
 class UserEventTabDlg : public QWidget
 {
   Q_OBJECT
 public:
-  UserEventTabDlg(QWidget *parent = 0, const char *name = 0);
+  UserEventTabDlg(CMainWindow *mainwin, QWidget *parent = 0, const char *name = 0);
   ~UserEventTabDlg();
 
   void addTab(UserEventCommon *tab, int index = -1);
@@ -69,13 +91,22 @@ public:
   void replaceTab(QWidget *oldTab, UserEventCommon *newTab);
   bool tabIsSelected(QWidget *tab);
   bool tabExists(QWidget *tab);
+  void updateConvoLabel(UserEventCommon *tab);
   void updateTabLabel(ICQUser *u);
-  void gotTyping(ICQUser *u);
+  void gotTyping(ICQUser *u, int);
 
+#ifdef USE_KDE
+#if KDE_VERSION >= 320
+  virtual void setIcon(const QPixmap &icon);
+#endif   // KDE_VERSION
+#endif // USE_KDE
+  
 private:
   CETabWidget *tabw;
+  CMainWindow *mainwin;
   void updateTitle(QWidget *tab);
   void clearEvents(QWidget *tab);
+  void flashTaskbar(bool _bFlash);
 
 public slots:
   void slot_currentChanged(QWidget *tab);
@@ -96,12 +127,16 @@ public:
                   const char *name = 0);
   virtual ~UserEventCommon();
 
-  char *Id()  { return m_szId; }
+  const char *Id()  { return m_lUsers.front().c_str(); }
   unsigned long PPID()  { return m_nPPID; }
-
+  unsigned long ConvoId() { return m_nConvoId; }
+  list<string>& ConvoUsers() { return m_lUsers; }
+  void SetConvoId(unsigned long n)  { m_nConvoId = n; }
+  
+  bool FindUserInConvo(char *);
   void AddEventTag(unsigned long n)  { if (n) m_lnEventTag.push_back(n); }
   void gotTyping(unsigned short);
- 
+   
   enum type {
   	UC_MESSAGE,
   	UC_URL,
@@ -114,6 +149,7 @@ protected:
   QTextCodec *codec;
   bool m_bOwner;
   char *m_szId;
+  list<string> m_lUsers;
   unsigned long m_nPPID;
   QBoxLayout* top_lay, *top_hlay;
   CICQDaemon *server;
@@ -121,7 +157,8 @@ protected:
   CSignalManager *sigman;
   std::list<unsigned long> m_lnEventTag;
   QWidget *mainWidget;
-  QPushButton *btnHistory, *btnInfo, *btnEncoding, *btnSecure, *btnForeColor, *btnBackColor;
+  QPushButton *btnHistory, *btnInfo, *btnEncoding, *btnSecure, 
+              *btnEmoticon, *btnForeColor, *btnBackColor;
   QPopupMenu * popupEncoding;
 
   CInfoField *nfoStatus, *nfoTimezone;
@@ -129,13 +166,15 @@ protected:
   QTimer *tmrTime, *tmrTyping;
   bool m_bDeleteUser;
   QString m_sBaseTitle, m_sProgressMsg;
-
+  unsigned long m_nConvoId;
+    
   // ID of the higest event we've processed. Helps determine
   // which events we already processed in the ctor.
   int m_highestEventId;
 
-  virtual void UserUpdated(CICQSignal *, ICQUser *) = 0;
+  virtual void UserUpdated(CICQSignal *, char * = 0, unsigned long = 0) = 0;
   void SetGeneralInfo(ICQUser *);
+  void FlashTaskbar(bool _bFlash);
 
 protected slots:
   void slot_connectsignal();
@@ -183,7 +222,7 @@ protected:
   void generateReply();
   void sendMsg(QString txt);
   void updateNextButton();
-  virtual void UserUpdated(CICQSignal *, ICQUser *);
+  virtual void UserUpdated(CICQSignal *, char * = 0, unsigned long = 0);
 
 protected slots:
   void slot_close();
@@ -214,11 +253,14 @@ public:
   virtual ~UserSendCommon();
 
   void setText(const QString& txt);
+  void convoJoin(const char *, unsigned long _nConvoId);
+  void convoLeave(const char *, unsigned long _nConvoId);
+  
 #if QT_VERSION >= 300
   virtual void windowActivationChange(bool oldActive);
 #endif
   int clearDelay;
-
+  
 signals:
   void autoCloseNotify();
   void updateUser(CICQSignal*);
@@ -239,7 +281,7 @@ protected:
   QTimer *tmrSendTyping;
 
   void RetrySend(ICQEvent *e, bool bOnline, unsigned short nLevel);
-  virtual void UserUpdated(CICQSignal *, ICQUser *);
+  virtual void UserUpdated(CICQSignal *, char * = 0, unsigned long = 0);
   virtual bool sendDone(ICQEvent *) = 0;
   bool checkSecure();
 
@@ -253,6 +295,8 @@ protected slots:
   void cancelSend();
   void massMessageToggled(bool);
   void slot_resettitle();
+  void slot_Emoticon();
+  void slot_insertEmoticon(const QString &);
   void slot_SetForegroundICQColor();
   void slot_SetBackgroundICQColor();
   void trySecure();
@@ -331,7 +375,8 @@ public:
   virtual ~UserSendFileEvent();
 
   void setFile(const QString& file, const QString& description);
-
+  void addFile(const QString& file);
+  
 protected:
   QLabel *lblItem;
   CInfoField *edtItem;
@@ -433,4 +478,34 @@ protected slots:
 
 /* ----------------------------------------------------------------------------- */
 
+class EmoticonLabel : public QLabel
+{
+  Q_OBJECT
+public:
+  EmoticonLabel(const QString &file, const QString &key, QWidget *parent);
+
+signals:
+  void clicked(const QString &);
+
+protected:
+  void mouseReleaseEvent(QMouseEvent *);
+  
+  QString m_sKey;
+};
+  
+/* ----------------------------------------------------------------------------- */
+
+class SelectEmoticon : public QFrame
+{
+  Q_OBJECT
+public:
+  SelectEmoticon(QWidget *parent);
+  
+signals:
+  void selected(const QString &);
+  
+protected slots:
+  void emoticonClicked(const QString &);
+};
+
 #endif
