@@ -44,6 +44,14 @@
 
 #ifdef USE_KDE
 #include <kfiledialog.h>
+#include <kdeversion.h>
+#if KDE_IS_VERSION(3, 1, 0)
+#include <kabc/stdaddressbook.h>
+#include <kabc/addressee.h>
+#include <kabc/addresseedialog.h>
+#include <licqkimiface.h>
+#define USE_KABC
+#endif
 #else
 #include <qfiledialog.h>
 #endif
@@ -80,7 +88,7 @@
 
 // -----------------------------------------------------------------------------
 UserInfoDlg::UserInfoDlg(CICQDaemon *s, CSignalManager *theSigMan, CMainWindow *m,
-                         const char *szId, unsigned long nPPID, QWidget* parent)
+                         const char *szId, unsigned long nPPID, QWidget *parent)
   : QWidget(parent, "UserInfoDialog", WDestructiveClose)
 {
   server = s;
@@ -102,6 +110,9 @@ UserInfoDlg::UserInfoDlg(CICQDaemon *s, CSignalManager *theSigMan, CMainWindow *
   CreatePicture();
   CreateHistory();
   CreateLastCountersInfo();
+#ifdef USE_KABC
+  CreateKABCInfo();
+#endif
 
   QBoxLayout *lay = new QVBoxLayout(this, 8);
 
@@ -117,6 +128,9 @@ UserInfoDlg::UserInfoDlg(CICQDaemon *s, CSignalManager *theSigMan, CMainWindow *
   tabs->addTab(tabList[PictureInfo].tab, tabList[PictureInfo].label);
   tabs->addTab(tabList[HistoryInfo].tab, tabList[HistoryInfo].label);
   tabs->addTab(tabList[LastCountersInfo].tab, tabList[LastCountersInfo].label);
+#ifdef USE_KABC
+  tabs->addTab(tabList[KABCInfo].tab, tabList[KABCInfo].label);
+#endif
 
   connect (tabs, SIGNAL(selected(const QString &)), this, SLOT(updateTab(const QString &)));
   connect (sigman, SIGNAL(signal_updatedUser(CICQSignal *)),
@@ -185,9 +199,9 @@ UserInfoDlg::UserInfoDlg(CICQDaemon *s, CSignalManager *theSigMan, CMainWindow *
     else
       tmp = tmp + lastname;
     if (!tmp.isEmpty()) tmp = " (" + tmp + ")";
-    m_sBasic = tr("Licq - Info ") + codec->toUnicode(u->GetAlias()) + tmp;
+    m_sBasic = tr("Licq - Info ") + QString::fromUtf8(u->GetAlias()) + tmp;
     resetCaption();
-    setIconText(codec->toUnicode(u->GetAlias()));
+    setIconText(u->GetAlias());
     gUserManager.DropUser(u);
   }
 
@@ -379,7 +393,7 @@ void UserInfoDlg::SetGeneralInfo(ICQUser *u)
   if(m_bOwner)
     chkKeepAliasOnUpdate->hide();
   chkKeepAliasOnUpdate->setChecked(u->KeepAliasOnUpdate());
-  nfoAlias->setData(codec->toUnicode(u->GetAlias()));
+  nfoAlias->setData(QString::fromUtf8(u->GetAlias()));
   connect(nfoAlias, SIGNAL(textChanged(const QString &)), this, SLOT(slot_aliasChanged(const QString &)));
   nfoFirstName->setData(codec->toUnicode(u->GetFirstName()));
   nfoLastName->setData(codec->toUnicode(u->GetLastName()));
@@ -455,7 +469,7 @@ void UserInfoDlg::SaveGeneralInfo()
 
   u->SetEnableSave(false);
 
-  u->SetAlias(codec->fromUnicode(nfoAlias->text()));
+  u->SetAlias(nfoAlias->text().utf8());
   u->SetKeepAliasOnUpdate(chkKeepAliasOnUpdate->isChecked());
   u->SetFirstName(codec->fromUnicode(nfoFirstName->text()));
   u->SetLastName(codec->fromUnicode(nfoLastName->text()));
@@ -1163,13 +1177,13 @@ void UserInfoDlg::CreateAbout()
   p->setSpacing(8);
 
   lblAbout = new QLabel(tr("About:"), p);
-  mleAbout = new MLEditWrap(true, p);
-  mleAbout->setReadOnly(!m_bOwner);
-#if QT_VERSION < 300
-  // Windows ICQ seems to limit the about info to 450 chars
-  // so we do the same
-  mleAbout->setMaxLength(450);
+  mlvAbout = new CHistoryWidget(p, "About");//EditWrap(true, p);
+  mlvAbout->setReadOnly(!m_bOwner);
+  mlvAbout->setTextFormat(RichText);
+#if QT_VERSION >= 300
+  connect(mlvAbout, SIGNAL(viewurl(QWidget*, QString)), mainwin, SLOT(slot_viewurl(QWidget *, QString)));
 #endif
+
 }
 
 void UserInfoDlg::SetAbout(ICQUser *u)
@@ -1185,11 +1199,13 @@ void UserInfoDlg::SetAbout(ICQUser *u)
   }
 
   QTextCodec * codec = UserCodec::codecForICQUser(u);
+  bool bUseHTML = isalpha(u->IdString()[0]);
 
   QString aboutstr = codec->toUnicode(u->GetAbout());
   aboutstr.replace(QRegExp("\r"), "");
-  mleAbout->setText(aboutstr);
-
+  mlvAbout->clear();
+  mlvAbout->append(MLView::toRichText(codec->toUnicode(u->GetAbout()), true, bUseHTML));
+  
   if (bDropUser) gUserManager.DropUser(u);
 }
 
@@ -1199,7 +1215,7 @@ void UserInfoDlg::SaveAbout()
   if (u == NULL) return;
 
   QTextCodec * codec = UserCodec::codecForICQUser(u);
-  QString str = mleAbout->text();
+  QString str = mlvAbout->text();
 
   u->SetAbout(codec->fromUnicode(str.left(450)));
   gUserManager.DropUser(u);
@@ -1479,6 +1495,7 @@ void UserInfoDlg::SavePicture()
   if (!m_bOwner) return;
   //FIXME other owners too
   ICQOwner *o = gUserManager.FetchOwner(LICQ_PPID, LOCK_W);
+  if (o == 0) return;
 
   o->SetEnableSave(false);
   o->SetPicture(m_sFilename.latin1());
@@ -1597,6 +1614,94 @@ void UserInfoDlg::SetLastCountersInfo(ICQUser *u)
   if (bDropUser) gUserManager.DropUser(u);
 }
 
+//-----KDE AddressBook info--------------------------------------------------------
+void UserInfoDlg::CreateKABCInfo()
+{
+#ifdef USE_KABC
+  tabList[KABCInfo].label = tr("KDE Addressbook");
+  tabList[KABCInfo].tab = new QWidget(this, tabList[KABCInfo].label.latin1());
+  tabList[KABCInfo].loaded = false;
+
+  QWidget *p = tabList[KABCInfo].tab;
+
+  QGridLayout *lay = new QGridLayout(p, 3, 2, 10, 5);
+
+  lay->addWidget(new QLabel(tr("Name:"), p), 0, 0);
+  nfoKABCName = new CInfoField(p, true);
+  lay->addWidget(nfoKABCName, 0, 1);
+
+  lay->addWidget(new QLabel(tr("Email:"), p), 1, 0);
+  nfoKABCEmail = new CInfoField(p, true);
+  lay->addWidget(nfoKABCEmail, 1, 1);
+  
+  lay->setRowStretch(2, 5);
+#endif
+}
+
+void UserInfoDlg::SetKABCInfo(ICQUser *u)
+{
+#ifdef USE_KABC
+  tabList[LastCountersInfo].loaded = true;
+  bool bDropUser = false;
+
+  if (u == NULL)
+  {
+    u = gUserManager.FetchUser(m_szId, m_nPPID, LOCK_R);
+    if (u == NULL) return;
+    bDropUser = true;
+  }
+
+  if (m_kabcID.isEmpty())
+  {
+    m_kabcID = mainwin->kdeIMInterface->kabcIDForUser(m_szId, m_nPPID);
+  }
+
+  if (!m_kabcID.isEmpty())
+  {
+    KABC::AddressBook* adrBook = KABC::StdAddressBook::self();
+    if (adrBook == 0)
+    {
+      if (bDropUser) gUserManager.DropUser(u);
+      return;
+    }
+
+    KABC::Addressee contact = adrBook->findByUid(m_kabcID);
+    if (!contact.isEmpty())
+    {
+      nfoKABCName->setData(contact.assembledName());
+      QString email = contact.preferredEmail();
+      nfoKABCEmail->setData(email);
+    }
+  }
+  
+  if (bDropUser) gUserManager.DropUser(u);
+
+#else
+    Q_UNUSED(u)
+#endif
+}
+
+void UserInfoDlg::UpdateKABCInfo()
+{
+#ifdef USE_KABC    
+    KABC::Addressee contact = KABC::AddresseeDialog::getAddressee(this);
+    if (!contact.isEmpty())
+    {
+      nfoKABCName->setData(contact.assembledName());
+      QString email = contact.preferredEmail();
+      nfoKABCEmail->setData(email);
+      m_kabcID = contact.uid();
+    }    
+#endif
+}
+
+void UserInfoDlg::SaveKABCInfo()
+{
+#ifdef USE_KABC
+    mainwin->kdeIMInterface->setKABCIDForUser(m_szId, m_nPPID, m_kabcID);
+#endif
+}
+
 // -----------------------------------------------------------------------------
 
 void UserInfoDlg::CreateHistory()
@@ -1621,7 +1726,8 @@ void UserInfoDlg::CreateHistory()
   chkHistoryReverse->setFixedSize(chkHistoryReverse->sizeHint());
   l->addWidget(chkHistoryReverse);
 
-  mlvHistory = new CHistoryWidget(p, "history");
+  mlvHistory = new CMessageViewWidget(m_szId, m_nPPID, mainwin, p, "history");
+  mlvHistory->m_nMsgStyle = 4; /* STYLE_HISTORY */
 #if QT_VERSION >= 300
   connect(mlvHistory, SIGNAL(viewurl(QWidget*, QString)), mainwin, SLOT(slot_viewurl(QWidget *, QString)));
 #endif
@@ -1908,7 +2014,7 @@ void UserInfoDlg::ShowHistory()
   {
       codec = UserCodec::codecForICQUser(u);
       if (!m_bOwner)
-         contactName = codec->toUnicode(u->GetAlias());
+         contactName = QString::fromUtf8(u->GetAlias());
       for (unsigned int x = 0; x < strlen(m_szId); x++)
       {
         if (!isdigit(m_szId[x]))
@@ -1922,6 +2028,8 @@ void UserInfoDlg::ShowHistory()
   barFiltering->setTotalSteps(NUM_MSG_PER_HISTORY);
   char* ftxt = qstrdup(codec->fromUnicode(lneFilter->text()));
   int flen = strlen(ftxt);
+  
+  mlvHistory->clear();
   while (m_nHistoryShowing < (NUM_MSG_PER_HISTORY))
   {
     if(UserInfoDlg::chkContains((*tempIter)->Text(), ftxt, flen))
@@ -1934,25 +2042,15 @@ void UserInfoDlg::ShowHistory()
         messageText = codec->toUnicode((*tempIter)->Text());
 
 #if QT_VERSION >= 300
-      const char *color = (*tempIter)->Direction() == D_RECEIVER ? "red" : "blue";
-      s = QString("<font color=\"%1\"><b>%2<br>%3 [%4%5%6%7]</b></font><br><br>")
-                  .arg(color)
-                  .arg(((*tempIter)->Direction() == D_RECEIVER ? tr("%1 from %2") : tr("%1 to %2"))
-                       .arg(EventDescription(*tempIter)).arg(QStyleSheet::escape(contactName)))
-                  .arg(date.toString())
-                  .arg((*tempIter)->IsDirect() ? 'D' : '-')
-                  .arg((*tempIter)->IsMultiRec() ? 'M' : '-')
-                  .arg((*tempIter)->IsUrgent() ? 'U' : '-')
-                  .arg((*tempIter)->IsEncrypted() ? 'E' : '-');
-      tmp.append(s);
-
-      // We break the paragraph here, since the history text
-      // could be in a different BiDi directionality than the
-      // header and timestamp text.
-      s = QString("<font color=\"%1\">%2</font><br><br>")
-                  .arg(color)
-                  .arg(MLView::toRichText(messageText, true, bUseHTML));
-      tmp.append(s);
+      mlvHistory->addMsg((*tempIter)->Direction(), false,
+                  EventDescription(*tempIter),
+                  date,
+                  (*tempIter)->IsDirect(),
+                  (*tempIter)->IsMultiRec(),
+                  (*tempIter)->IsUrgent(),
+                  (*tempIter)->IsEncrypted(),
+                  contactName,
+                  MLView::toRichText(messageText, true, bUseHTML));
 #else
       // See CHistoryWidget::paintCell for reference on those Qt 2-only
       // formatting escape codes.
@@ -1993,7 +2091,8 @@ void UserInfoDlg::ShowHistory()
     lblHistory->setText(tr("[<font color=\"%1\">Received</font>] "
                            "[<font color=\"%2\">Sent</font>] "
                            "%3 to %4 of %5")
-                        .arg(COLOR_RECEIVED).arg(COLOR_SENT)
+                        .arg(mainwin->m_colorRcv.name())
+                        .arg(mainwin->m_colorSnt.name())
                         .arg(m_nHistoryIndex - m_nHistoryShowing + 1)
                         .arg(m_nHistoryIndex)
                         .arg(m_lHistoryList.size()));
@@ -2001,11 +2100,11 @@ void UserInfoDlg::ShowHistory()
     lblHistory->setText(tr("[<font color=\"%1\">Received</font>] "
                            "[<font color=\"%2\">Sent</font>] "
                            "%3 out of %4 matches")
-                        .arg(COLOR_RECEIVED).arg(COLOR_SENT)
+                        .arg(mainwin->m_colorRcv.name())
+                        .arg(mainwin->m_colorSnt.name())
                         .arg(m_nHistoryShowing)
                         .arg(m_lHistoryList.size()));
-  mlvHistory->clear();
-  mlvHistory->append(tmp.left(tmp.length()-4));
+  /*mlvHistory->append(tmp.left(tmp.length()-4));*/
   if(!m_bHistoryReverse)
     mlvHistory->GotoEnd();
   else
@@ -2142,6 +2241,20 @@ void UserInfoDlg::updateTab(const QString& txt)
     if (!tabList[LastCountersInfo].loaded)
       SetLastCountersInfo(NULL);
   }
+#ifdef USE_KABC
+  else if (txt == tabList[KABCInfo].label)
+  {
+    currentTab = KABCInfo;
+    btnMain3->setText(tr("&Browse"));
+    btnMain2->setText(tr("&Save"));
+    btnMain1->setText(m_bOwner ? tr("&Save") : tr("&Menu"));
+    btnMain3->setEnabled(true);
+    btnMain2->setEnabled(true);
+    btnMain1->setEnabled(true);
+    if (!tabList[KABCInfo].loaded)
+      SetKABCInfo(NULL);
+  }
+#endif
 }
 
 void UserInfoDlg::SaveSettings()
@@ -2178,6 +2291,11 @@ void UserInfoDlg::SaveSettings()
     break;
   case LastCountersInfo:
     break;
+#ifdef USE_KABC
+  case KABCInfo:
+    SaveKABCInfo();
+    break;
+#endif
   }
 }
 
@@ -2194,6 +2312,14 @@ void UserInfoDlg::slotRetrieve()
       ShowHistoryNext();
     return;
   }
+
+#ifdef USE_KABC
+  if (currentTab == KABCInfo)
+  {
+    UpdateKABCInfo();
+    return;
+  }
+#endif
   
   ICQOwner *o = gUserManager.FetchOwner(m_nPPID, LOCK_R);
   if(o == NULL)  return;
@@ -2232,7 +2358,7 @@ void UserInfoDlg::slotRetrieve()
   }
   switch(currentTab)
   {
-    //TODO change in the daemon
+    //TODO change in the daemon to support other protocols
     case GeneralInfo:
     {
       // Before retrieving the meta data we have to 
@@ -2240,28 +2366,27 @@ void UserInfoDlg::slotRetrieve()
       // and the alias
       ICQUser *u = gUserManager.FetchUser(m_szId, m_nPPID, LOCK_R);
       if (u == NULL) return;
-      QTextCodec * codec = UserCodec::codecForICQUser(u);
       u->SetEnableSave(false);
-      u->SetAlias(codec->fromUnicode(nfoAlias->text()));
+      u->SetAlias(nfoAlias->text().utf8());
       u->SetKeepAliasOnUpdate(chkKeepAliasOnUpdate->isChecked());
       u->SetEnableSave(true);
       u->SaveGeneralInfo();
       gUserManager.DropUser(u);
       
-      icqEventTag = server->icqRequestMetaInfo(strtoul(m_szId, (char **)NULL, 10));
+      icqEventTag = server->ProtoRequestInfo(m_szId, m_nPPID);
       break;
     }
     case MoreInfo:
-      icqEventTag = server->icqRequestMetaInfo(strtoul(m_szId, (char **)NULL, 10));
+      icqEventTag = server->ProtoRequestInfo(m_szId, m_nPPID);
       break;
     case More2Info:
-      icqEventTag = server->icqRequestMetaInfo(strtoul(m_szId, (char **)NULL, 10));
+      icqEventTag = server->ProtoRequestInfo(m_szId, m_nPPID);
       break;
     case WorkInfo:
-      icqEventTag = server->icqRequestMetaInfo(strtoul(m_szId, (char **)NULL, 10));
+      icqEventTag = server->ProtoRequestInfo(m_szId, m_nPPID);
       break;
     case AboutInfo:
-      icqEventTag = server->icqRequestMetaInfo(strtoul(m_szId, (char **)NULL, 10));
+      icqEventTag = server->ProtoRequestInfo(m_szId, m_nPPID);
       break;
     case PhoneInfo:
     {
@@ -2324,8 +2449,7 @@ void UserInfoDlg::slotUpdate()
     cc = GetCountryByIndex(i)->nCode;
     server->icqSetEmailInfo(codec->fromUnicode(nfoEmailSecondary->text()),
 			    codec->fromUnicode(nfoEmailOld->text()));
-    //TODO in the daemon. along with the rest of the icqSet... family
-    icqEventTag = server->icqSetGeneralInfo(codec->fromUnicode(nfoAlias->text()),
+    icqEventTag = server->ProtoSetGeneralInfo(m_nPPID, nfoAlias->text(),
                                             codec->fromUnicode(nfoFirstName->text()),
                                             codec->fromUnicode(nfoLastName->text()),
                                             codec->fromUnicode(nfoEmailPrimary->text()),
@@ -2372,7 +2496,7 @@ void UserInfoDlg::slotUpdate()
                                          occupation,
                                          nfoCompanyHomepage->text().local8Bit());
   break;
-  case AboutInfo:    icqEventTag = server->icqSetAbout(codec->fromUnicode(mleAbout->text()));  break;
+  case AboutInfo:    icqEventTag = server->icqSetAbout(codec->fromUnicode(mlvAbout->text()));  break;
   case PhoneInfo:
   {
     EditPhoneDlg *epd = new EditPhoneDlg(this);
