@@ -42,6 +42,7 @@
 #ifdef USE_KDE
 #include <kwin.h>
 #include <kpopupmenu.h>
+#include <kdeversion.h>
 #endif
 
 #include "wharf.h"
@@ -61,18 +62,21 @@
 #undef FocusOut
 
 //#define DEBUG_WHARF
+#define SYSTEM_TRAY_REQUEST_DOCK    0
 
 /*
   Constructs a WharfIcon widget.
 */
 IconManager::IconManager(CMainWindow *_mainwin, QPopupMenu *_menu, QWidget *parent)
-  : QWidget(parent, "LicqWharf", WType_TopLevel)
+  : QWidget(NULL, "LicqWharf", WType_TopLevel)
 {
   setCaption("LicqWharf");
   m_nNewMsg = m_nSysMsg = 0;
   wharfIcon = NULL;
   mainwin = _mainwin;
   menu = _menu;
+
+  setBackgroundMode(X11ParentRelative);
 }
 
 
@@ -84,9 +88,14 @@ void IconManager::X11Init()
   WId win = winId();     // get the window
   XWMHints *hints;  // hints
   XClassHint classhint;  // class hints
+  XSizeHints sizeHints;
   classhint.res_name = "licq";  // res_name
   classhint.res_class = "Wharf";  // res_class
   XSetClassHint(dsp, win, &classhint); // set the class hints
+  sizeHints.flags= USPosition;
+  sizeHints.x = 0;
+  sizeHints.y = 0;
+  XSetWMNormalHints(dsp, win, &sizeHints);
   hints = XGetWMHints(dsp, win);  // init hints
   hints->initial_state = WithdrawnState;
   hints->icon_x = 0;
@@ -96,10 +105,16 @@ void IconManager::X11Init()
   hints->flags = WindowGroupHint | IconWindowHint | IconPositionHint | StateHint; // set the window group hint
   XSetWMHints(dsp, win, hints);  // set the window hints for WM to use.
   XFree( hints );
+
+  XMapWindow(dsp, wharfIcon->winId());
+  XMapWindow(dsp, win);
+  XSync(dsp, False);
+
 #endif
 
   resize (wharfIcon->width(), wharfIcon->height());
   setMask(*wharfIcon->vis->mask());
+  move(-100,-100);
   show();
 }
 
@@ -107,6 +122,10 @@ void IconManager::X11Init()
 
 IconManager::~IconManager()
 {
+  // Ensure that our main window is visible when the dock gets disabled:
+  if (mainwin != NULL)
+    mainwin->show();
+
   delete wharfIcon;
   // clean up behind us the prevent segfaults
   gMainWindow->licqIcon = NULL;
@@ -125,8 +144,13 @@ void IconManager::mousePressEvent( QMouseEvent *e )
   switch(e->button())
   {
     case LeftButton:
-      mainwin->show();
-      mainwin->raise();
+      if (mainwin->isVisible())
+        mainwin->hide();
+      else
+      {
+        mainwin->show();
+        mainwin->raise();
+      }
       break;
     case MidButton:
       mainwin->callMsgFunction();
@@ -188,6 +212,7 @@ void IconManager_Default::SetDockIconStatus()
 {
   QPixmap m;
   ICQOwner *o = gUserManager.FetchOwner(LOCK_R);
+  if (o == 0) return;
   switch (o->Status())
   {
     case ICQ_STATUS_ONLINE: m = QPixmap((const char **)iconOnline_xpm); break;
@@ -232,6 +257,8 @@ void IconManager_Default::SetDockIconStatus()
 QPixmap *IconManager_Default::GetDockIconStatusIcon()
 {
   ICQOwner *o = gUserManager.FetchOwner(LOCK_R);
+  if (o == 0) return NULL;
+  
   unsigned long s = o->Status();
   gUserManager.DropOwner();
   switch (s)
@@ -512,6 +539,7 @@ void IconManager_Themed::SetDockIconStatus()
 {
   QPixmap *p = NULL;
   ICQOwner *o = gUserManager.FetchOwner(LOCK_R);
+  if (o == 0) return;
   switch (o->Status())
   {
     case ICQ_STATUS_ONLINE: p = pixOnline; break;
@@ -586,10 +614,37 @@ IconManager_KDEStyle::IconManager_KDEStyle(CMainWindow *_mainwin, QPopupMenu *_m
   m_nSysMsg = 0;
   m_nNewMsg = 0;
   m_bStatusInvisible = false;
-  resize(24,24);
+  resize(22, 22);
+  setMinimumSize(22, 22);
+  setBackgroundMode(X11ParentRelative);
+
 #ifdef USE_KDE
   KWin::setSystemTrayWindowFor( winId(), _mainwin ? _mainwin->topLevelWidget()->winId() : qt_xrootwin() );
   setBackgroundMode(X11ParentRelative);
+#else
+  Display *dsp = x11Display();  // get the display
+  char buffer[128];
+  XEvent ev;
+
+  snprintf(buffer, sizeof(buffer), "_NET_SYSTEM_TRAY_S%d", x11Screen());
+  Atom a = XInternAtom(dsp, buffer, False);
+  Window systray = XGetSelectionOwner(dsp, a);
+
+
+  memset(&ev, 0, sizeof(ev));
+  ev.xclient.type = ClientMessage;
+  ev.xclient.window = systray;
+  ev.xclient.message_type = XInternAtom(dsp, "_NET_SYSTEM_TRAY_OPCODE", False);
+  ev.xclient.format = 32;
+  ev.xclient.data.l[0] = CurrentTime;
+  ev.xclient.data.l[1] = SYSTEM_TRAY_REQUEST_DOCK;
+  ev.xclient.data.l[2] = winId();
+  ev.xclient.data.l[3] = 0;
+  ev.xclient.data.l[4] = 0;
+
+  XSendEvent(dsp, systray, False, NoEventMask, &ev);
+  XSync(dsp, False);
+
 #endif
   show();
 }
@@ -603,6 +658,7 @@ void IconManager_KDEStyle::SetDockIconStatus()
 {
   //TODO make general for all plugins
   ICQOwner *o = gUserManager.FetchOwner(LOCK_R);
+  if (o == 0) return;
   m_statusIcon = CMainWindow::iconForStatus(o->StatusFull());
   m_ownerStatus = o->Status();
   m_bStatusInvisible = o->StatusInvisible();
@@ -648,12 +704,23 @@ void IconManager_KDEStyle::paintEvent( QPaintEvent *e)
     p.drawPixmap((width()-m_statusIcon.width())/2, (height()-m_statusIcon.height())/2, m_statusIcon);
 }
 
+bool IconManager_KDEStyle::x11Event(XEvent *ev)
+{
+  if ((ev->type == ReparentNotify))
+  {
+    XWindowAttributes a;
+    XGetWindowAttributes(qt_xdisplay(), ev->xreparent.parent, &a);
+  //  move((a.width - width()) / 2, (a.height - height()) / 2);
+  }
+  return QWidget::x11Event(ev);
+}
+
 void IconManager_KDEStyle::mousePressEvent( QMouseEvent *e )
 {
   switch(e->button())
   {
   case LeftButton:
-    if(mainwin->isVisible())
+    if(mainwin->isVisible() && !mainwin->isMinimized())
       mainwin->hide();
     else
     {
@@ -661,6 +728,10 @@ void IconManager_KDEStyle::mousePressEvent( QMouseEvent *e )
 #ifdef USE_KDE
       KWin::setOnDesktop(mainwin->winId(), KWin::currentDesktop());
 #endif
+      if (mainwin->isMaximized())
+      	mainwin->showMaximized();
+      else
+      	mainwin->showNormal();
       mainwin->raise();
     }
     break;
@@ -701,7 +772,7 @@ WharfIcon::WharfIcon(QPixmap *p, QWidget *parent)
   QToolTip::add(this, tr("Left click - Show main window\n"
                          "Middle click - Show next message\n"
                          "Right click - System menu"));
-  show();
+  //show();
 }
 
 
@@ -711,6 +782,7 @@ void WharfIcon::Set(QPixmap *pix)
   vis = new QPixmap(*pix);
   resize(vis->width(), vis->height());
   setMask(*vis->mask());
+  repaint();
 }
 
 WharfIcon::~WharfIcon()
@@ -733,6 +805,7 @@ void WharfIcon::paintEvent( QPaintEvent * )
 #ifdef DEBUG_WHARF
   printf("wharf paint\n");
 #endif
+  if (!vis) return;
   QPainter painter(this);
   painter.drawPixmap(0, 0, *vis);
   painter.end();
