@@ -43,10 +43,18 @@ const unsigned short CODE_VIEWxMSG = 208;
 const unsigned short CODE_VIEWxURL = 209;
 const unsigned short CODE_VIEWxCHAT= 210;
 const unsigned short CODE_VIEWxFILE = 211;
+const unsigned short CODE_STATUSxDONE = 212;
 const unsigned short CODE_VIEWxTIME = 220;
 const unsigned short CODE_VIEWxFLAGS = 221;
 const unsigned short CODE_VIEWxTEXTxSTART = 222;
 const unsigned short CODE_VIEWxTEXTxEND = 223;
+const unsigned short CODE_ADDUSERxDONE = 224;
+const unsigned short CODE_REMUSERxDONE = 225;
+const unsigned short CODE_SECURExOPEN = 226;
+const unsigned short CODE_SECURExCLOSE = 227;
+const unsigned short CODE_SECURExSTAT = 228;
+const unsigned short CODE_NOTIFYxON = 229;
+const unsigned short CODE_NOTIFYxOFF = 230;
 const unsigned short CODE_VIEWxUNKNOWN = 299;
 // 300 - further action required
 const unsigned short CODE_ENTERxUIN = 300;
@@ -64,6 +72,11 @@ const unsigned short CODE_VIEWxNONE = 405;
 const unsigned short CODE_EVENTxTIMEDOUT = 500;
 const unsigned short CODE_EVENTxFAILED = 501;
 const unsigned short CODE_EVENTxERROR = 502;
+const unsigned short CODE_ADDUSERxERROR = 503;
+const unsigned short CODE_SECURExNOTCOMPILED = 504;
+
+const unsigned short CODE_NOTIFYxSTATUS = 600;
+const unsigned short CODE_NOTIFYxMESSAGE = 601;
 
 const unsigned short STATE_UIN = 1;
 const unsigned short STATE_PASSWORD = 2;
@@ -72,6 +85,8 @@ const unsigned short STATE_ENTERxMESSAGE = 4;
 const unsigned short STATE_ENTERxURLxDESCRIPTION = 5;
 const unsigned short STATE_ENTERxURL = 6;
 const unsigned short STATE_ENTERxAUTOxRESPONSE = 7;
+const unsigned short STATE_ENTERxSMSxMESSAGE = 8;
+const unsigned short STATE_ENTERxSMSxNUMBER = 9;
 
 #define NEXT_WORD(s) while (*s != '\0' && *s == ' ') s++;
 
@@ -82,35 +97,45 @@ struct Command
   char *help;
 };
 
-static const unsigned short NUM_COMMANDS = 12;
-static struct Command commands[NUM_COMMANDS] =
+static struct Command commands[] =
 {
-  { "HELP", &CRMSClient::Process_HELP,
-    "Print out help on commands." },
-  { "QUIT", &CRMSClient::Process_QUIT,
-    "Close the connection.  With an argument of 1 causes the plugin to unload." },
-  { "TERM", &CRMSClient::Process_TERM,
-    "Terminate the licq daemon." },
-  { "INFO", &CRMSClient::Process_INFO,
-    "Print out user information.  Argument is the uin, or none for personal." },
-  { "STATUS", &CRMSClient::Process_STATUS,
-    "Set or show status.  Argument is new status, or blank to display current." },
+  { "ADDUSER", &CRMSClient::Process_ADDUSER,
+    "Add user to contact list { <id>[.<protocol>] }." },
+  { "AR", &CRMSClient::Process_AR,
+    "Set your (or a user custom) auto response { [ <id>[.<protocol>] ] }." },
   { "GROUPS", &CRMSClient::Process_GROUPS,
     "Show list of groups." },
+  { "HELP", &CRMSClient::Process_HELP,
+    "Print out help on commands." },
+  { "INFO", &CRMSClient::Process_INFO,
+    "Print out user information.  Argument is the id and protocol, or none for personal." },
   { "LIST", &CRMSClient::Process_LIST,
     "List users { [ <group #> ] [ <online|offline|all> ] [ <format> ] }." },
-  { "MESSAGE", &CRMSClient::Process_MESSAGE,
-    "Send a message { <uin> }." },
-  { "URL", &CRMSClient::Process_URL,
-    "Send a url { <uin> }." },
   { "LOG", &CRMSClient::Process_LOG,
     "Dump log messages { <log types> }." },
+  { "MESSAGE", &CRMSClient::Process_MESSAGE,
+    "Send a message { <id>[.<protocol>] }." },
+  { "QUIT", &CRMSClient::Process_QUIT,
+    "Close the connection.  With an argument of 1 causes the plugin to unload." },
+  { "REMUSER", &CRMSClient::Process_REMUSER,
+    "Remove user from contact list { <id>[.<protocol>] }." },
+  { "SECURE", &CRMSClient::Process_SECURE,
+    "Open/close/check secure channel { <uin> [ <open|close> ] } ." },
+  { "STATUS", &CRMSClient::Process_STATUS,
+    "Set or show status.  Argument is new status and protocol, or blank to display current. { [ <status>[.<protocol>] ] }." },
+  { "TERM", &CRMSClient::Process_TERM,
+    "Terminate the licq daemon." },
   { "VIEW", &CRMSClient::Process_VIEW,
-    "View event (next or specific user) { [ <uin> ] }." },
-  { "AR", &CRMSClient::Process_AR,
-    "Set your (or a user custom) auto response { [ <uin> ] }." }
+    "View event (next or specific user) { [ <id>[.<protocol>] ] }." },
+  { "URL", &CRMSClient::Process_URL,
+    "Send a url { <id>[.<protocol>] }." },
+  { "SMS", &CRMSClient::Process_SMS,
+    "Send an sms { <uin> }." },
+  { "NOTIFY", &CRMSClient::Process_NOTIFY,
+    "Notify events" },
 };
 
+static const unsigned short NUM_COMMANDS = sizeof(commands)/sizeof(*commands);
 
 /*---------------------------------------------------------------------------
  * CLicqRMS::Constructor
@@ -154,9 +179,9 @@ int CLicqRMS::Run(CICQDaemon *_licqDaemon)
   unsigned short nPort;
 
   // Register with the daemon, we only want the update user signal
-  m_nPipe = _licqDaemon->RegisterPlugin(SIGNAL_UPDATExUSER);
+  m_nPipe = _licqDaemon->RegisterPlugin(SIGNAL_ALL);
   licqDaemon = _licqDaemon;
-
+  
   char filename[256];
   sprintf (filename, "%s/licq_rms.conf", BASE_DIR);
   CIniFile conf;
@@ -175,7 +200,12 @@ int CLicqRMS::Run(CICQDaemon *_licqDaemon)
   }
   else
   {
-    server->StartServer(nPort);
+    if (!server->StartServer(nPort))
+    {
+      gLog.Error("Could not start server on port %u, "
+                 "maybe this port is already in use?\n", nPort);
+      return 1;
+    };
   }
 
   gLog.Info("%sRMS server started on port %d.\n", L_RMSxSTR, server->LocalPort());
@@ -225,8 +255,8 @@ int CLicqRMS::Run(CICQDaemon *_licqDaemon)
             {
               if ((*iter)->Activity() == -1)
               {
-                delete *iter;
                 clients.erase(iter);
+                delete *iter;
                 if (clients.size() == 0 && log != NULL)
                   log->SetLogTypes(0);
               }
@@ -322,14 +352,74 @@ void CLicqRMS::ProcessSignal(CICQSignal *s)
   switch (s->Signal())
   {
   case SIGNAL_UPDATExUSER:
+    if (s->SubSignal() == USER_STATUS)
+    {
+      ICQUser *u = gUserManager.FetchUser(s->Id(), s->PPID(), LOCK_R);
+      if (u)
+      {
+        ClientList::iterator iter;
+        for (iter = clients.begin(); iter != clients.end(); iter++)
+        {
+          if ((*iter)->m_bNotify)
+          {
+            char format[128], *ubuf;
+			strcpy(format, "%u %P %-20a %3m %s");
+			ubuf = u->usprintf(format);
+            fprintf((*iter)->fs, "%d %s\n", CODE_NOTIFYxSTATUS, ubuf);
+			free(ubuf);
+            fflush((*iter)->fs);
+          }
+        }
+        gUserManager.DropUser(u);
+      }
+      break;
+    }
+    else if (s->SubSignal() == USER_EVENTS)
+    {
+      ICQUser *u = gUserManager.FetchUser(s->Id(), s->PPID(), LOCK_R);
+      if (u)
+      {
+        ClientList::iterator iter;
+        for (iter = clients.begin(); iter != clients.end(); iter++)
+        {
+          if ((*iter)->m_bNotify)
+          {
+            char format[128], *ubuf;
+			strcpy(format, "%u %P %3m");
+			ubuf = u->usprintf(format);
+            fprintf((*iter)->fs, "%d %s\n", CODE_NOTIFYxMESSAGE, ubuf);
+			free(ubuf);
+            fflush((*iter)->fs);
+          }
+        }
+        gUserManager.DropUser(u);
+      }
+    }
   case SIGNAL_UPDATExLIST:
+    break;
   case SIGNAL_LOGON:
+    break;
+  case SIGNAL_EVENTxID:
+	AddEventTag(s->Id(), s->PPID(), s->Argument());
+    break;
   default:
     break;
+    
   }
   delete s;
 }
 
+/*---------------------------------------------------------------------------
+ * CLicqRMS::AddEventTag
+ *-------------------------------------------------------------------------*/
+void CLicqRMS::AddEventTag(const char *_szId, unsigned long _nPPID, unsigned long _nEventTag)
+{
+  ClientList ::iterator iter;
+  for (iter = clients.begin(); iter != clients.end(); iter++)
+  {
+    (*iter)->AddEventTag(_szId, _nPPID, _nEventTag);
+  }
+}
 
 /*---------------------------------------------------------------------------
  * CLicqRMS::ProcessEvent
@@ -372,15 +462,19 @@ CRMSClient::CRMSClient(TCPSocket *sin) : sock(0)
   sockman.DropSocket(&sock);
 
   gLog.Info("%sClient connected from %s.\n", L_RMSxSTR, sock.RemoteIpStr(buf));
-
   fs = fdopen(sock.Descriptor(), "r+");
   fprintf(fs, "Licq Remote Management Server v%s\n"
      "%d Enter your UIN:\n", LP_Version(), CODE_ENTERxUIN);
   fflush(fs);
 
+  m_szCheckId = 0;
+  m_szId = 0;
   m_nState = STATE_UIN;
   m_nLogTypes = 0;
   data_line_pos = 0;
+  m_bNotify = false;
+  m_szEventId = 0;
+  m_nEventPPID = 0;
 }
 
 
@@ -390,8 +484,85 @@ CRMSClient::CRMSClient(TCPSocket *sin) : sock(0)
 CRMSClient::~CRMSClient()
 {
   sockman.CloseSocket(sock.Descriptor(), false, false);
+  
+  if (m_szCheckId)
+    free(m_szCheckId);
 }
 
+/*---------------------------------------------------------------------------
+ * CRMSClient::AddEventTag
+ *-------------------------------------------------------------------------*/
+void CRMSClient::AddEventTag(const char *_szId, unsigned long _nPPID, unsigned long _nEventTag)
+{
+  if (m_szEventId && m_nEventPPID &&
+       !strcmp(m_szEventId, _szId) && m_nEventPPID == _nPPID)
+  {
+    fprintf(fs, "%d [%ld] Sending message to %s.\n", CODE_COMMANDxSTART,
+       _nEventTag, _szId);
+    tags.push_back(_nEventTag);
+	free(m_szEventId);
+	m_szEventId = 0;
+	m_nEventPPID = 0;
+  }
+}
+
+/*---------------------------------------------------------------------------
+ * CRMSClient::GetProtocol
+ *-------------------------------------------------------------------------*/
+unsigned long CRMSClient::GetProtocol(const char *szData)
+{
+  unsigned long nPPID = 0;
+  ProtoPluginsList pl;
+  ProtoPluginsListIter it;
+  licqDaemon->ProtoPluginList(pl);
+  for (it = pl.begin(); it != pl.end(); it++)
+  {
+    if (strcasecmp((*it)->Name(), szData) == 0)
+    {
+      nPPID = (*it)->PPID();
+      break;
+    }
+  }
+  
+  return nPPID;
+}
+
+/*---------------------------------------------------------------------------
+ * CRMSClient::ParseUser
+ *-------------------------------------------------------------------------*/
+void CRMSClient::ParseUser(const char *szData)
+{
+  if (m_szId)
+    free(m_szId);
+    
+  string strData(szData);
+  string::size_type nPos= strData.find_last_of(".");
+  if (nPos == string::npos)
+  {
+    m_szId = strdup(data_arg);
+    m_nPPID = 0;
+    ProtoPluginsList pl;
+    ProtoPluginsListIter it;
+    licqDaemon->ProtoPluginList(pl);
+    for (it = pl.begin(); it != pl.end(); it++)
+    {
+      ICQUser *u = gUserManager.FetchUser(m_szId, (*it)->PPID(), LOCK_R);
+      if (u)
+      {
+        gUserManager.DropUser(u);
+        m_nPPID = (*it)->PPID();
+        break;
+      }
+    }
+  }
+  else
+  {
+    string strId(strData, 0, strData.find_last_of("."));
+    string strProtocol(strData, strData.find_last_of(".")+1, strData.size());
+    m_szId = strdup(strId.c_str());
+    m_nPPID = GetProtocol(strProtocol.c_str());
+  }
+}
 
 /*---------------------------------------------------------------------------
  * CRMSClient::ProcessEvent
@@ -492,7 +663,7 @@ int CRMSClient::StateMachine()
   {
     case STATE_UIN:
     {
-      m_nCheckUin = atol(data_line);
+      m_szCheckId = data_line ? strdup(data_line) : 0;
       fprintf(fs, "%d Enter your password:\n", CODE_ENTERxPASSWORD);
       fflush(fs);
       m_nState = STATE_PASSWORD;
@@ -501,14 +672,16 @@ int CRMSClient::StateMachine()
     case STATE_PASSWORD:
     {
       ICQOwner *o = gUserManager.FetchOwner(LOCK_R);
-      bool ok = (m_nCheckUin == o->Uin() &&
-         strcmp(o->Password(), data_line) == 0);
+      bool ok = (strcmp(m_szCheckId, o->IdString()) == 0 &&
+         (strcmp(o->Password(), data_line) == 0));
+      free(m_szCheckId);
+      m_szCheckId = 0;
       if (!ok)
       {
         gUserManager.DropOwner();
         gLog.Info("%sClient failed validation from %s.\n", L_RMSxSTR,
            sock.RemoteIpStr(buf));
-        fprintf(fs, "%d Invalid UIN/Password.\n", CODE_INVALID);
+        fprintf(fs, "%d Invalid ID/Password.\n", CODE_INVALID);
         fflush(fs);
         return -1;
       }
@@ -542,6 +715,16 @@ int CRMSClient::StateMachine()
     {
       return Process_URL_url();
     }
+    case STATE_ENTERxSMSxMESSAGE:
+    {
+      if (AddLineToText())
+         return Process_SMS_message();
+      break;
+    }
+    case STATE_ENTERxSMSxNUMBER:
+    {
+      return Process_SMS_number();
+    }
     case STATE_ENTERxAUTOxRESPONSE:
     {
       if (AddLineToText())
@@ -596,27 +779,40 @@ int CRMSClient::ProcessCommand()
  *-------------------------------------------------------------------------*/
 int CRMSClient::Process_INFO()
 {
-  unsigned long nUin = atol(data_arg);
+  char *szId = strdup(data_arg);
+  NEXT_WORD(data_arg);
+  unsigned long nPPID = GetProtocol(data_arg);
 
-  if (nUin == 0) nUin = gUserManager.OwnerUin();
+  //XXX Handle the case when we have the owner
+  if (szId == 0)
+    m_nUin = gUserManager.OwnerUin();
 
   // Print the user info
-  ICQUser *u = gUserManager.FetchUser(nUin, LOCK_R);
+  ICQUser *u = gUserManager.FetchUser(szId, nPPID, LOCK_R);
   if (u == NULL)
   {
     fprintf(fs, "%d No such user.\n", CODE_INVALIDxUSER);
     return fflush(fs);
   }
 
-  fprintf(fs, "%d %ld Alias: %s\n", CODE_USERxINFO, u->Uin(), u->GetAlias());
-  fprintf(fs, "%d %ld Status: %s\n", CODE_USERxINFO, u->Uin(), u->StatusStr());
-  fprintf(fs, "%d %ld First Name: %s\n", CODE_USERxINFO, u->Uin(), u->GetFirstName());
-  fprintf(fs, "%d %ld Last Name: %s\n", CODE_USERxINFO, u->Uin(), u->GetLastName());
-  fprintf(fs, "%d %ld Email 1: %s\n", CODE_USERxINFO, u->Uin(), u->GetEmailPrimary());
-  fprintf(fs, "%d %ld Email 2: %s\n", CODE_USERxINFO, u->Uin(), u->GetEmailSecondary());
+  fprintf(fs, "%d %s Alias: %s\n", CODE_USERxINFO, u->IdString(),
+    u->GetAlias());
+  fprintf(fs, "%d %s Status: %s\n", CODE_USERxINFO, u->IdString(),
+    u->StatusStr());
+  fprintf(fs, "%d %s First Name: %s\n", CODE_USERxINFO, u->IdString(),
+    u->GetFirstName());
+  fprintf(fs, "%d %s Last Name: %s\n", CODE_USERxINFO, u->IdString(),
+    u->GetLastName());
+  fprintf(fs, "%d %s Email 1: %s\n", CODE_USERxINFO, u->IdString(),
+    u->GetEmailPrimary());
+  fprintf(fs, "%d %s Email 2: %s\n", CODE_USERxINFO, u->IdString(),
+    u->GetEmailSecondary());
 
   gUserManager.DropUser(u);
 
+  if (szId)
+    free(szId);
+    
   return fflush(fs);
 }
 
@@ -625,7 +821,7 @@ int CRMSClient::Process_INFO()
  * CRMSClient::Process_STATUS
  *
  * Command:
- *   STATUS [ status ]
+ *   STATUS [ status | protocol ]
  *
  * Response:
  *
@@ -635,57 +831,94 @@ int CRMSClient::Process_STATUS()
   // Show status
   if (data_arg[0] == '\0')
   {
-    ICQOwner *o = gUserManager.FetchOwner(LOCK_R);
-    fprintf(fs, "%d %s\n", CODE_STATUS, o->StatusStr());
-    gUserManager.DropOwner();
+    ProtoPluginsList l;
+    ProtoPluginsListIter it;
+    licqDaemon->ProtoPluginList(l);
+    for (it = l.begin(); it != l.end(); it++)
+    {
+      ICQOwner *o = gUserManager.FetchOwner((*it)->PPID(), LOCK_R);
+      if (o)
+      {
+        fprintf(fs, "%d %s %s %s\n", CODE_STATUS, o->IdString(), (*it)->Name(), o->StatusStr());
+        gUserManager.DropOwner((*it)->PPID());
+      }
+    }
+    fprintf(fs, "%d\n", CODE_STATUSxDONE);
     return fflush(fs);
   }
 
   // Set status
-  unsigned long nStatus = StringToStatus(data_arg);
-  if (nStatus == INT_MAX)
+  string strData(data_arg);
+  string::size_type nPos = strData.find_last_of(".");
+  if (nPos == string::npos)
   {
-    fprintf(fs, "%d Invalid status.\n", CODE_INVALIDxSTATUS);
-    return fflush(fs);
-  }
-
-  if (nStatus == ICQ_STATUS_OFFLINE)
-  {
-    fprintf(fs, "%d [0] Logging off.\n", CODE_COMMANDxSTART);
-    fflush(fs);
-    licqDaemon->icqLogoff();
-    fprintf(fs, "%d [0] Event done.\n", CODE_RESULTxSUCCESS);
-    return fflush(fs);
-  }
-
-  ICQOwner *o = gUserManager.FetchOwner(LOCK_R);
-  bool b = o->StatusOffline();
-  gUserManager.DropOwner();
-  unsigned long tag = 0;
-  if (b)
-  {
-    tag = licqDaemon->icqLogon(nStatus);
-    fprintf(fs, "%d [%ld] Logging on.\n", CODE_COMMANDxSTART, tag);
+    unsigned long nStatus = StringToStatus(data_arg);
+    ProtoPluginsList l;
+    ProtoPluginsListIter it;
+    licqDaemon->ProtoPluginList(l);
+    for (it = l.begin(); it != l.end(); it++)
+    {
+      ChangeStatus((*it)->PPID(), nStatus, data_arg);
+    }
   }
   else
   {
-    tag = licqDaemon->icqSetStatus(nStatus);
-    fprintf(fs, "%d [%ld] Setting status.\n", CODE_COMMANDxSTART, tag);
+    string strStatus(strData, 0, strData.find_last_of("."));
+    string strProtocol(strData, strData.find_last_of(".")+1, strData.size());
+    unsigned long nPPID = GetProtocol(strProtocol.c_str());
+    char *szStatus = strdup(strStatus.c_str());
+    unsigned long nStatus = StringToStatus(szStatus);
+    ChangeStatus(nPPID, nStatus, szStatus);
+    free(szStatus);
   }
-  tags.push_back(tag);
-
+  fprintf(fs, "%d Done setting status\n", CODE_STATUSxDONE);
   return fflush(fs);
 }
 
+int CRMSClient::ChangeStatus(unsigned long nPPID, unsigned long nStatus, const char *szStatus)
+{
+  if (nStatus == INT_MAX)
+  {
+    fprintf(fs, "%d Invalid status.\n", CODE_INVALIDxSTATUS);
+    return -1;
+  }
+  if (nStatus == ICQ_STATUS_OFFLINE)
+  {
+    fprintf(fs, "%d [0] Logging off %s.\n", CODE_COMMANDxSTART, szStatus);
+    fflush(fs);
+    licqDaemon->ProtoLogoff(nPPID);
+    fprintf(fs, "%d [0] Event done.\n", CODE_STATUSxDONE);
+    return 0;
+  }
+  else
+  {
+    ICQOwner *o = gUserManager.FetchOwner(nPPID, LOCK_R);
+    bool b = o->StatusOffline();
+    gUserManager.DropOwner(nPPID);
+    unsigned long tag = 0;
+    if (b)
+    {
+      tag = licqDaemon->ProtoLogon(nPPID, nStatus);
+      fprintf(fs, "%d [%ld] Logging on to %s.\n", CODE_COMMANDxSTART, tag, szStatus);
+    }
+    else
+    {
+      tag = licqDaemon->ProtoSetStatus(nPPID, nStatus);
+      fprintf(fs, "%d [%ld] Setting status for %s.\n", CODE_COMMANDxSTART, tag, szStatus);
+    }
+    tags.push_back(tag);
+  }
+  return 0;
+}
 
 /*---------------------------------------------------------------------------
  * CRMSClient::Process_QUIT
  *-------------------------------------------------------------------------*/
 int CRMSClient::Process_QUIT()
 {
-  fprintf(fs, "%d Aurevoir.\n", CODE_QUIT);
+  fprintf(fs, "%d Sayonara.\n", CODE_QUIT);
   fflush(fs);
-  if (atol(data_arg) > 0) licqRMS->m_bExit = true;
+  if (strtoul(data_arg, (char**)NULL, 10) > 0) licqRMS->m_bExit = true;
   return -1;
 }
 
@@ -750,14 +983,14 @@ int CRMSClient::Process_GROUPS()
  *     All options are optional and can be left out arbitrarily, ie
  *     "LIST all" is a valid call and will print all online and offline users.
  *     <format> is a printf style string using the user % symbols as
- *     documented in UTILITIES.HOWTO.  The default is "%9u %-20a %3m %s"
+ *     documented in UTILITIES.HOWTO.  The default is "%u %P %-20a %3m %s"
  *     and prints out users as follows.
  *
  * Response:
- *   CODE_LISTxUSER   5550000              AnAlias   2 Online
- *     The default line contains the uin, alias, number of new messages
- *     and status all column and white space deliminated.  Note that the
- *     alias may contain white space.
+ *   CODE_LISTxUSER   5550000 Licq            AnAlias   2 Online
+ *     The default line contains the uin, protocol, alias, number of new 
+ *     messages and status all column and white space deliminated.  Note that
+ *     the alias may contain white space.
  *   CODE_LISTxUSER ...
  *   ...
  *   CODE_LISTxDONE
@@ -768,7 +1001,7 @@ int CRMSClient::Process_LIST()
   unsigned short nGroup = 0;
   if (isdigit(*data_arg))
   {
-    nGroup = atol(data_arg);
+    nGroup = strtoul(data_arg, (char**)NULL, 10);
     while (*data_arg != '\0' && *data_arg != ' ') data_arg++;
     NEXT_WORD(data_arg);
   }
@@ -794,7 +1027,7 @@ int CRMSClient::Process_LIST()
   char format[128], *ubuf;
   if (*data_arg == '\0')
   {
-    strcpy(format, "%9u %-20a %3m %s");
+    strcpy(format, "%u %P %-20a %3m %s");
   }
   else
   {
@@ -823,7 +1056,7 @@ int CRMSClient::Process_LIST()
  * CRMSClient::Process_MESSAGE
  *
  * Command:
- *     MESSAGE <uin>
+ *     MESSAGE <id>[.<protocol>]
  *
  * Response:
  *   CODE_ENTERxTEXT | CODE_INVALIDxUSER
@@ -836,18 +1069,18 @@ int CRMSClient::Process_LIST()
  *-------------------------------------------------------------------------*/
 int CRMSClient::Process_MESSAGE()
 {
-  unsigned long nUin = atol(data_arg);
-
-  if (nUin < 10000)
+  if (m_nEventPPID || m_szEventId)
   {
-    fprintf(fs, "%d Invalid UIN.\n", CODE_INVALIDxUSER);
-    return fflush(fs);
+    //client is trying to send another message before we've received
+    //the event tag for a previous one
+    fprintf(fs, "%d Error, cannot send concurrent messages\n", CODE_INVALIDxCOMMAND);
+    return -1;
   }
-
   fprintf(fs, "%d Enter message, terminate with a . on a line by itself:\n",
      CODE_ENTERxTEXT);
 
-  m_nUin = nUin;
+  ParseUser(data_arg);
+
   m_szText[0] = '\0';
   m_nTextPos = 0;
 
@@ -857,14 +1090,24 @@ int CRMSClient::Process_MESSAGE()
 
 int CRMSClient::Process_MESSAGE_text()
 {
-  unsigned long tag = licqDaemon->icqSendMessage(m_nUin, m_szText, false, ICQ_TCPxMSG_NORMAL);
+  //XXX Give a tag...
+  m_szText[strlen(m_szText) - 1] = '\0';
+  unsigned long tag = licqDaemon->ProtoSendMessage(m_szId, m_nPPID, m_szText,
+    false, ICQ_TCPxMSG_NORMAL);
 
-  fprintf(fs, "%d [%ld] Sending message to %ld.\n", CODE_COMMANDxSTART,
-     tag, m_nUin);
-
-  tags.push_back(tag);
   m_nState = STATE_COMMAND;
 
+  if (m_nPPID == LICQ_PPID)
+  {
+    fprintf(fs, "%d [%ld] Sending message to %s.\n", CODE_COMMANDxSTART,
+       tag, m_szId);
+    tags.push_back(tag);
+  }
+  else
+  {
+    m_nEventPPID = m_nPPID;
+    m_szEventId = strdup(m_szId);
+  }
   return fflush(fs);
 }
 
@@ -874,7 +1117,7 @@ int CRMSClient::Process_MESSAGE_text()
  * CRMSClient::Process_URL
  *
  * Command:
- *   URL <uin>
+ *   URL <id>[.<protocol>]
  *
  * Response:
  *   CODE_ENTERxLINE | CODE_INVALIDxUSER
@@ -890,16 +1133,8 @@ int CRMSClient::Process_MESSAGE_text()
  *-------------------------------------------------------------------------*/
 int CRMSClient::Process_URL()
 {
-  unsigned long nUin = atol(data_arg);
-
-  if (nUin < 10000)
-  {
-    fprintf(fs, "%d Invalid UIN.\n", CODE_INVALIDxUSER);
-    return fflush(fs);
-  }
-  fprintf(fs, "%d Enter URL:\n", CODE_ENTERxLINE);
-
-  m_nUin = nUin;
+  ParseUser(data_arg);
+  
   m_nTextPos = 0;
 
   m_nState = STATE_ENTERxURL;
@@ -924,10 +1159,79 @@ int CRMSClient::Process_URL_url()
 
 int CRMSClient::Process_URL_text()
 {
-  unsigned long tag = licqDaemon->icqSendUrl(m_nUin, m_szLine, m_szText, false, ICQ_TCPxMSG_NORMAL);
+  unsigned long tag = licqDaemon->ProtoSendUrl(m_szId, m_nPPID, m_szLine,
+    m_szText, false, ICQ_TCPxMSG_NORMAL);
 
-  fprintf(fs, "%d [%ld] Sending URL to %ld.\n", CODE_COMMANDxSTART,
-     tag, m_nUin);
+  fprintf(fs, "%d [%ld] Sending URL to %s.\n", CODE_COMMANDxSTART,
+     tag, m_szId);
+
+  if (m_nPPID == LICQ_PPID)
+    tags.push_back(tag);
+    
+  m_nState = STATE_COMMAND;
+
+  return fflush(fs);
+}
+
+
+/*---------------------------------------------------------------------------
+ * CRMSClient::Process_SMS
+ *
+ * Command:
+ *   SMS <uin>
+ *
+ * Response:
+ *   CODE_ENTERxLINE | CODE_INVALIDxUSER
+ *     At which point the phone number should be entered on a line by itself
+ *     without the "+", but including country code.  Invalid user means the
+ *     uin was invalid (< 10000) and the url was aborted.
+ *   CODE_ENTERxTEXT
+ *     Now the message should be entered and terminated by a "." on a line
+ *     by itself.
+ *   CODE_COMMANDxSTART
+ *     < ...time... >
+ *   CODE_RESULTxSUCCESS | CODE_EVENTxTIMEDOUT | CODE_EVENTxERROR
+ *-------------------------------------------------------------------------*/
+int CRMSClient::Process_SMS()
+{
+  unsigned long nUin = strtoul(data_arg, (char**)NULL, 10);
+
+  if (nUin < 10000)
+  {
+    fprintf(fs, "%d Invalid UIN.\n", CODE_INVALIDxUSER);
+    return fflush(fs);
+  }
+  fprintf(fs, "%d Enter NUMBER:\n", CODE_ENTERxLINE);
+
+  m_nUin = nUin;
+  m_nTextPos = 0;
+
+  m_nState = STATE_ENTERxSMSxNUMBER;
+  return fflush(fs);
+}
+
+
+int CRMSClient::Process_SMS_number()
+{
+  strcpy(m_szLine, data_line);
+
+  fprintf(fs, "%d Enter message, terminate with a . on a line by itself:\n",
+     CODE_ENTERxTEXT);
+
+  m_szText[0] = '\0';
+  m_nTextPos = 0;
+
+  m_nState = STATE_ENTERxSMSxMESSAGE;
+  return fflush(fs);
+}
+
+
+int CRMSClient::Process_SMS_message()
+{
+  unsigned long tag = licqDaemon->icqSendSms(m_szLine,m_szText,m_nUin);
+
+  fprintf(fs, "%d [%lu] Sending SMS to %lu (%s).\n", CODE_COMMANDxSTART,
+     tag, m_nUin, m_szLine);
 
   tags.push_back(tag);
   m_nState = STATE_COMMAND;
@@ -940,7 +1244,7 @@ int CRMSClient::Process_URL_text()
  * CRMSClient::Process_AR
  *
  * Command:
- *     AR [ <uin> ]
+ *     AR [ <id>[.<protocol>] ]
  *
  * Response:
  *   CODE_ENTERxTEXT | CODE_INVALIDxUIN
@@ -950,27 +1254,17 @@ int CRMSClient::Process_URL_text()
  *-------------------------------------------------------------------------*/
 int CRMSClient::Process_AR()
 {
-/*
-  if (data_arg[0] == '\0')
-  {
-    ICQOwner *o = gUserManager.FetchOwner(LOCK_R);
-    // print...
-    gUserManager.DropOwner();
-    return fflush(fs);
-  }
-*/
-  unsigned long nUin = atol(data_arg);
+  ParseUser(data_arg);
 
-  if (nUin != 0 && !gUserManager.IsOnList(nUin))
+  if (m_szId && !gUserManager.IsOnList(m_szId, m_nPPID))
   {
-    fprintf(fs, "%d Invalid UIN.\n", CODE_INVALIDxUSER);
+    fprintf(fs, "%d Invalid User.\n", CODE_INVALIDxUSER);
     return fflush(fs);
   }
 
   fprintf(fs, "%d Enter %sauto response, terminate with a . on a line by itself:\n",
-     CODE_ENTERxTEXT, nUin == 0 ? "" : "custom " );
+     CODE_ENTERxTEXT, m_szId == 0 ? "" : "custom " );
 
-  m_nUin = nUin;
   m_szText[0] = '\0';
   m_nTextPos = 0;
 
@@ -980,15 +1274,15 @@ int CRMSClient::Process_AR()
 
 int CRMSClient::Process_AR_text()
 {
-  if (m_nUin == 0)
+  if (m_szId == 0)
   {
-    ICQOwner *o = gUserManager.FetchOwner(LOCK_W);
+    ICQOwner *o = gUserManager.FetchOwner(m_nPPID, LOCK_W);
     o->SetAutoResponse(m_szText);
     gUserManager.DropOwner();
   }
   else
   {
-    ICQUser *u = gUserManager.FetchUser(m_nUin, LOCK_W);
+    ICQUser *u = gUserManager.FetchUser(m_szId, m_nPPID, LOCK_W);
     u->SetCustomAutoResponse(m_szText);
     gUserManager.DropUser(u);
   }
@@ -1013,7 +1307,7 @@ int CRMSClient::Process_AR_text()
  *-------------------------------------------------------------------------*/
 int CRMSClient::Process_LOG()
 {
-  unsigned short lt = atol(data_arg);
+  unsigned short lt = strtoul(data_arg, (char**)NULL, 10);
 
   if (licqRMS->log == NULL)
   {
@@ -1029,23 +1323,42 @@ int CRMSClient::Process_LOG()
   return fflush(fs);
 }
 
+/*---------------------------------------------------------------------------
+ * CRMSClient::Process_NOTIFY
+ *
+ * Command:
+ *   NOTIFY
+ *
+ * Response:
+ *   CODE_NOTIFYxON|CODE_NOTIFYxOFF
+ *   ...
+ *-------------------------------------------------------------------------*/
+int CRMSClient::Process_NOTIFY()
+{
+  m_bNotify = !m_bNotify;
+
+  if (m_bNotify)
+    fprintf(fs, "%d Notify set ON.\n", CODE_NOTIFYxON);
+  else
+    fprintf(fs, "%d Notify set OFF.\n", CODE_NOTIFYxOFF);
+
+  return fflush(fs);
+}
 
 /*---------------------------------------------------------------------------
  * CRMSClient::Process_VIEW
  *
  * Command:
- *   VIEW <uin>
+ *   VIEW [ <id>[.<protocol> ]
  *
  * Response:
  *
  *-------------------------------------------------------------------------*/
 int CRMSClient::Process_VIEW()
 {
-  unsigned long nUin = 0;
-
   if (*data_arg != '\0')
   {
-    nUin = atol(data_arg);
+    ParseUser(data_arg);
   }
   else
   {
@@ -1055,18 +1368,24 @@ int CRMSClient::Process_VIEW()
     FOR_EACH_USER_START(LOCK_R)
     {
       if(pUser->NewMessages() > 0)
-        nUin = pUser->Uin();
+      {
+        if (m_szId)
+          free(m_szId);
+        m_szId = strdup(pUser->IdString());
+        m_nPPID = pUser->PPID();
+        FOR_EACH_USER_BREAK
+      }
     }
     FOR_EACH_USER_END
   
-    if (nUin == 0)
+    if (m_szId == 0)
     {
       fprintf(fs, "%d No new messages.\n", CODE_VIEWxNONE);
       return fflush(fs);
     }
   }
 
-  ICQUser *u = gUserManager.FetchUser(nUin, LOCK_W);
+  ICQUser *u = gUserManager.FetchUser(m_szId, m_nPPID, LOCK_W);
   if (u == NULL)
   {
     fprintf(fs, "%d No such user.\n", CODE_INVALIDxUSER);
@@ -1119,7 +1438,8 @@ int CRMSClient::Process_VIEW()
 
     // Message
     fprintf(fs, "%d Message Start\n", CODE_VIEWxTEXTxSTART);
-    fprintf(fs, e->Text());
+    fprintf(fs, "%s", e->Text());
+    fprintf(fs, "\n");
     fprintf(fs, "%d Message Complete\n", CODE_VIEWxTEXTxEND);
   }
   else
@@ -1128,8 +1448,125 @@ int CRMSClient::Process_VIEW()
   }
 
   gUserManager.DropUser(u);
+    
+  return fflush(fs);
+}
+
+/*---------------------------------------------------------------------------
+ * CRMSClient::Process_ADDUSER
+ *
+ * Command:
+ *   ADDUSER <id> <protocol>
+ *
+ * Response:
+ *
+ *-------------------------------------------------------------------------*/
+int CRMSClient::Process_ADDUSER()
+{
+  char *szId = strdup(data_arg);
+  NEXT_WORD(data_arg);
+  unsigned long nPPID = GetProtocol(data_arg);
+
+  if (licqDaemon->AddUserToList(szId, nPPID))
+  {
+    fprintf(fs, "%d User added\n", CODE_ADDUSERxDONE);
+  }
+  else
+  {
+    fprintf(fs, "%d User not added\n", CODE_ADDUSERxERROR);
+  }
+  
+  return fflush(fs);
+}
+
+/*---------------------------------------------------------------------------
+ * CRMSClient::Process_REMUSER
+ *
+ * Command:
+ *   REMUSER <uin>
+ *
+ * Response:
+ *
+ *-------------------------------------------------------------------------*/
+int CRMSClient::Process_REMUSER()
+{
+  unsigned long nUin = strtoul(data_arg, (char**)NULL, 10);
+
+  if (nUin >= 10000)
+  {
+    licqDaemon->RemoveUserFromList(nUin);
+    fprintf(fs, "%d User removed\n", CODE_REMUSERxDONE);
+  }
+  else
+  {
+    fprintf(fs, "%d Invalid UIN.\n", CODE_INVALIDxUSER);
+  }
 
   return fflush(fs);
 }
 
+/*---------------------------------------------------------------------------
+ * CRMSClient::Process_SECURE
+ *
+ * Command:
+ *   SECURE <uin> <what>
+ *
+ * Response:
+ *
+ *-------------------------------------------------------------------------*/
+int CRMSClient::Process_SECURE()
+{
+  unsigned long nUin = 0;
 
+  if(!licqDaemon->CryptoEnabled())
+  {
+    fprintf(fs, "%d Licq secure channel not compiled. Please recompile with OpenSSL.\n", CODE_SECURExNOTCOMPILED);
+    return fflush(fs);
+  }
+  
+
+  if (isdigit(*data_arg))
+  {
+    nUin = strtoul(data_arg, (char**)NULL, 10);
+    while (*data_arg != '\0' && *data_arg != ' ') data_arg++;
+    NEXT_WORD(data_arg);
+  }
+   else
+  {
+    fprintf(fs, "%d Invalid UIN.\n", CODE_INVALIDxUSER);
+    return fflush(fs);
+  }
+
+  if (nUin < 10000)
+  {
+    fprintf(fs, "%d Invalid UIN.\n", CODE_INVALIDxUSER);
+    return fflush(fs);
+  }
+
+  if (strncasecmp(data_arg, "open", 4) == 0)
+  {
+    fprintf(fs, "%d Opening secure connection.\n", CODE_SECURExOPEN);
+    licqDaemon->icqOpenSecureChannel(nUin);
+  }
+  else
+  if (strncasecmp(data_arg, "close", 5) == 0)
+  {
+    fprintf(fs, "%d Closing secure connection.\n", CODE_SECURExCLOSE);
+    licqDaemon->icqCloseSecureChannel(nUin);
+  }
+  else
+  {
+   ICQUser *u = gUserManager.FetchUser(nUin,LOCK_R);
+   if (u->Secure() == 0)
+   {
+    fprintf(fs, "%d Status: secure connection is closed.\n", CODE_SECURExSTAT);
+   }
+   if (u->Secure() == 1)
+   {
+    fprintf(fs, "%d Status: secure connection is open.\n", CODE_SECURExSTAT);
+   }
+   gUserManager.DropUser(u);
+  }
+  
+  return fflush(fs);
+}
