@@ -17,8 +17,12 @@
 #include <arpa/inet.h>
 #include <ctype.h>
 
+// Localization
+#include "gettext.h"
+
 #include "licq_buffer.h"
 #include "licq_log.h"
+#include "support.h"
 
 //=====Utilities=================================================================
 
@@ -321,6 +325,13 @@ CBuffer& CBuffer::operator>>(unsigned long &in)
   return(*this);
 }
 
+char *CBuffer::UnpackRaw(char *sz, unsigned short _nSize)
+{
+  for (unsigned short i = 0; i < _nSize; i++) *this >> sz[i];
+  sz[_nSize] = '\0';
+  return sz;
+}
+
 char *CBuffer::UnpackStringBE(char* sz, unsigned short _usiSize)
 {
   unsigned short nLen;
@@ -369,6 +380,18 @@ char *CBuffer::UnpackString()
   return sz;
 }
 
+// Need to dlete[] returned string
+char *CBuffer::UnpackUserString()
+{
+  unsigned char nLen;
+  *this >> nLen;
+  char *sz = new char[nLen+1];
+  sz[0] = '\0';
+  for (unsigned char i = 0; i < nLen; i++) *this >> sz[i];
+  sz[nLen] = '\0';
+  return sz;
+}
+
 unsigned long CBuffer::UnpackUnsignedLong()
 {
   unsigned long n;
@@ -482,8 +505,8 @@ char *CBuffer::PackUnsignedLong(unsigned long data)
 {
   if ( getDataSize() + 4 > getDataMaxSize() )
   {
-    gLog.Warn("%sPackUnsignedLong(): Trying to pack more data than "
-              "CBuffer can hold!\n", L_WARNxSTR);
+    gLog.Warn(tr("%sPackUnsignedLong(): Trying to pack more data than "
+                 "CBuffer can hold!\n"), L_WARNxSTR);
     return getDataPosWrite();
   }
   put_le_long(getDataPosWrite(), data);
@@ -495,8 +518,8 @@ char *CBuffer::PackUnsignedLongBE(unsigned long data)
 {
   if (getDataSize() + 4 > getDataMaxSize() )
   {
-    gLog.Warn("%sPackUnsignedLongBE(): Trying to pack more data than "
-              "CBuffer can hold!\n", L_WARNxSTR);
+    gLog.Warn(tr("%sPackUnsignedLongBE(): Trying to pack more data than "
+                 "CBuffer can hold!\n"), L_WARNxSTR);
     return getDataPosWrite();
   }
   put_be_long(getDataPosWrite(), data);
@@ -508,8 +531,8 @@ char *CBuffer::PackChar(char data)
 {
   if (getDataSize() + 1 > getDataMaxSize())
   {
-    gLog.Warn("%sPackChar(): Trying to pack more data than "
-              "CBuffer can hold!\n", L_WARNxSTR);
+    gLog.Warn(tr("%sPackChar(): Trying to pack more data than "
+                 "CBuffer can hold!\n"), L_WARNxSTR);
     return getDataPosWrite();
   }
   *getDataPosWrite() = data;
@@ -521,8 +544,8 @@ char *CBuffer::Pack(const char *data, int size)
 {
   if ( getDataSize() + size > getDataMaxSize() )
   {
-    gLog.Warn("%sPack(): Trying to pack more data than "
-              "CBuffer can hold!\n", L_WARNxSTR);
+    gLog.Warn(tr("%sPack(): Trying to pack more data than "
+                 "CBuffer can hold!\n"), L_WARNxSTR);
     return getDataPosWrite();
   }
   if (!size) return getDataPosWrite();
@@ -535,8 +558,8 @@ char *CBuffer::Pack(CBuffer *buf)
 {
   if ( getDataSize() + buf->getDataSize() > getDataMaxSize() )
   {
-    gLog.Warn("%sPack(): Trying to pack more data than "
-              "CBuffer can hold!\n", L_WARNxSTR);
+    gLog.Warn(tr("%sPack(): Trying to pack more data than "
+                 "CBuffer can hold!\n"), L_WARNxSTR);
     return getDataPosWrite();
   }
   memcpy(getDataPosWrite(), buf->getDataStart(), buf->getDataSize());
@@ -559,8 +582,8 @@ char *CBuffer::PackString(const char *data, unsigned short max)
   if (max > 0 && n > max) n = max;
   if ( getDataSize()  + n + 1 > getDataMaxSize() )
   {
-    gLog.Warn("%sPackString(): Trying to pack more data than "
-              "CBuffer can hold!\n", L_WARNxSTR);
+    gLog.Warn(tr("%sPackString(): Trying to pack more data than "
+                 "CBuffer can hold!\n"), L_WARNxSTR);
     return getDataPosWrite();
   }
   put_le_short(getDataPosWrite(), n + 1);
@@ -579,8 +602,8 @@ char *CBuffer::PackUnsignedShort(unsigned short data)
 {
   if ( getDataSize() + 2 > getDataMaxSize() )
   {
-    gLog.Warn("%sPackUnsignedShort(): Trying to pack more data than "
-              "CBuffer can hold!\n", L_WARNxSTR);
+    gLog.Warn(tr("%sPackUnsignedShort(): Trying to pack more data than "
+                 "CBuffer can hold!\n"), L_WARNxSTR);
     return getDataPosWrite();
   }
   put_le_short(getDataPosWrite(), data);
@@ -592,8 +615,8 @@ char *CBuffer::PackUnsignedShortBE(unsigned short data)
 {
   if ( getDataSize() + 2 > getDataMaxSize() )
   {
-    gLog.Warn("%sPackUnsignedShortBE(): Trying to pack more data than "
-              "CBuffer can hold!\n", L_WARNxSTR);
+    gLog.Warn(tr("%sPackUnsignedShortBE(): Trying to pack more data than "
+                 "CBuffer can hold!\n"), L_WARNxSTR);
     return getDataPosWrite();
   }
   put_be_short(getDataPosWrite(), data);
@@ -645,7 +668,7 @@ bool CBuffer::readTLV(int nCount, int nBytes)
 
     if (nBytes > 0 && nCurBytes > nBytes)
     {
-      gLog.Warn("%sRead too much TLV data!\n", L_WARNxSTR);
+      gLog.Warn(tr("%sRead too much TLV data!\n"), L_WARNxSTR);
       return true;
     }
   }
@@ -653,7 +676,7 @@ bool CBuffer::readTLV(int nCount, int nBytes)
   // Finish off the number of bytes we wanted
   if (nCurBytes < nBytes)
   { 
-    gLog.Warn("%sUnable to read requested amount of TLV data!\n", L_WARNxSTR);
+    gLog.Warn(tr("%sUnable to read requested amount of TLV data!\n"), L_WARNxSTR);
     for (; nCurBytes < nBytes; nCurBytes++)
       UnpackChar();
   }
