@@ -75,6 +75,53 @@ void CDaemonStats::Init()
 }
 
 
+unsigned long CConversation::s_nCID = 0;
+pthread_mutex_t CConversation::s_xMutex = PTHREAD_MUTEX_INITIALIZER;
+
+CConversation::CConversation(int nSocket, unsigned long nPPID)
+{
+  m_nSocket = nSocket;
+  m_nPPID = nPPID;
+  pthread_mutex_lock(&s_xMutex);
+  m_nCID = ++s_nCID;
+  pthread_mutex_unlock(&s_xMutex);
+}
+
+bool CConversation::HasUser(const char *szUser)
+{
+  return (std::find(m_vUsers.begin(), m_vUsers.end(), szUser) != m_vUsers.end()); 
+}
+
+bool CConversation::AddUser(const char *szUser)
+{
+  if (!HasUser(szUser))
+  {
+    m_vUsers.push_back(szUser);
+    return true;
+  }
+  else
+    return false;
+}
+
+bool CConversation::RemoveUser(const char *szUser)
+{
+  if (HasUser(szUser))
+  {
+    string strUser(szUser);
+    vector<string>::iterator i;
+    for (i = m_vUsers.begin(); i != m_vUsers.end(); ++i)
+    {
+      if (*i == strUser)
+      {
+        m_vUsers.erase(i);
+        return true;
+      }
+    }
+  }
+  
+  return false;
+}
+
 CICQDaemon *gLicqDaemon = NULL;
 
 
@@ -101,6 +148,7 @@ CICQDaemon::CICQDaemon(CLicq *_licq)
   m_nServerAck = 0;
   m_bLoggingOn = false;
   m_bOnlineNotifies = true;
+  m_bVerify = false;
   m_szRegisterPasswd = 0;
   m_nRegisterThreadId = 0;
 
@@ -307,6 +355,7 @@ CICQDaemon::CICQDaemon(CLicq *_licq)
   pthread_mutex_init(&mutex_cancelthread, NULL);
   pthread_cond_init(&cond_serverack, NULL);
   pthread_mutex_init(&mutex_serverack, NULL);
+  pthread_mutex_init(&mutex_conversations, NULL);
 }
 
 
@@ -533,6 +582,7 @@ void CICQDaemon::ProtoPluginList(ProtoPluginsList &lPlugins)
 
 bool CICQDaemon::ProtoPluginLoad(const char *szPlugin)
 {
+  // Check to make sure it's not already loaded
   CProtoPlugin *p = licq->LoadProtoPlugin(szPlugin);
   if (p == NULL) return false;
 
@@ -772,7 +822,7 @@ void CICQDaemon::SaveConf()
   int n = 1;
   FOR_EACH_OWNER_START(LOCK_R)
   {
-    char szOwnerId[11], szOwnerPPID[11];
+    char szOwnerId[12], szOwnerPPID[14];
     char szPPID[5];
     sprintf(szOwnerId, "Owner%d.Id", n);
     sprintf(szOwnerPPID, "Owner%d.PPID", n++);
@@ -996,13 +1046,24 @@ void CICQDaemon::SaveUserList()
     return;
   }
      
-  n = sprintf(buff, "[users]\nNumOfUsers = %d\n", gUserManager.NumUsers());
+  // Don't save the temporary "Not In List" users
+  int nNumUsers = 0;
+  FOR_EACH_USER_START(LOCK_R)
+  {
+    if (!pUser->NotInList())
+      nNumUsers++;
+  }
+  FOR_EACH_USER_END
+
+  n = sprintf(buff, "[users]\nNumOfUsers = %d\n", nNumUsers);
   nRet = write(fd, buff, n);
 
   unsigned short i = 1;
-  //TODO: Work with other protocols
   FOR_EACH_USER_START(LOCK_R)
   {
+    if (pUser->NotInList())
+      FOR_EACH_USER_CONTINUE
+
     char szPPID[5];
     unsigned long nPPID = pUser->PPID();
     szPPID[0] = (nPPID & 0xFF000000) >> 24;
@@ -1040,7 +1101,7 @@ void CICQDaemon::SetIgnore(unsigned short n, bool b)
 }
 
 bool CICQDaemon::AddUserToList(const char *szId, unsigned long nPPID,
-                               bool bNotify)
+                               bool bNotify, bool bTempUser)
 {
   // Don't add invalid uins
   if (szId == 0 || nPPID == 0) return false;
@@ -1052,17 +1113,20 @@ bool CICQDaemon::AddUserToList(const char *szId, unsigned long nPPID,
     return false;
   }
 
-  ICQUser *u = new ICQUser(szId, nPPID);
+  ICQUser *u = new ICQUser(szId, nPPID, bTempUser);
   gUserManager.AddUser(u);
   gUserManager.DropUser(u);
-  SaveUserList();
+  if (!bTempUser)
+    SaveUserList();
 
   // this notify is for local only adds
-  if (nPPID == LICQ_PPID && m_nTCPSrvSocketDesc != -1 && bNotify)
+  if (nPPID == LICQ_PPID && m_nTCPSrvSocketDesc != -1 && bNotify && !bTempUser)
     icqAddUser(szId);
-
+  else if (nPPID != LICQ_PPID && bNotify)
+    PushProtoSignal(new CAddUserSignal(szId, false), nPPID);
+  
   PushPluginSignal(new CICQSignal(SIGNAL_UPDATExLIST, LIST_ADD, szId, nPPID));
-  PushProtoSignal(new CAddUserSignal(szId, false), nPPID);
+
   return true;
 }
 
@@ -1149,13 +1213,19 @@ void CICQDaemon::RemoveUserFromList(unsigned long _nUin)
 
 void CICQDaemon::RemoveUserFromList(const char *szId, unsigned long nPPID)
 {
-  if (nPPID == LICQ_PPID && m_nTCPSrvSocketDesc != -1)
+  ICQUser *u = gUserManager.FetchUser(szId, nPPID, LOCK_R);
+  if (!u) return;
+  bool bTempUser = u->NotInList();
+  gUserManager.DropUser(u);
+  
+  if (nPPID == LICQ_PPID && m_nTCPSrvSocketDesc != -1 && !bTempUser)
     icqRemoveUser(szId);
   else if (nPPID != LICQ_PPID)
     ProtoRemoveUser(szId, nPPID);
 
   gUserManager.RemoveUser(szId, nPPID);
-  SaveUserList();
+  if (!bTempUser)
+    SaveUserList();
 
   PushPluginSignal(new CICQSignal(SIGNAL_UPDATExLIST, LIST_REMOVE, szId,
     nPPID));
@@ -1291,7 +1361,7 @@ void CICQDaemon::SendEvent_Server(CPacket *packet)
   int nResult = pthread_create(&e->thread_send, NULL, &ProcessRunningEvent_Server_tep, e);
   if (nResult != 0)
   {
-    gLog.Error("%sUnable to start server event thread (#%lu):\n%s%s.\n", L_ERRORxSTR,
+    gLog.Error("%sUnable to start server event thread (#%hu):\n%s%s.\n", L_ERRORxSTR,
        e->m_nSequence, L_BLANKxSTR, strerror(nResult));
     e->m_eResult = EVENT_ERROR;
   }
@@ -1424,7 +1494,7 @@ ICQEvent *CICQDaemon::SendExpectEvent(ICQEvent *e, void *(*fcn)(void *))
 
   if (nResult != 0)
   {
-    gLog.Error("%sUnable to start event thread (#%lu):\n%s%s.\n", L_ERRORxSTR,
+    gLog.Error("%sUnable to start event thread (#%hu):\n%s%s.\n", L_ERRORxSTR,
        e->m_nSequence, L_BLANKxSTR, strerror(nResult));
     DoneEvent(e, EVENT_ERROR);
     if (e->m_nSocketDesc == m_nTCPSrvSocketDesc)
@@ -1597,7 +1667,7 @@ ICQEvent *CICQDaemon::DoneEvent(ICQEvent *e, EventResult _eResult)
     gLog.Info("doneevents: for: %p pending: \n", e);
     for (iter = m_lxRunningEvents.begin(); iter != m_lxRunningEvents.end(); iter++)
     {
-      gLog.Info("%p Command: %d SubCommand: %d Sequence: %lu SubSequence: %d: Uin: %lu\n", *iter,
+      gLog.Info("%p Command: %d SubCommand: %d Sequence: %hu SubSequence: %d: Uin: %lu\n", *iter,
                 (*iter)->Command(), (*iter)->SubCommand(), (*iter)->Sequence(), (*iter)->SubSequence(),
                 (*iter)->Uin());
     }
@@ -1641,7 +1711,7 @@ ICQEvent *CICQDaemon::DoneEvent(ICQEvent *e, EventResult _eResult)
  *       thread during which the sending thread may continue to do stuff.
  *       The result is extra sends or time out warnings.
  *----------------------------------------------------------------------------*/
-ICQEvent *CICQDaemon::DoneEvent(int _nSD, unsigned long _nSequence, EventResult _eResult)
+ICQEvent *CICQDaemon::DoneEvent(int _nSD, unsigned short _nSequence, EventResult _eResult)
 {
   pthread_mutex_lock(&mutex_runningevents);
   ICQEvent *e = NULL;
@@ -1939,7 +2009,7 @@ void CICQDaemon::PushExtendedEvent(ICQEvent *e)
   pthread_mutex_lock(&mutex_extendedevents);
   m_lxExtendedEvents.push_back(e);
 #if 0
-  gLog.Info("%p pushing Command: %d SubCommand: %d Sequence: %lu SubSequence: %d: Uin: %lu\n", e,
+  gLog.Info("%p pushing Command: %d SubCommand: %d Sequence: %hu SubSequence: %d: Uin: %lu\n", e,
             e->Command(), e->SubCommand(), e->Sequence(), e->SubSequence(), e->Uin());
 #endif
   pthread_mutex_unlock(&mutex_extendedevents);
@@ -2177,11 +2247,140 @@ bool CICQDaemon::AddProtocolPlugins()
   return true;
 }
 
+//-----Conversation functions---------------------------------------------------
+CConversation *CICQDaemon::AddConversation(int _nSocket, unsigned long _nPPID)
+{
+  CConversation *pNew = new CConversation(_nSocket, _nPPID);
+  pthread_mutex_lock(&mutex_conversations);
+  m_lConversations.push_back(pNew);
+  pthread_mutex_unlock(&mutex_conversations);
+  
+  return pNew;
+}
+
+bool CICQDaemon::AddUserConversation(unsigned long _nCID, const char *_szId)
+{
+  bool bAdded = false;
+  ConversationList::iterator iter;
+  pthread_mutex_lock(&mutex_conversations);
+  for (iter = m_lConversations.begin(); iter != m_lConversations.end(); iter++)
+  {
+    if ((*iter)->CID() == _nCID)
+    {
+      bAdded = true;
+      (*iter)->AddUser(_szId);
+      break;
+    }
+  }
+  pthread_mutex_unlock(&mutex_conversations);
+  
+  return bAdded;
+}
+
+bool CICQDaemon::AddUserConversation(int _nSocket, const char *_szId)
+{
+  bool bAdded = false;
+  ConversationList::iterator iter;
+  pthread_mutex_lock(&mutex_conversations);
+  for (iter = m_lConversations.begin(); iter != m_lConversations.end(); iter++)
+  {
+    if ((*iter)->Socket() == _nSocket)
+    {
+      bAdded = true;
+      (*iter)->AddUser(_szId);
+      break;
+    }
+  }
+  pthread_mutex_unlock(&mutex_conversations);
+  
+  return bAdded;
+}
+
+bool CICQDaemon::RemoveUserConversation(unsigned long _nCID, const char *_szId)
+{
+  bool bRemoved = false;
+  ConversationList::iterator iter;
+  pthread_mutex_lock(&mutex_conversations);
+  for (iter = m_lConversations.begin(); iter != m_lConversations.end(); iter++)
+  {
+    if ((*iter)->CID() == _nCID)
+    {
+      bRemoved = (*iter)->RemoveUser(_szId);
+      break;
+    }
+  }
+  pthread_mutex_unlock(&mutex_conversations);
+  
+  return bRemoved;
+}
+
+bool CICQDaemon::RemoveUserConversation(int _nSocket, const char *_szId)
+{
+  bool bRemoved = false;
+  ConversationList::iterator iter;
+  pthread_mutex_lock(&mutex_conversations);
+  for (iter = m_lConversations.begin(); iter != m_lConversations.end(); iter++)
+  {
+    if ((*iter)->Socket() == _nSocket)
+    {
+      bRemoved = (*iter)->RemoveUser(_szId);
+      break;
+    }
+  }
+  pthread_mutex_unlock(&mutex_conversations);
+  
+  return bRemoved;
+}
+
+CConversation *CICQDaemon::FindConversation(int _nSocket)
+{
+  pthread_mutex_lock(&mutex_conversations);
+  ConversationList::iterator iter;
+  for (iter = m_lConversations.begin(); iter != m_lConversations.end(); iter++)
+    if ((*iter)->Socket() == _nSocket)
+      break;
+  pthread_mutex_unlock(&mutex_conversations);
+  
+  return (iter == m_lConversations.end()) ? 0 : *iter;
+}
+
+CConversation *CICQDaemon::FindConversation(unsigned long _nCID)
+{
+  pthread_mutex_lock(&mutex_conversations);
+  ConversationList::iterator iter;
+  for (iter = m_lConversations.begin(); iter != m_lConversations.end(); iter++)
+    if ((*iter)->CID() == _nCID)
+      break;
+  pthread_mutex_unlock(&mutex_conversations);
+  
+  return (iter == m_lConversations.end()) ? 0 : *iter;
+}
+
+bool CICQDaemon::RemoveConversation(unsigned long _nCID)
+{
+  bool bDeleted = false;
+  pthread_mutex_lock(&mutex_conversations);
+  ConversationList::iterator iter;
+  for (iter = m_lConversations.begin(); iter != m_lConversations.end(); iter++)
+  {
+    if ((*iter)->CID() == _nCID)
+    {
+      bDeleted = true;
+      m_lConversations.erase(iter);
+      delete *iter;  
+      break;
+    }
+  }
+  pthread_mutex_unlock(&mutex_conversations);  
+
+  return bDeleted;
+}
+
 //-----ProcessMessage-----------------------------------------------------------
 void CICQDaemon::ProcessMessage(ICQUser *u, CBuffer &packet, char *message,
                                 unsigned short nMsgType, unsigned long nMask,
                                 unsigned long nMsgID[2],
-                                unsigned long nSequence, bool bIsAck,
+                                unsigned short nSequence, bool bIsAck,
                                 bool &bNewUser)
 {
   char *szType = NULL;
@@ -2196,27 +2395,7 @@ void CICQDaemon::ProcessMessage(ICQUser *u, CBuffer &packet, char *message,
   unsigned short nOwnerStatus = o->Status();
   gUserManager.DropOwner();
 
-  unsigned short nLevel;
-  if (nMask & ICQ_TCPxMSG_URGENT)
-    nLevel = ICQ_TCPxMSG_URGENT2;
-  else if (nMask & ICQ_TCPxMSG_LIST)
-    nLevel = ICQ_TCPxMSG_LIST2;
-  else
-    nLevel = nMask;
-
-  bool bAccept = nMask & ICQ_TCPxMSG_URGENT || nMask & ICQ_TCPxMSG_LIST;
-  // Flag as sent urgent as well if we are in occ or dnd and auto-accept is on
-  if ( ((nOwnerStatus == ICQ_STATUS_OCCUPIED || u->StatusToUser() == ICQ_STATUS_OCCUPIED)
-         && u->AcceptInOccupied() ) ||
-       ((nOwnerStatus == ICQ_STATUS_DND || u->StatusToUser() == ICQ_STATUS_DND)
-         && u->AcceptInDND() ) ||
-       (u->StatusToUser() != ICQ_STATUS_OFFLINE && u->StatusToUser() != ICQ_STATUS_OCCUPIED
-         && u->StatusToUser() != ICQ_STATUS_DND) )
-  {
-    bAccept = true;
-    nLevel = ICQ_TCPxMSG_URGENT2;
-  }
-
+  unsigned short nLevel = nMask;
   unsigned long nFlags = ((nMask & ICQ_CMDxSUB_FxMULTIREC) ? E_MULTIxREC : 0)
                          | ((nMask & ICQ_TCPxMSG_URGENT) ? E_URGENT : 0);
 
@@ -2265,7 +2444,7 @@ void CICQDaemon::ProcessMessage(ICQUser *u, CBuffer &packet, char *message,
     if (!bIsAck)
     {
       CEventChat *e = new CEventChat(message, szChatClients, nPort, nSequence,
-                                     TIME_NOW, nFlags, nMsgID[0], nMsgID[1]);
+                                     TIME_NOW, nFlags, 0, nMsgID[0], nMsgID[1]);
       nEventType = ON_EVENT_CHAT;
       pEvent = e;
     }
@@ -2298,7 +2477,7 @@ void CICQDaemon::ProcessMessage(ICQUser *u, CBuffer &packet, char *message,
 
       CEventFile *e = new CEventFile(szFilename, message, nFileSize,
                                      filelist, nSequence, TIME_NOW, nFlags,
-                                     nMsgID[0], nMsgID[1]);
+                                     0, nMsgID[0], nMsgID[1]);
       nEventType = ON_EVENT_FILE;
       pEvent = e;
     }
@@ -2467,17 +2646,6 @@ void CICQDaemon::ProcessMessage(ICQUser *u, CBuffer &packet, char *message,
     // If it parsed, did it parse properly?
     if (pEvent)
     {
-      // If we are in DND or Occupied and message isn't urgent then we ignore it
-      if (!bAccept)
-      {
-        if (nOwnerStatus == ICQ_STATUS_OCCUPIED || nOwnerStatus == ICQ_STATUS_DND)
-        {
-          delete pEvent;
-          if (szType) free(szType);
-          return;
-        }
-      }
-
       if (bNewUser)
       {
         if (Ignore(IGNORE_NEWUSERS))
