@@ -1,10 +1,11 @@
 static const char LICQ_CONF[] =
 "\n"
 "[licq]\n"
-"Version = 1027\n"
+"Version = 1028\n"
 "\n"
 "[plugins]\n"
 "NumPlugins = 0\n"
+"NumProtoPlugins = 0\n"
 "\n"
 "[network]\n"
 "AllowNewUsers = 1\n"
@@ -36,12 +37,6 @@ static const char LICQ_CONF[] =
 "[onevent]\n"
 "Enable = 1\n"
 "Command = play\n"
-"Message = /usr/share/licq/sounds/icq/Message.wav\n"
-"Chat = /usr/share/licq/sounds/icq/Chat.wav\n"
-"File = /usr/share/licq/sounds/icq/File.wav\n"
-"Url = /usr/share/licq/sounds/icq/URL.wav\n"
-"OnlineNotify = /usr/share/licq/sounds/icq/Notify.wav\n"
-"SysMsg = /usr/share/licq/sounds/icq/System.wav\n"
 "\n"
 "[groups]\n"
 "NumOfGroups = 3\n"
@@ -53,4 +48,8 @@ static const char LICQ_CONF[] =
 "Group1.id = 0\n"
 "Group2.id = 0\n"
 "Group3.id = 0\n"
-"\n";
+"\n"
+"[owners]\n"
+"NumOfOwners = 0\n"
+"Owner1.Id = 0\n"
+"Owner1.PPID = Licq\n";
