@@ -8,13 +8,13 @@
 #include <pthread.h>
 #include <list>
 #include "licq_plugind.h"
-#ifdef PROTOCOL_PLUGIN
 #include "licq_protoplugind.h"
-#endif
 
 extern char **global_argv;
 extern int global_argc;
 
+class CIniFile;
+
 class CLicq
 {
 public:
@@ -26,11 +26,9 @@ public:
   CPlugin *LoadPlugin(const char *, int, char **);
   void StartPlugin(CPlugin *);
 
-#ifdef PROTOCOL_PLUGIN
   CProtoPlugin *LoadProtoPlugin(const char *);
   void StartProtoPlugin(CProtoPlugin *);
   void *FindFunction(void *, const char *);
-#endif
 
   void ShutdownPlugins();
 
@@ -39,14 +37,15 @@ public:
   void SaveLoadedPlugins();
 
 protected:
+  bool UpgradeLicq(CIniFile &);
+
   CICQDaemon *licqDaemon;
   unsigned short m_nNextId;
   PluginsList list_plugins;
   pthread_mutex_t mutex_plugins;
-#ifdef PROTOCOL_PLUGIN
   ProtoPluginsList list_protoplugins;
   pthread_mutex_t mutex_protoplugins;
-#endif
+  bool m_bDeletePID;
 
 friend class CICQDaemon;
 };
