@@ -12,6 +12,9 @@
 #include <string.h>
 #include <sys/stat.h>
 
+// Localization
+#include "gettext.h"
+
 #include "licq_sar.h"
 #include "licq_log.h"
 #include "sar.conf.h"
@@ -68,7 +71,7 @@ bool CSARManager::Load()
   unsigned short nTemp;
   char *n[] = SAR_SECTIONS;
 
-  gLog.Info("%sLoading saved auto-responses.\n", L_INITxSTR);
+  gLog.Info(tr("%sLoading saved auto-responses.\n"), L_INITxSTR);
 
   // Get data from the config file
   snprintf(filename, sizeof(filename), "%s/sar.conf", BASE_DIR);
@@ -96,7 +99,7 @@ bool CSARManager::Load()
       sprintf(szTemp1, "SAR%d.Name", j + 1);
       m_fConf.ReadStr(szTemp1, szTemp2, "");
       sprintf(szTemp1, "SAR%d.Text", j + 1);
-      m_fConf.ReadStr(szTemp1, szTemp3, "");
+      m_fConf.ReadStr(szTemp1, szTemp3, "", true, sizeof(szTemp3));
       m_lSAR[i].push_back(new CSavedAutoResponse(szTemp2, szTemp3));
     }
   }
