@@ -22,9 +22,11 @@
 extern int errno;
 #endif
 #include <string.h>
+#include <iconv.h>
 
 #include "licq_translate.h"
 #include "licq_log.h"
+#include "licq_icq.h"
 
 CTranslator gTranslator;
 
@@ -145,6 +147,27 @@ bool CTranslator::setTranslationMap(const char *_szMapFileName)
   return true;
 }
 
+//============ CheckEncoding ================//
+
+unsigned short CTranslator::CheckEncoding(const char *szCheck, int nSize)
+{
+  // We can't really differentiate between CUSTOM and UNICODE so the
+  // daemon will take care of. We just report ASCII or other (in this case
+  // UNICODE).
+  unsigned short nEncoding = CHARSET_ASCII;
+  
+  for (int i = 0; i < nSize; i++)
+  {
+    if ((unsigned char)szCheck[i] >= 0x80)
+    {
+      nEncoding = CHARSET_UNICODE;
+      break;
+    }
+  }
+  
+  return nEncoding;
+}
+
 //============ translateToClient ============//
 
 void CTranslator::ServerToClient(char *szString)
@@ -193,66 +216,201 @@ void CTranslator::ClientToServer(char &_cChar)
 
 
 //-----ToUTF8----------------------------------------------------------------
-char *CTranslator::ToUnicode(char *_sz)
+char *CTranslator::ToUnicode(char *_sz, char *_szFrom)
 {
-  if (_sz == NULL)  return NULL;
-  unsigned short nLen = strlen(_sz);
-  char *szNewStr = new char[(nLen << 1) + 1];
-  unsigned long j = 0;
-  for (unsigned long i = 0; i < nLen; i++)
+  if (_sz == NULL) return NULL;
+  unsigned short nLen = strlen(_sz) * 2;
+  char *szNewStr = new char[nLen + 1];
+  size_t nInSize, nOutSize;
+
+  char *szIn = _sz, *szOut = szNewStr;
+  iconv_t tr;
+
+  nInSize = nLen;
+  nOutSize = nLen;
+
+  // Clean up for iconv, remove any spaces
+  int nFromLen = strlen(_szFrom);
+  int j = 0;
+  char *szFrom = new char [nFromLen+1];
+  for (int i = 0; i < nFromLen; i++)
   {
-    if ((unsigned char)_sz[i] <= 0x7F) // good ole ascii
-      szNewStr[j++] = _sz[i];
-    else
-    {
-      szNewStr[j++] = ((unsigned char)_sz[i] >> 6) | 0xC0;
-      szNewStr[j++] = ((unsigned char)_sz[i] & 0x3F) | 0x80;
-    }
+    if (_szFrom[i] != ' ')
+      szFrom[j++] = _szFrom[i];
+  }
+  szFrom[j] = '\0';
+  
+  
+  tr = iconv_open("UTF-8", szFrom);
+  size_t ret = iconv(tr, &szIn, &nInSize, &szOut, &nOutSize);
+  iconv_close(tr);
+
+  if (ret == (size_t)(-1))
+  {
+    tr = iconv_open("UCS-2BE", szFrom);
+    iconv(tr, &szIn, &nInSize, &szOut, &nOutSize);
+    iconv_close(tr);
   }
 
-  szNewStr[j] = '\0';
+  *szOut = '\0';
+
+  delete [] szFrom;
+
   return szNewStr;
 }
 
 
 //-----FromUTF8--------------------------------------------------------------
-char *CTranslator::FromUnicode(char *_sz)
+char *CTranslator::FromUnicode(char *_sz, char *_szTo)
 {
-  if (_sz == NULL)  return NULL;
-  unsigned short nLen = strlen(_sz);
+  if (_sz == NULL) return NULL;
+  unsigned short nLen = strlen(_sz) * 2;
   char *szNewStr = new char[nLen + 1];
-  unsigned long j = 0;
-  for (unsigned long i = 0; i < nLen; i++)
+  size_t nInSize, nOutSize;
+
+  char *szIn = _sz, *szOut = szNewStr;
+  iconv_t tr;
+
+  nInSize = nLen;
+  nOutSize = nLen;
+
+  // Clean up for iconv, remove any spaces
+  int nToLen = strlen(_szTo);
+  int j = 0;
+  char *szTo = new char [nToLen+1];
+  for (int i = 0; i < nToLen; i++)
   {
-    if ((unsigned char)_sz[i] <= 0x7F) // good ole ascii
-      szNewStr[j++] = _sz[i];
-    else
-    {
-      unsigned short nChar;
+    if (_szTo[i] != ' ')
+      szTo[j++] = _szTo[i];
+  }
+  szTo[j] = '\0';
 
-      if ((unsigned char)_sz[i] >= 0xE0)
-      {
-        nChar = ((_sz[i] & 0x0F) << 12) | ((_sz[i+1] & 0x3F) << 6) |
-                (_sz[i+2] & 0x3F);
-        i += 2;
-      }
-      else
-      {
-        nChar = ((_sz[i] & 0x1F) << 6) | (_sz[i+1] & 0x3F);
-        i++;
-      }
 
-      if (nChar > 0xFF)
-        szNewStr[j++] = '?';
-      else
-        szNewStr[j++] = (char)nChar;
-    }
+  tr = iconv_open(szTo, "UTF-8");
+  size_t ret = iconv(tr, &szIn, &nInSize, &szOut, &nOutSize);
+  iconv_close(tr);
+
+  if (ret == (size_t)(-1))
+  {
+    tr = iconv_open(szTo, "UCS-2BE");
+    iconv(tr, &szIn, &nInSize, &szOut, &nOutSize);
+    iconv_close(tr);
   }
 
-  szNewStr[j] = '\0';
+  *szOut = '\0';
+
+  delete [] szTo;
+
   return szNewStr;
 }
 
+//-----FromUTF8--------------------------------------------------------------
+char *CTranslator::FromUTF16(char *_sz, int nMsgLen)
+{
+  if (_sz == NULL) return NULL;
+  unsigned short nLen = nMsgLen > 0 ? nMsgLen : strlen(_sz);
+  char *szNewStr = new char[nLen * 2];
+  size_t nInSize, nOutSize;
+  
+  char *szIn = _sz, *szOut = szNewStr;
+  iconv_t tr;
+  
+  nInSize = nLen;
+  nOutSize = nLen * 2;
+  
+  tr = iconv_open("", "UCS-2BE");
+  size_t ret = iconv(tr, &szIn, &nInSize, &szOut, &nOutSize);
+  iconv_close(tr);
+  
+  if (ret == (size_t)(-1))
+  {
+    gLog.Error("Error decoding a UTF-16 message.\n");
+  }
+  
+  *szOut = '\0';
+  
+  return szNewStr;
+}
+
+//-----ToUTF16-----------------------------------------------------------------
+char *CTranslator::ToUTF16(char *_sz, char *_szEncoding, size_t &nSize)
+{
+  if (_sz == NULL) return NULL;
+  unsigned short nLen = strlen(_sz) * 3;
+  char *szNewStr = new char[nLen + 1];
+  size_t nInSize, nOutSize;
+  char *szOut = szNewStr, *szIn = _sz;
+  iconv_t tr;
+  
+  nInSize = strlen(szIn);
+  nOutSize = nLen;
+  
+  tr = iconv_open("UCS-2BE", _szEncoding);
+  size_t ret = iconv(tr, &szIn, &nInSize, &szOut, &nOutSize);
+  iconv_close(tr);
+  
+  if (ret == (size_t)-1)
+    gLog.Error("Error encoding to UTF-16 from %s\n", _szEncoding);
+
+  *szOut = '\0';
+  nSize = nLen - nOutSize;
+  
+  return szNewStr;
+}
+
+bool CTranslator::utf16to8(unsigned long c, string &s)
+{
+    if (c <= 0x7F)
+    {
+        /* Leave ASCII encoded */
+        s += (char)c;
+    }
+    else if (c <= 0x07FF)
+    {
+        /* 110xxxxx 10xxxxxx */
+        s += (char)(0xC0 | (c >> 6));
+        s += (char)(0x80 | (c & 0x3F));
+    }
+    else if (c <= 0xFFFF)
+    {
+        /* 1110xxxx + 2 */
+        s += (char)(0xE0 | (c >> 12));
+        s += (char)(0x80 | ((c >> 6) & 0x3F));
+        s += (char)(0x80 | (c & 0x3F));
+    }
+    else if (c <= 0x1FFFFF)
+    {
+        /* 11110xxx + 3 */
+        s += (char)(0xF0 | (c >> 18));
+        s += (char)(0x80 | ((c >> 12) & 0x3F));
+        s += (char)(0x80 | ((c >> 6) & 0x3F));
+        s += (char)(0x80 | (c & 0x3F));
+    }
+    else if (c <= 0x3FFFFFF)
+    {
+        /* 111110xx + 4 */
+        s += (char)(0xF8 | (c >> 24));
+        s += (char)(0x80 | ((c >> 18) & 0x3F));
+        s += (char)(0x80 | ((c >> 12) & 0x3F));
+        s += (char)(0x80 | ((c >> 6) & 0x3F));
+        s += (char)(0x80 | (c & 0x3F));
+    }
+    else if (c <= 0x7FFFFFFF)
+    {
+        /* 1111110x + 5 */
+        s += (char)(0xFC | (c >> 30));
+        s += (char)(0x80 | ((c >> 24) & 0x3F));
+        s += (char)(0x80 | ((c >> 18) & 0x3F));
+        s += (char)(0x80 | ((c >> 12) & 0x3F));
+        s += (char)(0x80 | ((c >> 6) & 0x3F));
+        s += (char)(0x80 | (c & 0x3F));
+    }
+    else
+    {
+        return false;
+    }
+    return true;
+}
 
 //-----NToRN--------------------------------------------------------------------
 char *CTranslator::NToRN(const char *_szOldStr)
